@@ -16,6 +16,7 @@
 */
 #include <array>
 #include "sdf/Camera.hh"
+#include "sdf/parser.hh"
 #include "Utils.hh"
 
 using namespace sdf;
@@ -46,7 +47,7 @@ static std::array<std::string, 19> kPixelFormatNames =
 };
 
 // Private data class
-class sdf::CameraPrivate
+class sdf::Camera::Implementation
 {
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
@@ -78,6 +79,12 @@ class sdf::CameraPrivate
   /// \brief Far clip distance for depth camera.
   public: double depthFarClip{10.0};
 
+  /// \brief Segmentation type for segmentation camera.
+  public: std::string segmentationType{"semantic"};
+
+  /// \brief Boundingbox type for boundingbox camera.
+  public: std::string boundingBoxType{"2d"};
+
   /// \brief True indicates the depth camera was set.
   public: bool hasDepthCamera{false};
 
@@ -87,6 +94,12 @@ class sdf::CameraPrivate
   /// \brief True indicates the depth camera near clip distance was set.
   public: bool hasDepthNearClip{false};
 
+  /// \brief True indicates the segmentation type was set.
+  public: bool hasSegmentationType{false};
+
+  /// \brief True indicates the boundingobx type was set.
+  public: bool hasBoundingBoxType{false};
+
   /// \brief True indicates frames should be saved.
   public: bool save{false};
 
@@ -162,48 +175,19 @@ class sdf::CameraPrivate
   /// \brief lens instrinsics s.
   public: double lensIntrinsicsS{1.0};
 
+  /// \brief True if this camera has custom intrinsics values
+  public: bool hasIntrinsics = false;
+
   /// \brief Visibility mask of a camera. Defaults to 0xFFFFFFFF
   public: uint32_t visibilityMask{4294967295u};
 };
 
 /////////////////////////////////////////////////
 Camera::Camera()
-  : dataPtr(new CameraPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Camera::~Camera()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-Camera::Camera(const Camera &_camera)
-  : dataPtr(new CameraPrivate(*_camera.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Camera::Camera(Camera &&_camera) noexcept
-  : dataPtr(std::exchange(_camera.dataPtr, nullptr))
-{
-}
-
-//////////////////////////////////////////////////
-Camera &Camera::operator=(const Camera &_camera)
-{
-  return *this = Camera(_camera);
-}
-
-//////////////////////////////////////////////////
-Camera &Camera::operator=(Camera &&_camera) noexcept
-{
-  std::swap(this->dataPtr, _camera.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Camera::Load(ElementPtr _sdf)
 {
@@ -309,6 +293,16 @@ Errors Camera::Load(ElementPtr _sdf)
         "Camera sensor is missing a <clip> element."});
   }
 
+  if (_sdf->HasElement("segmentation_type"))
+  {
+    this->SetSegmentationType(_sdf->Get<std::string>("segmentation_type"));
+  }
+
+  if (_sdf->HasElement("box_type"))
+  {
+    this->SetBoundingBoxType(_sdf->Get<std::string>("box_type"));
+  }
+
   if (_sdf->HasElement("save"))
   {
     sdf::ElementPtr elem = _sdf->GetElement("save");
@@ -377,6 +371,7 @@ Errors Camera::Load(ElementPtr _sdf)
           this->dataPtr->lensIntrinsicsCy).first;
       this->dataPtr->lensIntrinsicsS = intrinsics->Get<double>("s",
           this->dataPtr->lensIntrinsicsS).first;
+      this->dataPtr->hasIntrinsics = true;
     }
   }
 
@@ -517,6 +512,56 @@ void Camera::SetFarClip(double _far)
   this->dataPtr->farClip = _far;
 }
 
+//////////////////////////////////////////////////
+const std::string &Camera::SegmentationType() const
+{
+  return this->dataPtr->segmentationType;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetSegmentationType(const std::string &_type)
+{
+  this->dataPtr->hasSegmentationType = true;
+  this->dataPtr->segmentationType = _type;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetHasSegmentationType(bool _type)
+{
+  this->dataPtr->hasSegmentationType = _type;
+}
+
+//////////////////////////////////////////////////
+bool Camera::HasSegmentationType() const
+{
+  return this->dataPtr->hasSegmentationType;
+}
+
+//////////////////////////////////////////////////
+const std::string &Camera::BoundingBoxType() const
+{
+  return this->dataPtr->boundingBoxType;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetBoundingBoxType(const std::string &_type)
+{
+  this->dataPtr->hasBoundingBoxType = true;
+  this->dataPtr->boundingBoxType = _type;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetHasBoundingBoxType(bool _type)
+{
+  this->dataPtr->hasBoundingBoxType = _type;
+}
+
+//////////////////////////////////////////////////
+bool Camera::HasBoundingBoxType() const
+{
+  return this->dataPtr->hasBoundingBoxType;
+}
+
 //////////////////////////////////////////////////
 void Camera::SetHasDepthCamera(bool _camera)
 {
@@ -678,54 +723,29 @@ const ignition::math::Vector2d &Camera::DistortionCenter() const
 }
 
 //////////////////////////////////////////////////
-void Camera::SetDistortionCenter(
-                const ignition::math::Vector2d &_center) const
+void Camera::SetDistortionCenter(const ignition::math::Vector2d &_center)
 {
   this->dataPtr->distortionCenter = _center;
 }
 
-/////////////////////////////////////////////////
-const ignition::math::Pose3d &Camera::Pose() const
-{
-  return this->RawPose();
-}
-
 /////////////////////////////////////////////////
 const ignition::math::Pose3d &Camera::RawPose() const
 {
   return this->dataPtr->pose;
 }
 
-/////////////////////////////////////////////////
-const std::string &Camera::PoseFrame() const
-{
-  return this->PoseRelativeTo();
-}
-
 /////////////////////////////////////////////////
 const std::string &Camera::PoseRelativeTo() const
 {
   return this->dataPtr->poseRelativeTo;
 }
 
-/////////////////////////////////////////////////
-void Camera::SetPose(const ignition::math::Pose3d &_pose)
-{
-  this->SetRawPose(_pose);
-}
-
 /////////////////////////////////////////////////
 void Camera::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
-/////////////////////////////////////////////////
-void Camera::SetPoseFrame(const std::string &_frame)
-{
-  this->SetPoseRelativeTo(_frame);
-}
-
 /////////////////////////////////////////////////
 void Camera::SetPoseRelativeTo(const std::string &_frame)
 {
@@ -850,6 +870,7 @@ double Camera::LensIntrinsicsFx() const
 void Camera::SetLensIntrinsicsFx(double _fx)
 {
   this->dataPtr->lensIntrinsicsFx = _fx;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -862,6 +883,7 @@ double Camera::LensIntrinsicsFy() const
 void Camera::SetLensIntrinsicsFy(double _fy)
 {
   this->dataPtr->lensIntrinsicsFy = _fy;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -874,6 +896,7 @@ double Camera::LensIntrinsicsCx() const
 void Camera::SetLensIntrinsicsCx(double _cx)
 {
   this->dataPtr->lensIntrinsicsCx = _cx;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -886,6 +909,7 @@ double Camera::LensIntrinsicsCy() const
 void Camera::SetLensIntrinsicsCy(double _cy)
 {
   this->dataPtr->lensIntrinsicsCy = _cy;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -898,6 +922,7 @@ double Camera::LensIntrinsicsSkew() const
 void Camera::SetLensIntrinsicsSkew(double _s)
 {
   this->dataPtr->lensIntrinsicsS = _s;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -926,6 +951,8 @@ PixelFormatType Camera::ConvertPixelFormat(const std::string &_format)
     return PixelFormatType::RGB_INT8;
   else if (_format == "L8")
     return PixelFormatType::L_INT8;
+  else if (_format == "L16")
+    return PixelFormatType::L_INT16;
   else if (_format == "B8G8R8")
     return PixelFormatType::BGR_INT8;
   else if (_format == "BAYER_RGGB8")
@@ -951,3 +978,116 @@ void Camera::SetVisibilityMask(uint32_t _mask)
 {
   this->dataPtr->visibilityMask = _mask;
 }
+
+/////////////////////////////////////////////////
+bool Camera::HasLensIntrinsics() const
+{
+  return this->dataPtr->hasIntrinsics;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Camera::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("camera.sdf", elem);
+
+  elem->GetAttribute("name")->Set<std::string>(this->Name());
+  sdf::ElementPtr poseElem = elem->GetElement("pose");
+  if (!this->dataPtr->poseRelativeTo.empty())
+  {
+    poseElem->GetAttribute("relative_to")->Set<std::string>(
+        this->dataPtr->poseRelativeTo);
+  }
+  poseElem->Set<ignition::math::Pose3d>(this->RawPose());
+  elem->GetElement("horizontal_fov")->Set<double>(
+      this->HorizontalFov().Radian());
+  sdf::ElementPtr imageElem = elem->GetElement("image");
+  imageElem->GetElement("width")->Set<double>(this->ImageWidth());
+  imageElem->GetElement("height")->Set<double>(this->ImageHeight());
+  imageElem->GetElement("format")->Set<std::string>(this->PixelFormatStr());
+  sdf::ElementPtr clipElem = elem->GetElement("clip");
+  clipElem->GetElement("near")->Set<double>(this->NearClip());
+  clipElem->GetElement("far")->Set<double>(this->FarClip());
+
+  if (this->dataPtr->hasDepthCamera)
+  {
+    sdf::ElementPtr depthElem = elem->GetElement("depth_camera");
+    sdf::ElementPtr depthClipElem = depthElem->GetElement("clip");
+    depthClipElem->GetElement("near")->Set<double>(this->DepthNearClip());
+    depthClipElem->GetElement("far")->Set<double>(this->DepthFarClip());
+  }
+
+  sdf::ElementPtr saveElem = elem->GetElement("save");
+  saveElem->GetAttribute("enabled")->Set<bool>(this->SaveFrames());
+  if (this->SaveFrames())
+    saveElem->GetElement("path")->Set<std::string>(this->SaveFramesPath());
+  elem->GetElement("visibility_mask")->Set<uint32_t>(
+      this->VisibilityMask());
+
+  sdf::ElementPtr noiseElem = elem->GetElement("noise");
+  std::string noiseType;
+  switch (this->dataPtr->imageNoise.Type())
+  {
+    case sdf::NoiseType::NONE:
+      noiseType = "none";
+      break;
+    case sdf::NoiseType::GAUSSIAN:
+      noiseType = "gaussian";
+      break;
+    case sdf::NoiseType::GAUSSIAN_QUANTIZED:
+      noiseType = "gaussian_quantized";
+      break;
+    default:
+      noiseType = "none";
+  }
+
+  // camera does not use noise.sdf description
+  noiseElem->GetElement("type")->Set<std::string>(noiseType);
+  noiseElem->GetElement("mean")->Set<double>(this->dataPtr->imageNoise.Mean());
+  noiseElem->GetElement("stddev")->Set<double>(
+      this->dataPtr->imageNoise.StdDev());
+
+  sdf::ElementPtr distortionElem = elem->GetElement("distortion");
+  distortionElem->GetElement("k1")->Set<double>(this->DistortionK1());
+  distortionElem->GetElement("k2")->Set<double>(this->DistortionK2());
+  distortionElem->GetElement("k3")->Set<double>(this->DistortionK3());
+  distortionElem->GetElement("p1")->Set<double>(this->DistortionP1());
+  distortionElem->GetElement("p2")->Set<double>(this->DistortionP2());
+  distortionElem->GetElement("center")->Set<ignition::math::Vector2d>(
+      this->DistortionCenter());
+
+  sdf::ElementPtr lensElem = elem->GetElement("lens");
+  lensElem->GetElement("type")->Set<std::string>(this->LensType());
+  lensElem->GetElement("scale_to_hfov")->Set<bool>(this->LensScaleToHfov());
+  lensElem->GetElement("cutoff_angle")->Set<double>(
+     this->LensCutoffAngle().Radian());
+  lensElem->GetElement("env_texture_size")->Set<double>(
+      this->LensEnvironmentTextureSize());
+  if (this->LensType() == "custom")
+  {
+    sdf::ElementPtr customLensElem = lensElem->GetElement("custom_function");
+    customLensElem->GetElement("c1")->Set<double>(this->LensC1());
+    customLensElem->GetElement("c2")->Set<double>(this->LensC2());
+    customLensElem->GetElement("c3")->Set<double>(this->LensC3());
+    customLensElem->GetElement("f")->Set<double>(this->LensFocalLength());
+    customLensElem->GetElement("fun")->Set<std::string>(
+        this->LensFunction());
+  }
+  if (this->HasLensIntrinsics())
+  {
+    sdf::ElementPtr intrinsicsElem = lensElem->GetElement("intrinsics");
+    intrinsicsElem->GetElement("fx")->Set<double>(this->LensIntrinsicsFx());
+    intrinsicsElem->GetElement("fy")->Set<double>(this->LensIntrinsicsFy());
+    intrinsicsElem->GetElement("cx")->Set<double>(this->LensIntrinsicsCx());
+    intrinsicsElem->GetElement("cy")->Set<double>(this->LensIntrinsicsCy());
+    intrinsicsElem->GetElement("s")->Set<double>(this->LensIntrinsicsSkew());
+  }
+
+  if (this->HasSegmentationType())
+  {
+    elem->GetElement("segmentation_type")->Set<std::string>(
+        this->SegmentationType());
+  }
+
+  return elem;
+}
