@@ -52,7 +52,7 @@ void Element::SetParent(const ElementPtr _parent)
 
   // If this element doesn't have a path, get it from the parent
   if (nullptr != _parent && (this->FilePath().empty() ||
-      this->FilePath() == "data-string"))
+      this->FilePath() == std::string(kSdfStringSource)))
   {
     this->SetFilePath(_parent->FilePath());
   }
@@ -141,6 +141,21 @@ void Element::AddValue(const std::string &_type,
       _type, _defaultValue, _required, _description);
 }
 
+/////////////////////////////////////////////////
+void Element::AddValue(const std::string &_type,
+                       const std::string &_defaultValue,
+                       bool _required,
+                       const std::string &_minValue,
+                       const std::string &_maxValue,
+                       const std::string &_description)
+{
+  this->dataPtr->value =
+      std::make_shared<Param>(this->dataPtr->name, _type, _defaultValue,
+                              _required, _minValue, _maxValue, _description);
+  SDF_ASSERT(this->dataPtr->value->SetParentElement(shared_from_this()),
+      "Cannot set parent Element of value to itself.");
+}
+
 /////////////////////////////////////////////////
 ParamPtr Element::CreateParam(const std::string &_key,
                               const std::string &_type,
@@ -148,8 +163,11 @@ ParamPtr Element::CreateParam(const std::string &_key,
                               bool _required,
                               const std::string &_description)
 {
-  return ParamPtr(
-      new Param(_key, _type, _defaultValue, _required, _description));
+  ParamPtr param = std::make_shared<Param>(
+      _key, _type, _defaultValue, _required, _description);
+  SDF_ASSERT(param->SetParentElement(shared_from_this()),
+      "Cannot set parent Element of created Param to itself.");
+  return param;
 }
 
 /////////////////////////////////////////////////
@@ -171,9 +189,10 @@ ElementPtr Element::Clone() const
   clone->dataPtr->name = this->dataPtr->name;
   clone->dataPtr->required = this->dataPtr->required;
   clone->dataPtr->copyChildren = this->dataPtr->copyChildren;
-  clone->dataPtr->includeFilename = this->dataPtr->includeFilename;
   clone->dataPtr->referenceSDF = this->dataPtr->referenceSDF;
   clone->dataPtr->path = this->dataPtr->path;
+  clone->dataPtr->lineNumber = this->dataPtr->lineNumber;
+  clone->dataPtr->xmlPath = this->dataPtr->xmlPath;
   clone->dataPtr->originalVersion = this->dataPtr->originalVersion;
   clone->dataPtr->explicitlySetInFile = this->dataPtr->explicitlySetInFile;
 
@@ -181,7 +200,11 @@ ElementPtr Element::Clone() const
   for (aiter = this->dataPtr->attributes.begin();
        aiter != this->dataPtr->attributes.end(); ++aiter)
   {
-    clone->dataPtr->attributes.push_back((*aiter)->Clone());
+    auto clonedAttribute = (*aiter)->Clone();
+    SDF_ASSERT(clonedAttribute->SetParentElement(clone),
+        "Cannot set parent Element of cloned attribute Param to cloned "
+        "Element.");
+    clone->dataPtr->attributes.push_back(clonedAttribute);
   }
 
   ElementPtr_V::const_iterator eiter;
@@ -201,6 +224,13 @@ ElementPtr Element::Clone() const
   if (this->dataPtr->value)
   {
     clone->dataPtr->value = this->dataPtr->value->Clone();
+    SDF_ASSERT(clone->dataPtr->value->SetParentElement(clone),
+        "Cannot set parent Element of cloned value Param to cloned Element.");
+  }
+
+  if (this->dataPtr->includeElement)
+  {
+    clone->dataPtr->includeElement = this->dataPtr->includeElement->Clone();
   }
 
   return clone;
@@ -213,10 +243,11 @@ void Element::Copy(const ElementPtr _elem)
   this->dataPtr->description = _elem->GetDescription();
   this->dataPtr->required = _elem->GetRequired();
   this->dataPtr->copyChildren = _elem->GetCopyChildren();
-  this->dataPtr->includeFilename = _elem->dataPtr->includeFilename;
   this->dataPtr->referenceSDF = _elem->ReferenceSDF();
   this->dataPtr->originalVersion = _elem->OriginalVersion();
   this->dataPtr->path = _elem->FilePath();
+  this->dataPtr->lineNumber = _elem->LineNumber();
+  this->dataPtr->xmlPath = _elem->XmlPath();
   this->dataPtr->explicitlySetInFile = _elem->GetExplicitlySetInFile();
 
   for (Param_V::iterator iter = _elem->dataPtr->attributes.begin();
@@ -228,6 +259,8 @@ void Element::Copy(const ElementPtr _elem)
     }
     ParamPtr param = this->GetAttribute((*iter)->GetKey());
     (*param) = (**iter);
+    SDF_ASSERT(param->SetParentElement(shared_from_this()),
+        "Cannot set parent Element of copied attribute Param to itself.");
   }
 
   if (_elem->GetValue())
@@ -240,6 +273,8 @@ void Element::Copy(const ElementPtr _elem)
     {
       *(this->dataPtr->value) = *(_elem->GetValue());
     }
+    SDF_ASSERT(this->dataPtr->value->SetParentElement(shared_from_this()),
+        "Cannot set parent Element of copied value Param to itself.");
   }
 
   this->dataPtr->elementDescriptions.clear();
@@ -259,6 +294,18 @@ void Element::Copy(const ElementPtr _elem)
     elem->SetParent(shared_from_this());
     this->dataPtr->elements.push_back(elem);
   }
+
+  if (_elem->dataPtr->includeElement)
+  {
+    if (!this->dataPtr->includeElement)
+    {
+      this->dataPtr->includeElement = _elem->dataPtr->includeElement->Clone();
+    }
+    else
+    {
+      this->dataPtr->includeElement->Copy(_elem->dataPtr->includeElement);
+    }
+  }
 }
 
 /////////////////////////////////////////////////
@@ -273,6 +320,17 @@ void Element::PrintDescription(const std::string &_prefix) const
               << "'"
               << " default ='" << this->dataPtr->value->GetDefaultAsString()
               << "'";
+    auto minValue = this->dataPtr->value->GetMinValueAsString();
+    if (minValue.has_value())
+    {
+      std::cout << " min ='" << *minValue << "'";
+    }
+
+    auto maxValue = this->dataPtr->value->GetMaxValueAsString();
+    if (maxValue.has_value())
+    {
+      std::cout << " max ='" << *maxValue << "'";
+    }
   }
 
   std::cout << ">\n";
@@ -444,6 +502,7 @@ void Element::PrintDocLeftPane(std::string &_html, int _spacing,
 void Element::PrintValuesImpl(const std::string &_prefix,
                               bool _includeDefaultElements,
                               bool _includeDefaultAttributes,
+                              const PrintConfig &_config,
                               std::ostringstream &_out) const
 {
   if (this->GetExplicitlySetInFile() || _includeDefaultElements)
@@ -463,7 +522,7 @@ void Element::PrintValuesImpl(const std::string &_prefix,
           _includeDefaultAttributes)
       {
         _out << " " << (*aiter)->GetKey() << "='"
-             << (*aiter)->GetAsString() << "'";
+             << (*aiter)->GetAsString(_config) << "'";
       }
     }
 
@@ -479,6 +538,7 @@ void Element::PrintValuesImpl(const std::string &_prefix,
           (*eiter)->ToString(_prefix + "  ",
                              _includeDefaultElements,
                              _includeDefaultAttributes,
+                             _config,
                              _out);
         }
       }
@@ -488,7 +548,7 @@ void Element::PrintValuesImpl(const std::string &_prefix,
     {
       if (this->dataPtr->value)
       {
-        _out << ">" << this->dataPtr->value->GetAsString()
+        _out << ">" << this->dataPtr->value->GetAsString(_config)
              << "</" << this->dataPtr->name << ">\n";
       }
       else
@@ -500,43 +560,48 @@ void Element::PrintValuesImpl(const std::string &_prefix,
 }
 
 /////////////////////////////////////////////////
-void Element::PrintValues(std::string _prefix) const
+void Element::PrintValues(std::string _prefix, const PrintConfig &_config) const
 {
   std::ostringstream ss;
-  PrintValuesImpl(_prefix, true, false, ss);
+  PrintValuesImpl(_prefix, true, false, _config, ss);
   std::cout << ss.str();
 }
 
 /////////////////////////////////////////////////
 void Element::PrintValues(const std::string &_prefix,
                           bool _includeDefaultElements,
-                          bool _includeDefaultAttributes) const
+                          bool _includeDefaultAttributes,
+                          const PrintConfig &_config) const
 {
   std::ostringstream ss;
   PrintValuesImpl(_prefix,
                   _includeDefaultElements,
                   _includeDefaultAttributes,
+                  _config,
                   ss);
   std::cout << ss.str();
 }
 
 /////////////////////////////////////////////////
-std::string Element::ToString(const std::string &_prefix) const
+std::string Element::ToString(const std::string &_prefix,
+                              const PrintConfig &_config) const
 {
   std::ostringstream out;
-  this->ToString(_prefix, true, false, out);
+  this->ToString(_prefix, true, false, _config, out);
   return out.str();
 }
 
 /////////////////////////////////////////////////
 std::string Element::ToString(const std::string &_prefix,
                               bool _includeDefaultElements,
-                              bool _includeDefaultAttributes) const
+                              bool _includeDefaultAttributes,
+                              const PrintConfig &_config) const
 {
   std::ostringstream out;
   this->ToString(_prefix,
                  _includeDefaultElements,
                  _includeDefaultAttributes,
+                 _config,
                  out);
   return out.str();
 }
@@ -545,20 +610,14 @@ std::string Element::ToString(const std::string &_prefix,
 void Element::ToString(const std::string &_prefix,
                        bool _includeDefaultElements,
                        bool _includeDefaultAttributes,
+                       const PrintConfig &_config,
                        std::ostringstream &_out) const
 {
-  if (this->dataPtr->includeFilename.empty())
-  {
-    PrintValuesImpl(_prefix,
-                    _includeDefaultElements,
-                    _includeDefaultAttributes,
-                    _out);
-  }
-  else
-  {
-    _out << _prefix << "<include filename='"
-         << this->dataPtr->includeFilename << "'/>\n";
-  }
+  PrintValuesImpl(_prefix,
+                  _includeDefaultElements,
+                  _includeDefaultAttributes,
+                  _config,
+                  _out);
 }
 
 /////////////////////////////////////////////////
@@ -622,6 +681,12 @@ size_t Element::GetAttributeCount() const
   return this->dataPtr->attributes.size();
 }
 
+/////////////////////////////////////////////////
+const Param_V &Element::GetAttributes() const
+{
+  return this->dataPtr->attributes;
+}
+
 /////////////////////////////////////////////////
 ParamPtr Element::GetAttribute(unsigned int _index) const
 {
@@ -921,6 +986,8 @@ void Element::Clear()
   this->ClearElements();
   this->dataPtr->originalVersion.clear();
   this->dataPtr->path.clear();
+  this->dataPtr->lineNumber = std::nullopt;
+  this->dataPtr->xmlPath.clear();
 }
 
 /////////////////////////////////////////////////
@@ -993,15 +1060,15 @@ void Element::AddElementDescription(ElementPtr _elem)
 }
 
 /////////////////////////////////////////////////
-void Element::SetInclude(const std::string &_filename)
+void Element::SetIncludeElement(sdf::ElementPtr _includeElem)
 {
-  this->dataPtr->includeFilename = _filename;
+  this->dataPtr->includeElement = _includeElem;
 }
 
 /////////////////////////////////////////////////
-std::string Element::GetInclude() const
+sdf::ElementPtr Element::GetIncludeElement() const
 {
-  return this->dataPtr->includeFilename;
+  return this->dataPtr->includeElement;
 }
 
 /////////////////////////////////////////////////
@@ -1016,6 +1083,30 @@ const std::string &Element::FilePath() const
   return this->dataPtr->path;
 }
 
+/////////////////////////////////////////////////
+void Element::SetLineNumber(int _lineNumber)
+{
+  this->dataPtr->lineNumber = _lineNumber;
+}
+
+/////////////////////////////////////////////////
+std::optional<int> Element::LineNumber() const
+{
+  return this->dataPtr->lineNumber;
+}
+
+/////////////////////////////////////////////////
+void Element::SetXmlPath(const std::string &_path)
+{
+  this->dataPtr->xmlPath = _path;
+}
+
+/////////////////////////////////////////////////
+const std::string &Element::XmlPath() const
+{
+  return this->dataPtr->xmlPath;
+}
+
 /////////////////////////////////////////////////
 void Element::SetOriginalVersion(const std::string &_version)
 {
