@@ -191,40 +191,42 @@ TEST(Element, ReferenceSDF)
 /////////////////////////////////////////////////
 TEST(Element, AddValue)
 {
-  sdf::Element elem;
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
 
-  elem.SetName("test");
-  elem.AddValue("string", "foo", false, "foo description");
+  elem->SetName("test");
+  elem->AddValue("string", "foo", false, "foo description");
 
-  sdf::ParamPtr param = elem.GetValue();
+  sdf::ParamPtr param = elem->GetValue();
   ASSERT_TRUE(param->IsType<std::string>());
   ASSERT_EQ(param->GetKey(), "test");
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "foo");
   ASSERT_EQ(param->GetDescription(), "foo description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), elem);
 }
 
 /////////////////////////////////////////////////
 TEST(Element, AddAttribute)
 {
-  sdf::Element elem;
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
 
-  ASSERT_EQ(elem.GetAttributeCount(), 0UL);
+  ASSERT_EQ(elem->GetAttributeCount(), 0UL);
 
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  ASSERT_EQ(elem.GetAttributeCount(), 1UL);
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  ASSERT_EQ(elem->GetAttributeCount(), 1UL);
 
-  elem.AddAttribute("attr", "float", "0.0", false, "float description");
-  ASSERT_EQ(elem.GetAttributeCount(), 2UL);
+  elem->AddAttribute("attr", "float", "0.0", false, "float description");
+  ASSERT_EQ(elem->GetAttributeCount(), 2UL);
 
-  sdf::ParamPtr param = elem.GetAttribute("test");
+  sdf::ParamPtr param = elem->GetAttribute("test");
   ASSERT_TRUE(param->IsType<std::string>());
   ASSERT_EQ(param->GetKey(), "test");
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "foo");
   ASSERT_EQ(param->GetDescription(), "foo description");
 
-  param = elem.GetAttribute("attr");
+  param = elem->GetAttribute("attr");
   ASSERT_TRUE(param->IsType<float>());
   ASSERT_EQ(param->GetKey(), "attr");
   ASSERT_EQ(param->GetTypeName(), "float");
@@ -235,48 +237,48 @@ TEST(Element, AddAttribute)
 /////////////////////////////////////////////////
 TEST(Element, GetAttributeSet)
 {
-  sdf::Element elem;
-  ASSERT_EQ(elem.GetAttributeCount(), 0UL);
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  ASSERT_EQ(elem.GetAttributeCount(), 1UL);
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
+  ASSERT_EQ(elem->GetAttributeCount(), 0UL);
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  ASSERT_EQ(elem->GetAttributeCount(), 1UL);
 
-  EXPECT_FALSE(elem.GetAttributeSet("test"));
-  elem.GetAttribute("test")->Set("asdf");
-  EXPECT_TRUE(elem.GetAttributeSet("test"));
+  EXPECT_FALSE(elem->GetAttributeSet("test"));
+  elem->GetAttribute("test")->Set("asdf");
+  EXPECT_TRUE(elem->GetAttributeSet("test"));
 }
 
 /////////////////////////////////////////////////
 TEST(Element, RemoveAttribute)
 {
-  sdf::Element elem;
-  ASSERT_EQ(elem.GetAttributeCount(), 0UL);
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
+  ASSERT_EQ(elem->GetAttributeCount(), 0UL);
 
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  elem.AddAttribute("attr", "float", "0.0", false, "float description");
-  ASSERT_EQ(elem.GetAttributeCount(), 2UL);
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  elem->AddAttribute("attr", "float", "0.0", false, "float description");
+  ASSERT_EQ(elem->GetAttributeCount(), 2UL);
 
-  elem.RemoveAttribute("test");
-  EXPECT_EQ(elem.GetAttributeCount(), 1UL);
-  EXPECT_EQ(elem.GetAttribute("test"), nullptr);
-  EXPECT_NE(elem.GetAttribute("attr"), nullptr);
+  elem->RemoveAttribute("test");
+  EXPECT_EQ(elem->GetAttributeCount(), 1UL);
+  EXPECT_EQ(elem->GetAttribute("test"), nullptr);
+  EXPECT_NE(elem->GetAttribute("attr"), nullptr);
 }
 
 /////////////////////////////////////////////////
 TEST(Element, RemoveAllAttributes)
 {
-  sdf::Element elem;
-  ASSERT_EQ(elem.GetAttributeCount(), 0UL);
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
+  ASSERT_EQ(elem->GetAttributeCount(), 0UL);
 
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  elem.AddAttribute("test2", "string", "foo", false, "foo description");
-  elem.AddAttribute("attr", "float", "0.0", false, "float description");
-  ASSERT_EQ(elem.GetAttributeCount(), 3UL);
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  elem->AddAttribute("test2", "string", "foo", false, "foo description");
+  elem->AddAttribute("attr", "float", "0.0", false, "float description");
+  ASSERT_EQ(elem->GetAttributeCount(), 3UL);
 
-  elem.RemoveAllAttributes();
-  EXPECT_EQ(elem.GetAttributeCount(), 0UL);
-  EXPECT_EQ(elem.GetAttribute("test"), nullptr);
-  EXPECT_EQ(elem.GetAttribute("test2"), nullptr);
-  EXPECT_EQ(elem.GetAttribute("attr"), nullptr);
+  elem->RemoveAllAttributes();
+  EXPECT_EQ(elem->GetAttributeCount(), 0UL);
+  EXPECT_EQ(elem->GetAttribute("test"), nullptr);
+  EXPECT_EQ(elem->GetAttribute("test2"), nullptr);
+  EXPECT_EQ(elem->GetAttribute("attr"), nullptr);
 }
 
 /////////////////////////////////////////////////
@@ -284,11 +286,20 @@ TEST(Element, Include)
 {
   sdf::Element elem;
 
-  ASSERT_EQ(elem.GetInclude(), "");
+  auto includeElemToStore = std::make_shared<sdf::Element>();
+  includeElemToStore->SetName("include");
+  auto uriDesc = std::make_shared<sdf::Element>();
+  uriDesc->SetName("uri");
+  uriDesc->AddValue("string", "", true);
+  includeElemToStore->AddElementDescription(uriDesc);
 
-  elem.SetInclude("foo.txt");
+  includeElemToStore->GetElement("uri")->Set("foo.txt");
+  elem.SetIncludeElement(includeElemToStore);
 
-  ASSERT_EQ(elem.GetInclude(), "foo.txt");
+  auto includeElem = elem.GetIncludeElement();
+  ASSERT_NE(nullptr, includeElem);
+  ASSERT_TRUE(includeElem->HasElement("uri"));
+  EXPECT_EQ("foo.txt", includeElem->GetElement("uri")->Get<std::string>());
 }
 
 /////////////////////////////////////////////////
@@ -388,6 +399,9 @@ TEST(Element, Clone)
   sdf::ElementPtr child = std::make_shared<sdf::Element>();
   sdf::ElementPtr desc = std::make_shared<sdf::Element>();
 
+  parent->SetName("parent");
+  child->SetName("child");
+
   parent->InsertElement(child);
   ASSERT_NE(parent->GetFirstElement(), nullptr);
 
@@ -400,16 +414,36 @@ TEST(Element, Clone)
   parent->AddValue("string", "foo", false, "foo description");
 
   parent->SetFilePath("/path/to/file.sdf");
+  parent->SetLineNumber(12);
+  parent->SetXmlPath("/sdf/world[@name=\"default\"]");
   parent->SetOriginalVersion("1.5");
 
+  auto includeElemToStore = std::make_shared<sdf::Element>();
+  includeElemToStore->SetName("include");
+  parent->SetIncludeElement(includeElemToStore);
+
   sdf::ElementPtr newelem = parent->Clone();
 
   EXPECT_EQ("/path/to/file.sdf", newelem->FilePath());
+  ASSERT_TRUE(newelem->LineNumber().has_value());
+  EXPECT_EQ(12, newelem->LineNumber().value());
+  EXPECT_EQ("/sdf/world[@name=\"default\"]", newelem->XmlPath());
   EXPECT_EQ("1.5", newelem->OriginalVersion());
   ASSERT_NE(newelem->GetFirstElement(), nullptr);
   ASSERT_EQ(newelem->GetElementDescriptionCount(), 1UL);
   ASSERT_EQ(newelem->GetAttributeCount(), 1UL);
+  ASSERT_NE(newelem->GetIncludeElement(), nullptr);
+  EXPECT_EQ("include", newelem->GetIncludeElement()->GetName());
   ASSERT_TRUE(newelem->GetExplicitlySetInFile());
+
+  ASSERT_NE(nullptr, parent->GetValue()->GetParentElement());
+  EXPECT_EQ(parent, parent->GetValue()->GetParentElement());
+
+  ASSERT_NE(nullptr, newelem->GetValue()->GetParentElement());
+  EXPECT_EQ(newelem, newelem->GetValue()->GetParentElement());
+
+  auto clonedAttribs = newelem->GetAttributes();
+  EXPECT_EQ(newelem, clonedAttribs[0]->GetParentElement());
 }
 
 /////////////////////////////////////////////////
@@ -419,11 +453,16 @@ TEST(Element, ClearElements)
   sdf::ElementPtr child = std::make_shared<sdf::Element>();
 
   parent->SetFilePath("/path/to/file.sdf");
+  parent->SetLineNumber(12);
+  parent->SetXmlPath("/sdf/world[@name=\"default\"]");
   parent->SetOriginalVersion("1.5");
   child->SetParent(parent);
   parent->InsertElement(child);
 
   EXPECT_EQ("/path/to/file.sdf", parent->FilePath());
+  ASSERT_TRUE(parent->LineNumber().has_value());
+  EXPECT_EQ(12, parent->LineNumber().value());
+  EXPECT_EQ("/sdf/world[@name=\"default\"]", parent->XmlPath());
   EXPECT_EQ("1.5", parent->OriginalVersion());
   ASSERT_NE(parent->GetFirstElement(), nullptr);
   EXPECT_EQ("/path/to/file.sdf", parent->GetFirstElement()->FilePath());
@@ -433,6 +472,8 @@ TEST(Element, ClearElements)
 
   ASSERT_EQ(parent->GetFirstElement(), nullptr);
   EXPECT_EQ("/path/to/file.sdf", parent->FilePath());
+  ASSERT_TRUE(parent->LineNumber().has_value());
+  EXPECT_EQ(12, parent->LineNumber().value());
   EXPECT_EQ("1.5", parent->OriginalVersion());
 }
 
@@ -443,11 +484,16 @@ TEST(Element, Clear)
   sdf::ElementPtr child = std::make_shared<sdf::Element>();
 
   parent->SetFilePath("/path/to/file.sdf");
+  parent->SetLineNumber(12);
+  parent->SetXmlPath("/sdf/world[@name=\"default\"]");
   parent->SetOriginalVersion("1.5");
   child->SetParent(parent);
   parent->InsertElement(child);
 
   EXPECT_EQ("/path/to/file.sdf", parent->FilePath());
+  ASSERT_TRUE(parent->LineNumber().has_value());
+  EXPECT_EQ(12, parent->LineNumber().value());
+  EXPECT_EQ("/sdf/world[@name=\"default\"]", parent->XmlPath());
   EXPECT_EQ("1.5", parent->OriginalVersion());
   ASSERT_NE(parent->GetFirstElement(), nullptr);
   EXPECT_EQ("/path/to/file.sdf", parent->GetFirstElement()->FilePath());
@@ -457,6 +503,8 @@ TEST(Element, Clear)
 
   ASSERT_EQ(parent->GetFirstElement(), nullptr);
   EXPECT_TRUE(parent->FilePath().empty());
+  EXPECT_FALSE(parent->LineNumber().has_value());
+  EXPECT_TRUE(parent->XmlPath().empty());
   EXPECT_TRUE(parent->OriginalVersion().empty());
 }
 
@@ -559,21 +607,6 @@ TEST(Element, ToStringClonedElement)
   EXPECT_EQ(parent->ToString("myprefix"), parentClone->ToString("myprefix"));
 }
 
-/////////////////////////////////////////////////
-TEST(Element, ToStringInclude)
-{
-  sdf::Element elem;
-
-  ASSERT_EQ(elem.GetInclude(), "");
-
-  elem.SetInclude("foo.txt");
-
-  ASSERT_EQ(elem.GetInclude(), "foo.txt");
-
-  std::string stringval = elem.ToString("myprefix");
-  ASSERT_EQ(stringval, "myprefix<include filename='foo.txt'/>\n");
-}
-
 /////////////////////////////////////////////////
 TEST(Element, ToStringDefaultElements)
 {
@@ -664,16 +697,16 @@ TEST(Element, DocLeftPane)
 /////////////////////////////////////////////////
 TEST(Element, DocRightPane)
 {
-  sdf::Element elem;
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
 
-  elem.SetDescription("Element description");
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  elem.AddValue("string", "val", false, "val description");
-  elem.AddElementDescription(std::make_shared<sdf::Element>());
+  elem->SetDescription("Element description");
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  elem->AddValue("string", "val", false, "val description");
+  elem->AddElementDescription(std::make_shared<sdf::Element>());
 
   std::string html;
   int index = 1;
-  elem.PrintDocRightPane(html, 0, index);
+  elem->PrintDocRightPane(html, 0, index);
   ASSERT_EQ(html,
             "<a name=\"1\">&lt&gt</a><div style='padding-left:0px;'>\n"
             "<div style='background-color: #ffffff'>\n"
@@ -722,11 +755,11 @@ TEST(Element, SetEmpty)
 /////////////////////////////////////////////////
 TEST(Element, Set)
 {
-  sdf::Element elem;
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
 
-  elem.AddValue("string", "val", false, "val description");
+  elem->AddValue("string", "val", false, "val description");
 
-  ASSERT_TRUE(elem.Set<std::string>("hello"));
+  ASSERT_TRUE(elem->Set<std::string>("hello"));
 }
 
 /////////////////////////////////////////////////
@@ -737,14 +770,23 @@ TEST(Element, Copy)
 
   src->SetName("test");
   src->SetFilePath("/path/to/file.sdf");
+  src->SetLineNumber(12);
+  src->SetXmlPath("/sdf/world[@name=\"default\"]");
   src->SetOriginalVersion("1.5");
   src->AddValue("string", "val", false, "val description");
   src->AddAttribute("test", "string", "foo", false, "foo description");
   src->InsertElement(std::make_shared<sdf::Element>());
 
+  auto includeElemToStore = std::make_shared<sdf::Element>();
+  includeElemToStore->SetName("include");
+  src->SetIncludeElement(includeElemToStore);
+
   dest->Copy(src);
 
   EXPECT_EQ("/path/to/file.sdf", dest->FilePath());
+  ASSERT_TRUE(dest->LineNumber().has_value());
+  EXPECT_EQ(12, dest->LineNumber().value());
+  EXPECT_EQ("/sdf/world[@name=\"default\"]", dest->XmlPath());
   EXPECT_EQ("1.5", dest->OriginalVersion());
 
   sdf::ParamPtr param = dest->GetValue();
@@ -753,6 +795,8 @@ TEST(Element, Copy)
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "val");
   ASSERT_EQ(param->GetDescription(), "val description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), dest);
 
   ASSERT_EQ(dest->GetAttributeCount(), 1UL);
   ASSERT_TRUE(dest->GetExplicitlySetInFile());
@@ -762,8 +806,12 @@ TEST(Element, Copy)
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "foo");
   ASSERT_EQ(param->GetDescription(), "foo description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), dest);
 
   ASSERT_NE(dest->GetFirstElement(), nullptr);
+  ASSERT_NE(dest->GetIncludeElement(), nullptr);
+  EXPECT_EQ("include", dest->GetIncludeElement()->GetName());
 }
 
 /////////////////////////////////////////////////
@@ -785,6 +833,8 @@ TEST(Element, CopyDestValue)
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "val");
   ASSERT_EQ(param->GetDescription(), "val description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), dest);
 
   ASSERT_EQ(dest->GetAttributeCount(), 1UL);
   param = dest->GetAttribute("test");
@@ -793,6 +843,8 @@ TEST(Element, CopyDestValue)
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "foo");
   ASSERT_EQ(param->GetDescription(), "foo description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), dest);
 
   ASSERT_NE(dest->GetFirstElement(), nullptr);
 }
