@@ -64,6 +64,12 @@ TEST(DOMJoint, Construction)
   joint.SetChildLinkName("child");
   EXPECT_EQ("child", joint.ChildLinkName());
 
+  std::string body;
+  EXPECT_FALSE(joint.ResolveChildLink(body).empty());
+  EXPECT_TRUE(body.empty());
+  EXPECT_FALSE(joint.ResolveParentLink(body).empty());
+  EXPECT_TRUE(body.empty());
+
   joint.SetType(sdf::JointType::BALL);
   EXPECT_EQ(sdf::JointType::BALL, joint.Type());
   joint.SetType(sdf::JointType::CONTINUOUS);
@@ -84,10 +90,10 @@ TEST(DOMJoint, Construction)
   EXPECT_EQ(nullptr, joint.Axis(0));
   EXPECT_EQ(nullptr, joint.Axis(1));
   sdf::JointAxis axis;
-  axis.SetXyz(ignition::math::Vector3d(1, 0, 0));
+  EXPECT_TRUE(axis.SetXyz(ignition::math::Vector3d(1, 0, 0)).empty());
   joint.SetAxis(0, axis);
   sdf::JointAxis axis1;
-  axis1.SetXyz(ignition::math::Vector3d(0, 1, 0));
+  EXPECT_TRUE(axis1.SetXyz(ignition::math::Vector3d(0, 1, 0)).empty());
   joint.SetAxis(1, axis1);
   ASSERT_TRUE(nullptr != joint.Axis(0));
   ASSERT_TRUE(nullptr != joint.Axis(1));
@@ -113,10 +119,10 @@ TEST(DOMJoint, MoveConstructor)
   sdf::Joint joint;
   joint.SetName("test_joint");
   sdf::JointAxis axis;
-  axis.SetXyz(ignition::math::Vector3d(1, 0, 0));
+  EXPECT_TRUE(axis.SetXyz(ignition::math::Vector3d(1, 0, 0)).empty());
   joint.SetAxis(0, axis);
   sdf::JointAxis axis1;
-  axis1.SetXyz(ignition::math::Vector3d(0, 1, 0));
+  EXPECT_TRUE(axis1.SetXyz(ignition::math::Vector3d(0, 1, 0)).empty());
   joint.SetAxis(1, axis1);
 
   sdf::Joint joint2(std::move(joint));
@@ -134,10 +140,10 @@ TEST(DOMJoint, CopyConstructor)
   sdf::Joint joint;
   joint.SetName("test_joint");
   sdf::JointAxis axis;
-  axis.SetXyz(ignition::math::Vector3d(1, 0, 0));
+  EXPECT_TRUE(axis.SetXyz(ignition::math::Vector3d(1, 0, 0)).empty());
   joint.SetAxis(0, axis);
   sdf::JointAxis axis1;
-  axis1.SetXyz(ignition::math::Vector3d(0, 1, 0));
+  EXPECT_TRUE(axis1.SetXyz(ignition::math::Vector3d(0, 1, 0)).empty());
   joint.SetAxis(1, axis1);
 
   sdf::Joint joint2(joint);
@@ -161,10 +167,10 @@ TEST(DOMJoint, MoveAssignment)
   sdf::Joint joint;
   joint.SetName("test_joint");
   sdf::JointAxis axis;
-  axis.SetXyz(ignition::math::Vector3d(1, 0, 0));
+  EXPECT_TRUE(axis.SetXyz(ignition::math::Vector3d(1, 0, 0)).empty());
   joint.SetAxis(0, axis);
   sdf::JointAxis axis1;
-  axis1.SetXyz(ignition::math::Vector3d(0, 1, 0));
+  EXPECT_TRUE(axis1.SetXyz(ignition::math::Vector3d(0, 1, 0)).empty());
   joint.SetAxis(1, axis1);
 
   sdf::Joint joint2;
@@ -183,10 +189,10 @@ TEST(DOMJoint, CopyAssignment)
   sdf::Joint joint;
   joint.SetName("test_joint");
   sdf::JointAxis axis;
-  axis.SetXyz(ignition::math::Vector3d(1, 0, 0));
+  EXPECT_TRUE(axis.SetXyz(ignition::math::Vector3d(1, 0, 0)).empty());
   joint.SetAxis(0, axis);
   sdf::JointAxis axis1;
-  axis1.SetXyz(ignition::math::Vector3d(0, 1, 0));
+  EXPECT_TRUE(axis1.SetXyz(ignition::math::Vector3d(0, 1, 0)).empty());
   joint.SetAxis(1, axis1);
 
   sdf::Joint joint2;
@@ -211,19 +217,19 @@ TEST(DOMJoint, CopyAssignmentAfterMove)
   sdf::Joint joint1;
   joint1.SetName("test_joint1");
   sdf::JointAxis joint1Axis;
-  joint1Axis.SetXyz(ignition::math::Vector3d(1, 0, 0));
+  EXPECT_TRUE(joint1Axis.SetXyz(ignition::math::Vector3d(1, 0, 0)).empty());
   joint1.SetAxis(0, joint1Axis);
   sdf::JointAxis joint1Axis1;
-  joint1Axis1.SetXyz(ignition::math::Vector3d(0, 1, 0));
+  EXPECT_TRUE(joint1Axis1.SetXyz(ignition::math::Vector3d(0, 1, 0)).empty());
   joint1.SetAxis(1, joint1Axis1);
 
   sdf::Joint joint2;
   joint2.SetName("test_joint2");
   sdf::JointAxis joint2Axis;
-  joint2Axis.SetXyz(ignition::math::Vector3d(0, 0, 1));
+  EXPECT_TRUE(joint2Axis.SetXyz(ignition::math::Vector3d(0, 0, 1)).empty());
   joint2.SetAxis(0, joint2Axis);
   sdf::JointAxis joint2Axis1;
-  joint2Axis1.SetXyz(ignition::math::Vector3d(-1, 0, 0));
+  EXPECT_TRUE(joint2Axis1.SetXyz(ignition::math::Vector3d(-1, 0, 0)).empty());
   joint2.SetAxis(1, joint2Axis1);
 
   // This is similar to what std::swap does except it uses std::move for each
@@ -244,3 +250,50 @@ TEST(DOMJoint, CopyAssignmentAfterMove)
   EXPECT_EQ(joint1Axis.Xyz(), joint2.Axis(0)->Xyz());
   EXPECT_EQ(joint1Axis1.Xyz(), joint2.Axis(1)->Xyz());
 }
+
+/////////////////////////////////////////////////
+TEST(DOMJoint, ToElement)
+{
+  // test calling ToElement on a DOM object constructed without calling Load
+  sdf::Joint joint;
+  joint.SetRawPose({-1, -2, -3, 0, IGN_PI, 0});
+  joint.SetPoseRelativeTo("link");
+  joint.SetName("test_joint");
+  joint.SetParentLinkName("parent");
+  joint.SetChildLinkName("child");
+  joint.SetType(sdf::JointType::BALL);
+  sdf::JointAxis axis;
+  EXPECT_TRUE(axis.SetXyz(ignition::math::Vector3d(1, 0, 0)).empty());
+  joint.SetAxis(0, axis);
+  sdf::JointAxis axis1;
+  EXPECT_TRUE(axis1.SetXyz(ignition::math::Vector3d(0, 1, 0)).empty());
+  joint.SetAxis(1, axis1);
+
+  sdf::ElementPtr jointElem = joint.ToElement();
+  EXPECT_NE(nullptr, jointElem);
+  EXPECT_EQ(nullptr, joint.Element());
+
+  // verify values after loading the element back
+  sdf::Joint joint2;
+  joint2.Load(jointElem);
+
+  EXPECT_EQ(ignition::math::Pose3d(-1, -2, -3, 0, IGN_PI, 0),
+            joint2.RawPose());
+  EXPECT_EQ("link", joint2.PoseRelativeTo());
+  EXPECT_EQ("test_joint", joint2.Name());
+  EXPECT_EQ("parent", joint2.ParentLinkName());
+  EXPECT_EQ("child", joint2.ChildLinkName());
+  EXPECT_EQ(sdf::JointType::BALL, joint2.Type());
+  ASSERT_TRUE(nullptr != joint2.Axis(0));
+  ASSERT_TRUE(nullptr != joint2.Axis(1));
+  EXPECT_EQ(axis.Xyz(), joint2.Axis(0)->Xyz());
+  EXPECT_EQ(axis1.Xyz(), joint2.Axis(1)->Xyz());
+
+  // make changes to DOM and verify ToElement produces updated values
+  joint2.SetParentLinkName("new_parent");
+  sdf::ElementPtr joint2Elem = joint2.ToElement();
+  EXPECT_NE(nullptr, joint2Elem);
+  sdf::Joint joint3;
+  joint3.Load(joint2Elem);
+  EXPECT_EQ("new_parent", joint3.ParentLinkName());
+}
