@@ -18,12 +18,15 @@
 #include <ignition/math/Pose3.hh>
 #include "sdf/Error.hh"
 #include "sdf/Light.hh"
+#include "sdf/parser.hh"
+#include "FrameSemantics.hh"
+#include "ScopedGraph.hh"
 #include "Utils.hh"
 
 using namespace sdf;
 
 /// \brief Light private data.
-class sdf::LightPrivate
+class sdf::Light::Implementation
 {
   /// \brief Name of the light.
   public: std::string name = "";
@@ -43,12 +46,15 @@ class sdf::LightPrivate
   /// \brief Name of xml parent object.
   public: std::string xmlParentName;
 
-  /// \brief Weak pointer to model's Pose Relative-To Graph.
-  public: std::weak_ptr<const sdf::PoseRelativeToGraph> poseRelativeToGraph;
+  /// \brief Scoped Pose Relative-To graph at the parent model or world scope.
+  public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseRelativeToGraph;
 
   /// \brief True if the light should cast shadows.
   public: bool castShadows = false;
 
+  /// \brief Light intensity
+  public: double intensity = 1.0;
+
   /// \brief The attenation range.
   public: double attenuationRange = 10.0;
 
@@ -82,64 +88,10 @@ class sdf::LightPrivate
 
 /////////////////////////////////////////////////
 Light::Light()
-  : dataPtr(new LightPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Light::~Light()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-//////////////////////////////////////////////////
-Light::Light(const Light &_light)
-  : dataPtr(new LightPrivate)
-{
-  this->CopyFrom(_light);
-}
-
-/////////////////////////////////////////////////
-Light::Light(Light &&_light) noexcept
-  : dataPtr(std::exchange(_light.dataPtr, nullptr))
-{
-}
-
-//////////////////////////////////////////////////
-Light &Light::operator=(const Light &_light)
-{
-  return *this = Light(_light);
-}
-
-//////////////////////////////////////////////////
-Light &Light::operator=(Light &&_light)
-{
-  std::swap(this->dataPtr, _light.dataPtr);
-  return *this;
-}
-
-//////////////////////////////////////////////////
-void Light::CopyFrom(const Light &_light)
-{
-  this->dataPtr->name = _light.dataPtr->name;
-  this->dataPtr->pose = _light.dataPtr->pose;
-  this->dataPtr->poseRelativeTo = _light.dataPtr->poseRelativeTo;
-  this->dataPtr->type = _light.dataPtr->type;
-  this->dataPtr->sdf = _light.dataPtr->sdf;
-  this->dataPtr->castShadows = _light.dataPtr->castShadows;
-  this->dataPtr->attenuationRange = _light.dataPtr->attenuationRange;
-  this->dataPtr->linearAttenuation = _light.dataPtr->linearAttenuation;
-  this->dataPtr->constantAttenuation = _light.dataPtr->constantAttenuation;
-  this->dataPtr->quadraticAttenuation = _light.dataPtr->quadraticAttenuation;
-  this->dataPtr->direction = _light.dataPtr->direction;
-  this->dataPtr->diffuse = _light.dataPtr->diffuse;
-  this->dataPtr->specular = _light.dataPtr->specular;
-  this->dataPtr->spotInnerAngle = _light.dataPtr->spotInnerAngle;
-  this->dataPtr->spotOuterAngle = _light.dataPtr->spotOuterAngle;
-  this->dataPtr->spotFalloff = _light.dataPtr->spotFalloff;
-}
-
 /////////////////////////////////////////////////
 Errors Light::Load(ElementPtr _sdf)
 {
@@ -193,6 +145,9 @@ Errors Light::Load(ElementPtr _sdf)
   this->dataPtr->castShadows = _sdf->Get<bool>("cast_shadows",
       this->dataPtr->castShadows).first;
 
+  this->dataPtr->intensity = _sdf->Get<double>("intensity",
+      this->dataPtr->intensity).first;
+
   this->dataPtr->diffuse = _sdf->Get<ignition::math::Color>("diffuse",
       this->dataPtr->diffuse).first;
 
@@ -280,53 +235,29 @@ std::string Light::Name() const
 }
 
 /////////////////////////////////////////////////
-void Light::SetName(const std::string &_name) const
+void Light::SetName(const std::string &_name)
 {
   this->dataPtr->name = _name;
 }
 
-/////////////////////////////////////////////////
-const ignition::math::Pose3d &Light::Pose() const
-{
-  return this->RawPose();
-}
-
 /////////////////////////////////////////////////
 const ignition::math::Pose3d &Light::RawPose() const
 {
   return this->dataPtr->pose;
 }
 
-/////////////////////////////////////////////////
-const std::string &Light::PoseFrame() const
-{
-  return this->PoseRelativeTo();
-}
-
 /////////////////////////////////////////////////
 const std::string &Light::PoseRelativeTo() const
 {
   return this->dataPtr->poseRelativeTo;
 }
 
-/////////////////////////////////////////////////
-void Light::SetPose(const ignition::math::Pose3d &_pose)
-{
-  this->SetRawPose(_pose);
-}
-
 /////////////////////////////////////////////////
 void Light::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
-/////////////////////////////////////////////////
-void Light::SetPoseFrame(const std::string &_frame)
-{
-  this->SetPoseRelativeTo(_frame);
-}
-
 /////////////////////////////////////////////////
 void Light::SetPoseRelativeTo(const std::string &_frame)
 {
@@ -341,7 +272,7 @@ void Light::SetXmlParentName(const std::string &_xmlParentName)
 
 /////////////////////////////////////////////////
 void Light::SetPoseRelativeToGraph(
-    std::weak_ptr<const PoseRelativeToGraph> _graph)
+    sdf::ScopedGraph<PoseRelativeToGraph> _graph)
 {
   this->dataPtr->poseRelativeToGraph = _graph;
 }
@@ -362,6 +293,18 @@ sdf::ElementPtr Light::Element() const
   return this->dataPtr->sdf;
 }
 
+/////////////////////////////////////////////////
+double Light::Intensity() const
+{
+  return this->dataPtr->intensity;
+}
+
+/////////////////////////////////////////////////
+void Light::SetIntensity(const double _intensity)
+{
+  this->dataPtr->intensity = _intensity;
+}
+
 /////////////////////////////////////////////////
 bool Light::CastShadows() const
 {
@@ -381,7 +324,7 @@ ignition::math::Color Light::Diffuse() const
 }
 
 /////////////////////////////////////////////////
-void Light::SetDiffuse(const ignition::math::Color &_color) const
+void Light::SetDiffuse(const ignition::math::Color &_color)
 {
   this->dataPtr->diffuse = _color;
 }
@@ -393,7 +336,7 @@ ignition::math::Color Light::Specular() const
 }
 
 /////////////////////////////////////////////////
-void Light::SetSpecular(const ignition::math::Color &_color) const
+void Light::SetSpecular(const ignition::math::Color &_color)
 {
   this->dataPtr->specular = _color;
 }
@@ -506,3 +449,59 @@ void Light::SetType(const LightType _type)
 {
   this->dataPtr->type = _type;
 }
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Light::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("light.sdf", elem);
+
+  std::string lightTypeStr = "point";
+  switch (this->Type())
+  {
+    case LightType::POINT:
+      lightTypeStr = "point";
+      break;
+    case LightType::DIRECTIONAL:
+      lightTypeStr = "directional";
+      break;
+    case LightType::SPOT:
+      lightTypeStr = "spot";
+      break;
+    default:
+      break;
+  }
+  elem->GetAttribute("type")->Set<std::string>(lightTypeStr);
+  elem->GetAttribute("name")->Set<std::string>(this->Name());
+  sdf::ElementPtr poseElem = elem->GetElement("pose");
+  if (!this->dataPtr->poseRelativeTo.empty())
+  {
+    poseElem->GetAttribute("relative_to")->Set<std::string>(
+        this->dataPtr->poseRelativeTo);
+  }
+  poseElem->Set<ignition::math::Pose3d>(this->RawPose());
+
+  elem->GetElement("cast_shadows")->Set<bool>(this->CastShadows());
+  elem->GetElement("intensity")->Set<double>(this->Intensity());
+  elem->GetElement("direction")->Set<ignition::math::Vector3d>(
+      this->Direction());
+  elem->GetElement("diffuse")->Set<ignition::math::Color>(this->Diffuse());
+  elem->GetElement("specular")->Set<ignition::math::Color>(this->Specular());
+  sdf::ElementPtr attenuationElem = elem->GetElement("attenuation");
+  attenuationElem->GetElement("linear")->Set<double>(
+      this->LinearAttenuationFactor());
+  attenuationElem->GetElement("constant")->Set<double>(
+      this->ConstantAttenuationFactor());
+  attenuationElem->GetElement("quadratic")->Set<double>(
+      this->QuadraticAttenuationFactor());
+  attenuationElem->GetElement("range")->Set<double>(
+      this->AttenuationRange());
+
+  sdf::ElementPtr spotElem = elem->GetElement("spot");
+  spotElem->GetElement("inner_angle")->Set<double>(
+      this->SpotInnerAngle().Radian());
+  spotElem->GetElement("outer_angle")->Set<double>(
+      this->SpotOuterAngle().Radian());
+  spotElem->GetElement("falloff")->Set<double>(this->SpotFalloff());
+  return elem;
+}
