@@ -25,14 +25,18 @@
 #include "sdf/Error.hh"
 #include "sdf/Light.hh"
 #include "sdf/Link.hh"
+#include "sdf/ParticleEmitter.hh"
 #include "sdf/Sensor.hh"
 #include "sdf/Types.hh"
 #include "sdf/Visual.hh"
+
+#include "FrameSemantics.hh"
+#include "ScopedGraph.hh"
 #include "Utils.hh"
 
 using namespace sdf;
 
-class sdf::LinkPrivate
+class sdf::Link::Implementation
 {
   /// \brief Name of the link.
   public: std::string name = "";
@@ -55,6 +59,9 @@ class sdf::LinkPrivate
   /// \brief The sensors specified in this link.
   public: std::vector<Sensor> sensors;
 
+  /// \brief The particle emitters specified in this link.
+  public: std::vector<ParticleEmitter> emitters;
+
   /// \brief The inertial information for this link.
   public: ignition::math::Inertiald inertial {{1.0,
             ignition::math::Vector3d::One, ignition::math::Vector3d::Zero},
@@ -66,48 +73,16 @@ class sdf::LinkPrivate
   /// \brief True if this link should be subject to wind, false otherwise.
   public: bool enableWind = false;
 
-  /// \brief Weak pointer to model's Pose Relative-To Graph.
-  public: std::weak_ptr<const sdf::PoseRelativeToGraph> poseRelativeToGraph;
+  /// \brief Scoped Pose Relative-To graph at the parent model scope.
+  public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseRelativeToGraph;
 };
 
 /////////////////////////////////////////////////
 Link::Link()
-  : dataPtr(new LinkPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Link::~Link()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-Link::Link(const Link &_link)
-  : dataPtr(new LinkPrivate(*_link.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Link::Link(Link &&_link) noexcept
-  : dataPtr(std::exchange(_link.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-Link &Link::operator=(const Link &_link)
-{
-  return *this = Link(_link);
-}
-
-/////////////////////////////////////////////////
-Link &Link::operator=(Link &&_link)
-{
-  std::swap(this->dataPtr, _link.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Link::Load(ElementPtr _sdf)
 {
@@ -163,6 +138,12 @@ Errors Link::Load(ElementPtr _sdf)
       this->dataPtr->sensors);
   errors.insert(errors.end(), sensorLoadErrors.begin(), sensorLoadErrors.end());
 
+  // Load all the particle emitters.
+  Errors emitterLoadErrors = loadUniqueRepeated<ParticleEmitter>(_sdf,
+      "particle_emitter", this->dataPtr->emitters);
+  errors.insert(errors.end(), emitterLoadErrors.begin(),
+      emitterLoadErrors.end());
+
   ignition::math::Vector3d xxyyzz = ignition::math::Vector3d::One;
   ignition::math::Vector3d xyxzyz = ignition::math::Vector3d::Zero;
   ignition::math::Pose3d inertiaPose;
@@ -217,7 +198,7 @@ std::string Link::Name() const
 }
 
 /////////////////////////////////////////////////
-void Link::SetName(const std::string &_name) const
+void Link::SetName(const std::string &_name)
 {
   this->dataPtr->name = _name;
 }
@@ -336,6 +317,48 @@ const Sensor *Link::SensorByName(const std::string &_name) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+uint64_t Link::ParticleEmitterCount() const
+{
+  return this->dataPtr->emitters.size();
+}
+
+/////////////////////////////////////////////////
+const ParticleEmitter *Link::ParticleEmitterByIndex(const uint64_t _index) const
+{
+  if (_index < this->dataPtr->emitters.size())
+    return &this->dataPtr->emitters[_index];
+  return nullptr;
+}
+
+/////////////////////////////////////////////////
+bool Link::ParticleEmitterNameExists(const std::string &_name) const
+{
+  for (auto const &e : this->dataPtr->emitters)
+  {
+    if (e.Name() == _name)
+    {
+      return true;
+    }
+  }
+  return false;
+}
+
+/////////////////////////////////////////////////
+const ParticleEmitter *Link::ParticleEmitterByName(
+    const std::string &_name) const
+{
+  for (auto const &e : this->dataPtr->emitters)
+  {
+    if (e.Name() == _name)
+    {
+      return &e;
+    }
+  }
+  return nullptr;
+}
+
+
 /////////////////////////////////////////////////
 const ignition::math::Inertiald &Link::Inertial() const
 {
@@ -349,48 +372,24 @@ bool Link::SetInertial(const ignition::math::Inertiald &_inertial)
   return _inertial.MassMatrix().IsValid();
 }
 
-/////////////////////////////////////////////////
-const ignition::math::Pose3d &Link::Pose() const
-{
-  return this->RawPose();
-}
-
 /////////////////////////////////////////////////
 const ignition::math::Pose3d &Link::RawPose() const
 {
   return this->dataPtr->pose;
 }
 
-/////////////////////////////////////////////////
-const std::string &Link::PoseFrame() const
-{
-  return this->PoseRelativeTo();
-}
-
 /////////////////////////////////////////////////
 const std::string &Link::PoseRelativeTo() const
 {
   return this->dataPtr->poseRelativeTo;
 }
 
-/////////////////////////////////////////////////
-void Link::SetPose(const ignition::math::Pose3d &_pose)
-{
-  this->SetRawPose(_pose);
-}
-
 /////////////////////////////////////////////////
 void Link::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
-/////////////////////////////////////////////////
-void Link::SetPoseFrame(const std::string &_frame)
-{
-  this->SetPoseRelativeTo(_frame);
-}
-
 /////////////////////////////////////////////////
 void Link::SetPoseRelativeTo(const std::string &_frame)
 {
@@ -398,8 +397,7 @@ void Link::SetPoseRelativeTo(const std::string &_frame)
 }
 
 /////////////////////////////////////////////////
-void Link::SetPoseRelativeToGraph(
-    std::weak_ptr<const PoseRelativeToGraph> _graph)
+void Link::SetPoseRelativeToGraph(sdf::ScopedGraph<PoseRelativeToGraph> _graph)
 {
   this->dataPtr->poseRelativeToGraph = _graph;
 
@@ -424,12 +422,19 @@ void Link::SetPoseRelativeToGraph(
     visual.SetXmlParentName(this->dataPtr->name);
     visual.SetPoseRelativeToGraph(_graph);
   }
+
+  for (auto &emitter : this->dataPtr->emitters)
+  {
+    emitter.SetXmlParentName(this->dataPtr->name);
+    emitter.SetPoseRelativeToGraph(_graph);
+  }
 }
 
 /////////////////////////////////////////////////
 sdf::SemanticPose Link::SemanticPose() const
 {
   return sdf::SemanticPose(
+      this->dataPtr->name,
       this->dataPtr->pose,
       this->dataPtr->poseRelativeTo,
       "__model__",
@@ -492,3 +497,63 @@ void Link::SetEnableWind(const bool _enableWind)
 {
   this->dataPtr->enableWind = _enableWind;
 }
+
+//////////////////////////////////////////////////
+bool Link::AddCollision(const Collision &_collision)
+{
+  if (this->CollisionNameExists(_collision.Name()))
+    return false;
+  this->dataPtr->collisions.push_back(_collision);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::AddVisual(const Visual &_visual)
+{
+  if (this->VisualNameExists(_visual.Name()))
+    return false;
+  this->dataPtr->visuals.push_back(_visual);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::AddLight(const Light &_light)
+{
+  if (this->LightNameExists(_light.Name()))
+    return false;
+  this->dataPtr->lights.push_back(_light);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Link::AddSensor(const Sensor &_sensor)
+{
+  if (this->SensorNameExists(_sensor.Name()))
+    return false;
+  this->dataPtr->sensors.push_back(_sensor);
+  return true;
+}
+
+//////////////////////////////////////////////////
+void Link::ClearCollisions()
+{
+  this->dataPtr->collisions.clear();
+}
+
+//////////////////////////////////////////////////
+void Link::ClearVisuals()
+{
+  this->dataPtr->visuals.clear();
+}
+
+//////////////////////////////////////////////////
+void Link::ClearLights()
+{
+  this->dataPtr->lights.clear();
+}
+
+//////////////////////////////////////////////////
+void Link::ClearSensors()
+{
+  this->dataPtr->sensors.clear();
+}
