@@ -22,17 +22,22 @@
 #include <ignition/math/SemanticVersion.hh>
 #include "sdf/Error.hh"
 #include "sdf/Frame.hh"
+#include "sdf/InterfaceElements.hh"
+#include "sdf/InterfaceModel.hh"
+#include "sdf/InterfaceModelPoseGraph.hh"
 #include "sdf/Joint.hh"
 #include "sdf/Link.hh"
 #include "sdf/Model.hh"
+#include "sdf/ParserConfig.hh"
 #include "sdf/Types.hh"
 #include "FrameSemantics.hh"
+#include "ScopedGraph.hh"
 #include "Utils.hh"
 #include "sdf/parser.hh"
 
 using namespace sdf;
 
-class sdf::ModelPrivate
+class sdf::Model::Implementation
 {
   /// \brief Name of the model.
   public: std::string name = "";
@@ -53,6 +58,9 @@ class sdf::ModelPrivate
   /// \brief Name of the canonical link.
   public: std::string canonicalLink = "";
 
+  /// \brief Name of the placement frame
+  public: std::string placementFrameName = "";
+
   /// \brief Pose of the model
   public: ignition::math::Pose3d pose = ignition::math::Pose3d::Zero;
 
@@ -71,90 +79,37 @@ class sdf::ModelPrivate
   /// \brief The nested models specified in this model.
   public: std::vector<Model> models;
 
+  /// \brief The interface models specified in this model.
+  public: std::vector<std::pair<sdf::NestedInclude, sdf::InterfaceModelPtr>>
+              interfaceModels;
+
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
 
-  /// \brief Frame Attached-To Graph constructed during Load.
-  public: std::shared_ptr<sdf::FrameAttachedToGraph> frameAttachedToGraph;
+  /// \brief Scoped Frame Attached-To graph at the parent model or world scope.
+  public: sdf::ScopedGraph<sdf::FrameAttachedToGraph> frameAttachedToGraph;
 
-  /// \brief Pose Relative-To Graph constructed during Load.
-  public: std::shared_ptr<sdf::PoseRelativeToGraph> poseGraph;
-
-  /// \brief Pose Relative-To Graph in parent (world or __model__) scope.
-  public: std::weak_ptr<const sdf::PoseRelativeToGraph> parentPoseGraph;
+  /// \brief Scoped Pose Relative-To graph at the parent model or world scope.
+  public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseGraph;
 
   /// \brief Scope name of parent Pose Relative-To Graph (world or __model__).
-  public: std::string parentPoseGraphScopeName;
+  public: std::string poseGraphScopeVertexName;
 };
 
 /////////////////////////////////////////////////
 Model::Model()
-  : dataPtr(new ModelPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Model::~Model()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-Model::Model(const Model &_model)
-  : dataPtr(new ModelPrivate(*_model.dataPtr))
-{
-  if (_model.dataPtr->frameAttachedToGraph)
-  {
-    this->dataPtr->frameAttachedToGraph =
-        std::make_shared<sdf::FrameAttachedToGraph>(
-            *_model.dataPtr->frameAttachedToGraph);
-  }
-  if (_model.dataPtr->poseGraph)
-  {
-    this->dataPtr->poseGraph = std::make_shared<sdf::PoseRelativeToGraph>(
-        *_model.dataPtr->poseGraph);
-  }
-  for (auto &link : this->dataPtr->links)
-  {
-    link.SetPoseRelativeToGraph(this->dataPtr->poseGraph);
-  }
-  for (auto &model : this->dataPtr->models)
-  {
-    model.SetPoseRelativeToGraph(this->dataPtr->poseGraph);
-  }
-  for (auto &joint : this->dataPtr->joints)
-  {
-    joint.SetPoseRelativeToGraph(this->dataPtr->poseGraph);
-  }
-  for (auto &frame : this->dataPtr->frames)
-  {
-    frame.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
-    frame.SetPoseRelativeToGraph(this->dataPtr->poseGraph);
-  }
-}
-
-/////////////////////////////////////////////////
-Model::Model(Model &&_model) noexcept
-  : dataPtr(std::exchange(_model.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-Model &Model::operator=(const Model &_model)
-{
-  return *this = Model(_model);
-}
-
-/////////////////////////////////////////////////
-Model &Model::operator=(Model &&_model)
-{
-  std::swap(this->dataPtr, _model.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Model::Load(ElementPtr _sdf)
+{
+  return this->Load(_sdf, ParserConfig::GlobalConfig());
+}
+
+/////////////////////////////////////////////////
+Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
 {
   Errors errors;
 
@@ -196,6 +151,9 @@ Errors Model::Load(ElementPtr _sdf)
     }
   }
 
+  this->dataPtr->placementFrameName = _sdf->Get<std::string>("placement_frame",
+                             this->dataPtr->placementFrameName).first;
+
   this->dataPtr->isStatic = _sdf->Get<bool>("static", false).first;
 
   this->dataPtr->selfCollide = _sdf->Get<bool>("self_collide", false).first;
@@ -225,7 +183,7 @@ Errors Model::Load(ElementPtr _sdf)
 
   // Load nested models.
   Errors nestedModelLoadErrors = loadUniqueRepeated<Model>(_sdf, "model",
-    this->dataPtr->models);
+    this->dataPtr->models, _config);
   errors.insert(errors.end(),
                 nestedModelLoadErrors.begin(),
                 nestedModelLoadErrors.end());
@@ -238,6 +196,17 @@ Errors Model::Load(ElementPtr _sdf)
     frameNames.insert(model.Name());
   }
 
+  // Load included models via the interface API
+  Errors interfaceModelLoadErrors = loadIncludedInterfaceModels(
+      _sdf, _config, this->dataPtr->interfaceModels);
+  errors.insert(errors.end(), interfaceModelLoadErrors.begin(),
+      interfaceModelLoadErrors.end());
+
+  for (const auto &ifaceModelPair : this->dataPtr->interfaceModels)
+  {
+    frameNames.insert(ifaceModelPair.second->Name());
+  }
+
   // Load all the links.
   Errors linkLoadErrors = loadUniqueRepeated<Link>(_sdf, "link",
     this->dataPtr->links);
@@ -279,7 +248,7 @@ Errors Model::Load(ElementPtr _sdf)
   // Require at least one link so the implicit model frame can be attached to
   // something.
   if (!this->Static() && this->dataPtr->links.empty() &&
-      this->dataPtr->models.empty())
+      this->dataPtr->models.empty() && this->dataPtr->interfaceModels.empty())
   {
     errors.push_back({ErrorCode::MODEL_WITHOUT_LINK,
                      "A model must have at least one link."});
@@ -359,58 +328,6 @@ Errors Model::Load(ElementPtr _sdf)
     frameNames.insert(frameName);
   }
 
-  // Build the graphs.
-
-  // Build the FrameAttachedToGraph if the model is not static.
-  // Re-enable this when the buildFrameAttachedToGraph implementation handles
-  // static models.
-  if (!this->Static())
-  {
-    this->dataPtr->frameAttachedToGraph
-        = std::make_shared<FrameAttachedToGraph>();
-    Errors frameAttachedToGraphErrors =
-    buildFrameAttachedToGraph(*this->dataPtr->frameAttachedToGraph, this);
-    errors.insert(errors.end(), frameAttachedToGraphErrors.begin(),
-                                frameAttachedToGraphErrors.end());
-    Errors validateFrameAttachedGraphErrors =
-      validateFrameAttachedToGraph(*this->dataPtr->frameAttachedToGraph);
-    errors.insert(errors.end(), validateFrameAttachedGraphErrors.begin(),
-                                validateFrameAttachedGraphErrors.end());
-    for (auto &frame : this->dataPtr->frames)
-    {
-      frame.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
-    }
-  }
-
-  // Build the PoseRelativeToGraph
-  this->dataPtr->poseGraph = std::make_shared<PoseRelativeToGraph>();
-  Errors poseGraphErrors =
-  buildPoseRelativeToGraph(*this->dataPtr->poseGraph, this);
-  errors.insert(errors.end(), poseGraphErrors.begin(),
-                              poseGraphErrors.end());
-  Errors validatePoseGraphErrors =
-    validatePoseRelativeToGraph(*this->dataPtr->poseGraph);
-  errors.insert(errors.end(), validatePoseGraphErrors.begin(),
-                              validatePoseGraphErrors.end());
-  for (auto &link : this->dataPtr->links)
-  {
-    link.SetPoseRelativeToGraph(this->dataPtr->poseGraph);
-  }
-  for (auto &model : this->dataPtr->models)
-  {
-    Errors setPoseRelativeToGraphErrors =
-      model.SetPoseRelativeToGraph(this->dataPtr->poseGraph);
-    errors.insert(errors.end(), setPoseRelativeToGraphErrors.begin(),
-                                setPoseRelativeToGraphErrors.end());
-  }
-  for (auto &joint : this->dataPtr->joints)
-  {
-    joint.SetPoseRelativeToGraph(this->dataPtr->poseGraph);
-  }
-  for (auto &frame : this->dataPtr->frames)
-  {
-    frame.SetPoseRelativeToGraph(this->dataPtr->poseGraph);
-  }
 
   return errors;
 }
@@ -418,29 +335,11 @@ Errors Model::Load(ElementPtr _sdf)
 /////////////////////////////////////////////////
 Errors Model::ValidateGraphs() const
 {
-  Errors errors;
-  if (!this->dataPtr->frameAttachedToGraph)
-  {
-    errors.push_back({ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR,
-        "FrameAttachedToGraph pointer is null."});
-  }
-  else
-  {
-    errors = validateFrameAttachedToGraph(*this->dataPtr->frameAttachedToGraph);
-  }
-
-  if (!this->dataPtr->poseGraph)
-  {
-    errors.push_back({ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR,
-        "PoseRelativeToGraph pointer is null."});
-  }
-  else
-  {
-    Errors poseErrors =
-        validatePoseRelativeToGraph(*this->dataPtr->poseGraph);
-    errors.insert(errors.end(), poseErrors.begin(), poseErrors.end());
-  }
-
+  Errors errors =
+      validateFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
+  Errors poseErrors =
+      validatePoseRelativeToGraph(this->dataPtr->poseGraph);
+  errors.insert(errors.end(), poseErrors.begin(), poseErrors.end());
   return errors;
 }
 
@@ -521,14 +420,7 @@ const Link *Model::LinkByIndex(const uint64_t _index) const
 /////////////////////////////////////////////////
 bool Model::LinkNameExists(const std::string &_name) const
 {
-  for (auto const &l : this->dataPtr->links)
-  {
-    if (l.Name() == _name)
-    {
-      return true;
-    }
-  }
-  return false;
+  return nullptr != this->LinkByName(_name);
 }
 
 /////////////////////////////////////////////////
@@ -548,19 +440,28 @@ const Joint *Model::JointByIndex(const uint64_t _index) const
 /////////////////////////////////////////////////
 bool Model::JointNameExists(const std::string &_name) const
 {
-  for (auto const &j : this->dataPtr->joints)
-  {
-    if (j.Name() == _name)
-    {
-      return true;
-    }
-  }
-  return false;
+  return nullptr != this->JointByName(_name);
 }
 
 /////////////////////////////////////////////////
 const Joint *Model::JointByName(const std::string &_name) const
 {
+  auto index = _name.rfind("::");
+  if (index != std::string::npos)
+  {
+    const Model *model = this->ModelByName(_name.substr(0, index));
+    if (nullptr != model)
+    {
+      return model->JointByName(_name.substr(index + 2));
+    }
+
+    // The nested model name preceding the last "::" could not be found.
+    // For now, try to find a link that matches _name exactly.
+    // When "::" are reserved and not allowed in names, then uncomment
+    // the following line to return a nullptr.
+    // return nullptr;
+  }
+
   for (auto const &j : this->dataPtr->joints)
   {
     if (j.Name() == _name)
@@ -588,19 +489,28 @@ const Frame *Model::FrameByIndex(const uint64_t _index) const
 /////////////////////////////////////////////////
 bool Model::FrameNameExists(const std::string &_name) const
 {
-  for (auto const &f : this->dataPtr->frames)
-  {
-    if (f.Name() == _name)
-    {
-      return true;
-    }
-  }
-  return false;
+  return nullptr != this->FrameByName(_name);
 }
 
 /////////////////////////////////////////////////
 const Frame *Model::FrameByName(const std::string &_name) const
 {
+  auto index = _name.rfind("::");
+  if (index != std::string::npos)
+  {
+    const Model *model = this->ModelByName(_name.substr(0, index));
+    if (nullptr != model)
+    {
+      return model->FrameByName(_name.substr(index + 2));
+    }
+
+    // The nested model name preceding the last "::" could not be found.
+    // For now, try to find a link that matches _name exactly.
+    // When "::" are reserved and not allowed in names, then uncomment
+    // the following line to return a nullptr.
+    // return nullptr;
+  }
+
   for (auto const &f : this->dataPtr->frames)
   {
     if (f.Name() == _name)
@@ -628,27 +538,30 @@ const Model *Model::ModelByIndex(const uint64_t _index) const
 /////////////////////////////////////////////////
 bool Model::ModelNameExists(const std::string &_name) const
 {
-  for (auto const &m : this->dataPtr->models)
-  {
-    if (m.Name() == _name)
-    {
-      return true;
-    }
-  }
-  return false;
+  return nullptr != this->ModelByName(_name);
 }
 
 /////////////////////////////////////////////////
 const Model *Model::ModelByName(const std::string &_name) const
 {
+  auto index = _name.find("::");
+  const std::string nextModelName = _name.substr(0, index);
+  const Model *nextModel = nullptr;
+
   for (auto const &m : this->dataPtr->models)
   {
-    if (m.Name() == _name)
+    if (m.Name() == nextModelName)
     {
-      return &m;
+      nextModel = &m;
+      break;
     }
   }
-  return nullptr;
+
+  if (nullptr != nextModel && index != std::string::npos)
+  {
+    return nextModel->ModelByName(_name.substr(index + 2));
+  }
+  return nextModel;
 }
 
 /////////////////////////////////////////////////
@@ -681,6 +594,20 @@ std::pair<const Link*, std::string> Model::CanonicalLinkAndRelativeName() const
       }
       return canonicalLinkAndName;
     }
+    else if (this->InterfaceModelCount() > 0)
+    {
+      // Recursively choose the canonical link of the first nested model
+      // (depth first search).
+      auto firstModel = this->InterfaceModelByIndex(0);
+      auto canonicalLinkName = firstModel->CanonicalLinkName();
+      // Prepend firstModelName if a valid link is found.
+      if (canonicalLinkName != "")
+      {
+        canonicalLinkName =
+            sdf::JoinName(firstModel->Name(), canonicalLinkName);
+      }
+      return {nullptr, canonicalLinkName};
+    }
     else
     {
       return std::make_pair(nullptr, "");
@@ -706,9 +633,15 @@ void Model::SetCanonicalLinkName(const std::string &_canonicalLink)
 }
 
 /////////////////////////////////////////////////
-const ignition::math::Pose3d &Model::Pose() const
+const std::string &Model::PlacementFrameName() const
 {
-  return this->RawPose();
+  return this->dataPtr->placementFrameName;
+}
+
+/////////////////////////////////////////////////
+void Model::SetPlacementFrameName(const std::string &_placementFrame)
+{
+  this->dataPtr->placementFrameName = _placementFrame;
 }
 
 /////////////////////////////////////////////////
@@ -717,36 +650,18 @@ const ignition::math::Pose3d &Model::RawPose() const
   return this->dataPtr->pose;
 }
 
-/////////////////////////////////////////////////
-const std::string &Model::PoseFrame() const
-{
-  return this->PoseRelativeTo();
-}
-
 /////////////////////////////////////////////////
 const std::string &Model::PoseRelativeTo() const
 {
   return this->dataPtr->poseRelativeTo;
 }
 
-/////////////////////////////////////////////////
-void Model::SetPose(const ignition::math::Pose3d &_pose)
-{
-  this->SetRawPose(_pose);
-}
-
 /////////////////////////////////////////////////
 void Model::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
-/////////////////////////////////////////////////
-void Model::SetPoseFrame(const std::string &_frame)
-{
-  this->SetPoseRelativeTo(_frame);
-}
-
 /////////////////////////////////////////////////
 void Model::SetPoseRelativeTo(const std::string &_frame)
 {
@@ -754,38 +669,88 @@ void Model::SetPoseRelativeTo(const std::string &_frame)
 }
 
 /////////////////////////////////////////////////
-Errors Model::SetPoseRelativeToGraph(
-    std::weak_ptr<const PoseRelativeToGraph> _graph)
+void Model::SetPoseRelativeToGraph(sdf::ScopedGraph<PoseRelativeToGraph> _graph)
 {
-  Errors errors;
+  this->dataPtr->poseGraph = _graph;
+  this->dataPtr->poseGraphScopeVertexName =
+      _graph.VertexLocalName(_graph.ScopeVertexId());
 
-  auto graph = _graph.lock();
-  if (!graph)
+  auto childPoseGraph =
+      this->dataPtr->poseGraph.ChildModelScope(this->Name());
+  for (auto &model : this->dataPtr->models)
   {
-    errors.push_back({ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR,
-        "Tried to set PoseRelativeToGraph with invalid pointer."});
-    return errors;
+    model.SetPoseRelativeToGraph(childPoseGraph);
   }
+  for (auto &ifaceModelPair : this->dataPtr->interfaceModels)
+  {
+    ifaceModelPair.second->InvokeRespostureFunction(childPoseGraph);
+  }
+  for (auto &link : this->dataPtr->links)
+  {
+    link.SetPoseRelativeToGraph(childPoseGraph);
+  }
+  for (auto &joint : this->dataPtr->joints)
+  {
+    joint.SetPoseRelativeToGraph(childPoseGraph);
+  }
+  for (auto &frame : this->dataPtr->frames)
+  {
+    frame.SetPoseRelativeToGraph(childPoseGraph);
+  }
+}
 
-  this->dataPtr->parentPoseGraphScopeName = graph->sourceName;
-  this->dataPtr->parentPoseGraph = _graph;
+/////////////////////////////////////////////////
+void Model::SetFrameAttachedToGraph(
+    sdf::ScopedGraph<FrameAttachedToGraph> _graph)
+{
+  this->dataPtr->frameAttachedToGraph = _graph;
 
-  return errors;
+  auto childFrameAttachedToGraph =
+      this->dataPtr->frameAttachedToGraph.ChildModelScope(this->Name());
+  for (auto &joint : this->dataPtr->joints)
+  {
+    joint.SetFrameAttachedToGraph(childFrameAttachedToGraph);
+  }
+  for (auto &frame : this->dataPtr->frames)
+  {
+    frame.SetFrameAttachedToGraph(childFrameAttachedToGraph);
+  }
+  for (auto &model : this->dataPtr->models)
+  {
+    model.SetFrameAttachedToGraph(childFrameAttachedToGraph);
+  }
 }
 
 /////////////////////////////////////////////////
 sdf::SemanticPose Model::SemanticPose() const
 {
   return sdf::SemanticPose(
+      this->dataPtr->name,
       this->dataPtr->pose,
       this->dataPtr->poseRelativeTo,
-      this->dataPtr->parentPoseGraphScopeName,
-      this->dataPtr->parentPoseGraph);
+      this->dataPtr->poseGraphScopeVertexName,
+      this->dataPtr->poseGraph);
 }
 
 /////////////////////////////////////////////////
 const Link *Model::LinkByName(const std::string &_name) const
 {
+  auto index = _name.rfind("::");
+  if (index != std::string::npos)
+  {
+    const Model *model = this->ModelByName(_name.substr(0, index));
+    if (nullptr != model)
+    {
+      return model->LinkByName(_name.substr(index + 2));
+    }
+
+    // The nested model name preceding the last "::" could not be found.
+    // For now, try to find a link that matches _name exactly.
+    // When "::" are reserved and not allowed in names, then uncomment
+    // the following line to return a nullptr.
+    // return nullptr;
+  }
+
   for (auto const &l : this->dataPtr->links)
   {
     if (l.Name() == _name)
@@ -801,3 +766,72 @@ sdf::ElementPtr Model::Element() const
 {
   return this->dataPtr->sdf;
 }
+
+/////////////////////////////////////////////////
+uint64_t Model::InterfaceModelCount() const
+{
+  return this->dataPtr->interfaceModels.size();
+}
+
+/////////////////////////////////////////////////
+InterfaceModelConstPtr Model::InterfaceModelByIndex(
+    const uint64_t _index) const
+{
+  if (_index < this->dataPtr->interfaceModels.size())
+    return this->dataPtr->interfaceModels[_index].second;
+  return nullptr;
+}
+
+/////////////////////////////////////////////////
+const NestedInclude *Model::InterfaceModelNestedIncludeByIndex(
+    const uint64_t _index) const
+{
+  if (_index < this->dataPtr->interfaceModels.size())
+    return &this->dataPtr->interfaceModels[_index].first;
+  return nullptr;
+}
+
+//////////////////////////////////////////////////
+bool Model::AddLink(const Link &_link)
+{
+  if (this->LinkNameExists(_link.Name()))
+    return false;
+  this->dataPtr->links.push_back(_link);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Model::AddJoint(const Joint &_joint)
+{
+  if (this->JointNameExists(_joint.Name()))
+    return false;
+  this->dataPtr->joints.push_back(_joint);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Model::AddModel(const Model &_model)
+{
+  if (this->ModelNameExists(_model.Name()))
+    return false;
+  this->dataPtr->models.push_back(_model);
+  return true;
+}
+
+//////////////////////////////////////////////////
+void Model::ClearLinks()
+{
+  this->dataPtr->links.clear();
+}
+
+//////////////////////////////////////////////////
+void Model::ClearJoints()
+{
+  this->dataPtr->joints.clear();
+}
+
+//////////////////////////////////////////////////
+void Model::ClearModels()
+{
+  this->dataPtr->models.clear();
+}
