@@ -17,12 +17,13 @@
 
 #include <algorithm>
 #include "sdf/Noise.hh"
+#include "sdf/parser.hh"
 #include "sdf/Types.hh"
 
 using namespace sdf;
 
 /// \brief Private noise data.
-class sdf::NoisePrivate
+class sdf::Noise::Implementation
 {
   /// \brief The noise type.
   public: NoiseType type = NoiseType::NONE;
@@ -56,42 +57,10 @@ class sdf::NoisePrivate
 
 //////////////////////////////////////////////////
 Noise::Noise()
-  : dataPtr(new NoisePrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-//////////////////////////////////////////////////
-Noise::~Noise()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-//////////////////////////////////////////////////
-Noise::Noise(const Noise &_noise)
-  : dataPtr(new NoisePrivate(*_noise.dataPtr))
-{
-}
-
-//////////////////////////////////////////////////
-Noise::Noise(Noise &&_noise) noexcept
-  : dataPtr(std::exchange(_noise.dataPtr, nullptr))
-{
-}
-
-//////////////////////////////////////////////////
-Noise &Noise::operator=(const Noise &_noise)
-{
-  return *this = Noise(_noise);
-}
-
-//////////////////////////////////////////////////
-Noise &Noise::operator=(Noise &&_noise)
-{
-  std::swap(this->dataPtr, _noise.dataPtr);
-  return *this;
-}
-
 //////////////////////////////////////////////////
 Errors Noise::Load(ElementPtr _sdf)
 {
@@ -280,3 +249,40 @@ bool Noise::operator==(const Noise &_noise) const
     ignition::math::equal(this->dataPtr->dynamicBiasCorrelationTime,
                           _noise.DynamicBiasCorrelationTime());
 }
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Noise::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("noise.sdf", elem);
+
+  std::string noiseType;
+  switch (this->Type())
+  {
+    case sdf::NoiseType::NONE:
+      noiseType = "none";
+      break;
+    case sdf::NoiseType::GAUSSIAN:
+      noiseType = "gaussian";
+      break;
+    case sdf::NoiseType::GAUSSIAN_QUANTIZED:
+      noiseType = "gaussian_quantized";
+      break;
+    default:
+      noiseType = "none";
+  }
+  elem->GetAttribute("type")->Set<std::string>(noiseType);
+  elem->GetElement("mean")->Set<double>(this->Mean());
+  elem->GetElement("stddev")->Set<double>(this->StdDev());
+
+  // camera and lidar <noise> does not have the sdf params below
+  elem->GetElement("bias_mean")->Set<double>(this->BiasMean());
+  elem->GetElement("bias_stddev")->Set<double>(this->BiasStdDev());
+  elem->GetElement("dynamic_bias_stddev")->Set<double>(
+    this->DynamicBiasStdDev());
+  elem->GetElement("dynamic_bias_correlation_time")->Set<double>(
+      this->DynamicBiasCorrelationTime());
+  elem->GetElement("precision")->Set<double>(this->Precision());
+
+  return elem;
+}
