@@ -20,18 +20,8 @@
 using namespace sdf;
 
 /// \brief Scene private data.
-class sdf::ScenePrivate
+class sdf::Scene::Implementation
 {
-  /// \brief Default constructor
-  public: ScenePrivate() = default;
-
-  /// \brief Copy constructor
-  /// \param[in] _scenePrivate private data to copy
-  public: explicit ScenePrivate(const ScenePrivate &_scenePrivate);
-
-  // Delete copy assignment so it is not accidentally used
-  public: ScenePrivate &operator=(const ScenePrivate &) = delete;
-
   /// \brief True if grid should be enabled
   public: bool grid = true;
 
@@ -50,66 +40,18 @@ class sdf::ScenePrivate
       ignition::math::Color(0.7f, 0.7f, .7f);
 
   /// \brief Pointer to the sky properties.
-  public: std::unique_ptr<Sky> sky;
+  public: std::optional<sdf::Sky> sky;
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
 };
 
-/////////////////////////////////////////////////
-ScenePrivate::ScenePrivate(const ScenePrivate &_scenePrivate)
-    : grid(_scenePrivate.grid),
-      shadows(_scenePrivate.shadows),
-      originVisual(_scenePrivate.originVisual),
-      ambient(_scenePrivate.ambient),
-      background(_scenePrivate.background),
-      sdf(_scenePrivate.sdf)
-{
-  if (_scenePrivate.sky)
-  {
-    this->sky =
-        std::make_unique<Sky>(*(_scenePrivate.sky));
-  }
-}
-
 /////////////////////////////////////////////////
 Scene::Scene()
-  : dataPtr(new ScenePrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Scene::~Scene()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-Scene::Scene(const Scene &_scene)
-  : dataPtr(new ScenePrivate(*_scene.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Scene::Scene(Scene &&_scene) noexcept
-  : dataPtr(std::exchange(_scene.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-Scene &Scene::operator=(const Scene &_scene)
-{
-  return *this = Scene(_scene);
-}
-
-/////////////////////////////////////////////////
-Scene &Scene::operator=(Scene &&_scene)
-{
-  std::swap(this->dataPtr, _scene.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Scene::Load(ElementPtr _sdf)
 {
@@ -150,7 +92,7 @@ Errors Scene::Load(ElementPtr _sdf)
   // load sky
   if (_sdf->HasElement("sky"))
   {
-    this->dataPtr->sky = std::make_unique<sdf::Sky>();
+    this->dataPtr->sky.emplace();
     Errors err = this->dataPtr->sky->Load(_sdf->GetElement("sky"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
@@ -220,13 +162,13 @@ void Scene::SetOriginVisual(const bool _enabled)
 /////////////////////////////////////////////////
 void Scene::SetSky(const sdf::Sky &_sky)
 {
-  this->dataPtr->sky = std::make_unique<sdf::Sky>(_sky);
+  this->dataPtr->sky = _sky;
 }
 
 /////////////////////////////////////////////////
 const sdf::Sky *Scene::Sky() const
 {
-  return this->dataPtr->sky.get();
+  return optionalToPointer(this->dataPtr->sky);
 }
 
 /////////////////////////////////////////////////
