@@ -16,6 +16,7 @@
 */
 #include <memory>
 #include <string>
+#include <optional>
 #include <vector>
 #include <ignition/math/Pose3.hh>
 #include "sdf/AirPressure.hh"
@@ -27,8 +28,11 @@
 #include "sdf/Imu.hh"
 #include "sdf/Magnetometer.hh"
 #include "sdf/Lidar.hh"
+#include "sdf/parser.hh"
 #include "sdf/Sensor.hh"
 #include "sdf/Types.hh"
+#include "FrameSemantics.hh"
+#include "ScopedGraph.hh"
 #include "Utils.hh"
 
 using namespace sdf;
@@ -58,68 +62,15 @@ const std::vector<std::string> sensorTypeStrs =
   "air_pressure",
   "rgbd_camera",
   "thermal_camera",
-  "navsat"
+  "navsat",
+  "segmentation_camera",
+  "boundingbox_camera",
+  "custom",
+  "wide_angle_camera"
 };
 
-class sdf::SensorPrivate
+class sdf::Sensor::Implementation
 {
-  /// \brief Default constructor
-  public: SensorPrivate() = default;
-
-  /// \brief Copy constructor
-  public: explicit SensorPrivate(const SensorPrivate &_sensor)
-          : type(_sensor.type),
-            name(_sensor.name),
-            topic(_sensor.topic),
-            pose(_sensor.pose),
-            poseRelativeTo(_sensor.poseRelativeTo),
-            sdf(_sensor.sdf),
-            enableMetrics(_sensor.enableMetrics),
-            updateRate(_sensor.updateRate)
-
-  {
-    if (_sensor.magnetometer)
-    {
-      this->magnetometer = std::make_unique<sdf::Magnetometer>(
-          *_sensor.magnetometer);
-    }
-    if (_sensor.navSat)
-    {
-      this->navSat = std::make_unique<sdf::NavSat>(*_sensor.navSat);
-    }
-    if (_sensor.altimeter)
-    {
-      this->altimeter = std::make_unique<sdf::Altimeter>(*_sensor.altimeter);
-    }
-    if (_sensor.airPressure)
-    {
-      this->airPressure = std::make_unique<sdf::AirPressure>(
-          *_sensor.airPressure);
-    }
-    if (_sensor.camera)
-    {
-      this->camera = std::make_unique<sdf::Camera>(*_sensor.camera);
-    }
-    if (_sensor.forceTorque)
-    {
-      this->forceTorque = std::make_unique<sdf::ForceTorque>(
-          *_sensor.forceTorque);
-    }
-    if (_sensor.imu)
-    {
-      this->imu = std::make_unique<sdf::Imu>(*_sensor.imu);
-    }
-    if (_sensor.lidar)
-    {
-      this->lidar = std::make_unique<sdf::Lidar>(*_sensor.lidar);
-    }
-    // Developer note: If you add a new sensor type, make sure to also
-    // update the Sensor::operator== function. Please bump this text down as
-    // new sensors are added so that the next developer sees the message.
-  }
-  // Delete copy assignment so it is not accidentally used
-  public: SensorPrivate &operator=(const SensorPrivate &) = delete;
-
   // \brief The sensor type.
   public: SensorType type = SensorType::NONE;
 
@@ -144,32 +95,32 @@ class sdf::SensorPrivate
   /// \brief Name of xml parent object.
   public: std::string xmlParentName;
 
-  /// \brief Weak pointer to model's Pose Relative-To Graph.
-  public: std::weak_ptr<const sdf::PoseRelativeToGraph> poseRelativeToGraph;
+  /// \brief Scoped Pose Relative-To graph at the parent model scope.
+  public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseRelativeToGraph;
 
-  /// \brief Pointer to a magnetometer.
-  public: std::unique_ptr<Magnetometer> magnetometer;
+  /// \brief Optional magnetometer.
+  public: std::optional<Magnetometer> magnetometer;
 
-  /// \brief Pointer to an altimeter.
-  public: std::unique_ptr<Altimeter> altimeter;
+  /// \brief Optional altimeter.
+  public: std::optional<Altimeter> altimeter;
 
-  /// \brief Pointer to NAVSAT sensor.
-  public: std::unique_ptr<NavSat> navSat;
+  /// \brief Optional NAVSAT sensor.
+  public: std::optional<NavSat> navSat;
 
-  /// \brief Pointer to an air pressure sensor.
-  public: std::unique_ptr<AirPressure> airPressure;
+  /// \brief Optional air pressure sensor.
+  public: std::optional<AirPressure> airPressure;
 
-  /// \brief Pointer to a camera.
-  public: std::unique_ptr<Camera> camera;
+  /// \brief Optional camera.
+  public: std::optional<Camera> camera;
 
-  /// \brief Pointer to a force torque sensor.
-  public: std::unique_ptr<ForceTorque> forceTorque;
+  /// \brief Optional force torque sensor.
+  public: std::optional<ForceTorque> forceTorque;
 
-  /// \brief Pointer to an IMU.
-  public: std::unique_ptr<Imu> imu;
+  /// \brief Optional IMU.
+  public: std::optional<Imu> imu;
 
-  /// \brief Pointer to a lidar.
-  public: std::unique_ptr<Lidar> lidar;
+  /// \brief Optional lidar.
+  public: std::optional<Lidar> lidar;
 
   // Developer note: If you add a new sensor type, make sure to also
   // update the Sensor::operator== function. Please bump this text down as
@@ -182,42 +133,10 @@ class sdf::SensorPrivate
 
 /////////////////////////////////////////////////
 Sensor::Sensor()
-  : dataPtr(new SensorPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Sensor::~Sensor()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-Sensor::Sensor(const Sensor &_sensor)
-  : dataPtr(new SensorPrivate(*_sensor.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Sensor::Sensor(Sensor &&_sensor) noexcept
-  : dataPtr(std::exchange(_sensor.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-Sensor &Sensor::operator=(const Sensor &_sensor)
-{
-  return *this = Sensor(_sensor);
-}
-
-/////////////////////////////////////////////////
-Sensor &Sensor::operator=(Sensor &&_sensor)
-{
-  std::swap(this->dataPtr, _sensor.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 bool Sensor::operator==(const Sensor &_sensor) const
 {
@@ -252,6 +171,9 @@ bool Sensor::operator==(const Sensor &_sensor) const
     case SensorType::DEPTH_CAMERA:
     case SensorType::RGBD_CAMERA:
     case SensorType::THERMAL_CAMERA:
+    case SensorType::WIDE_ANGLE_CAMERA:
+    case SensorType::SEGMENTATION_CAMERA:
+    case SensorType::BOUNDINGBOX_CAMERA:
       return *(this->dataPtr->camera) == *(_sensor.dataPtr->camera);
     case SensorType::LIDAR:
       return *(this->dataPtr->lidar) == *(_sensor.dataPtr->lidar);
@@ -321,7 +243,7 @@ Errors Sensor::Load(ElementPtr _sdf)
   if (type == "air_pressure")
   {
     this->dataPtr->type = SensorType::AIR_PRESSURE;
-    this->dataPtr->airPressure.reset(new AirPressure());
+    this->dataPtr->airPressure.emplace();
     Errors err = this->dataPtr->airPressure->Load(
         _sdf->GetElement("air_pressure"));
     errors.insert(errors.end(), err.begin(), err.end());
@@ -329,14 +251,14 @@ Errors Sensor::Load(ElementPtr _sdf)
   else if (type == "altimeter")
   {
     this->dataPtr->type = SensorType::ALTIMETER;
-    this->dataPtr->altimeter.reset(new Altimeter());
+    this->dataPtr->altimeter.emplace();
     Errors err = this->dataPtr->altimeter->Load(_sdf->GetElement("altimeter"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
   else if (type == "camera")
   {
     this->dataPtr->type = SensorType::CAMERA;
-    this->dataPtr->camera.reset(new Camera());
+    this->dataPtr->camera.emplace();
     Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
@@ -344,31 +266,56 @@ Errors Sensor::Load(ElementPtr _sdf)
   {
     this->dataPtr->type = SensorType::CONTACT;
   }
+  else if (type == "custom")
+  {
+    this->dataPtr->type = SensorType::CUSTOM;
+  }
   else if (type == "depth" || type == "depth_camera")
   {
     this->dataPtr->type = SensorType::DEPTH_CAMERA;
-    this->dataPtr->camera.reset(new Camera());
+    this->dataPtr->camera.emplace();
     Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
   else if (type == "rgbd" || type == "rgbd_camera")
   {
     this->dataPtr->type = SensorType::RGBD_CAMERA;
-    this->dataPtr->camera.reset(new Camera());
+    this->dataPtr->camera.emplace();
     Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
   else if (type == "thermal" || type == "thermal_camera")
   {
     this->dataPtr->type = SensorType::THERMAL_CAMERA;
-    this->dataPtr->camera.reset(new Camera());
+    this->dataPtr->camera.emplace();
+    Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
+    errors.insert(errors.end(), err.begin(), err.end());
+  }
+  else if (type == "segmentation" || type == "segmentation_camera")
+  {
+    this->dataPtr->type = SensorType::SEGMENTATION_CAMERA;
+    this->dataPtr->camera.emplace();
+    Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
+    errors.insert(errors.end(), err.begin(), err.end());
+  }
+  else if (type == "boundingbox" || type == "boundingbox_camera")
+  {
+    this->dataPtr->type = SensorType::BOUNDINGBOX_CAMERA;
+    this->dataPtr->camera.emplace();
+    Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
+    errors.insert(errors.end(), err.begin(), err.end());
+  }
+  else if (type == "wideanglecamera" || type == "wide_angle_camera")
+  {
+    this->dataPtr->type = SensorType::WIDE_ANGLE_CAMERA;
+    this->dataPtr->camera.emplace();
     Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
   else if (type == "force_torque")
   {
     this->dataPtr->type = SensorType::FORCE_TORQUE;
-    this->dataPtr->forceTorque.reset(new ForceTorque());
+    this->dataPtr->forceTorque.emplace();
     Errors err = this->dataPtr->forceTorque->Load(
         _sdf->GetElement("force_torque"));
     errors.insert(errors.end(), err.begin(), err.end());
@@ -376,7 +323,7 @@ Errors Sensor::Load(ElementPtr _sdf)
   else if (type == "navsat" || type == "gps")
   {
     this->dataPtr->type = SensorType::NAVSAT;
-    this->dataPtr->navSat.reset(new NavSat());
+    this->dataPtr->navSat.emplace();
     Errors err = this->dataPtr->navSat->Load(
       _sdf->GetElement(_sdf->HasElement("navsat") ? "navsat" : "gps"));
     errors.insert(errors.end(), err.begin(), err.end());
@@ -384,7 +331,7 @@ Errors Sensor::Load(ElementPtr _sdf)
   else if (type == "gpu_ray" || type == "gpu_lidar")
   {
     this->dataPtr->type = SensorType::GPU_LIDAR;
-    this->dataPtr->lidar.reset(new Lidar());
+    this->dataPtr->lidar.emplace();
     Errors err = this->dataPtr->lidar->Load(
         _sdf->GetElement(_sdf->HasElement("lidar") ? "lidar" : "ray"));
     errors.insert(errors.end(), err.begin(), err.end());
@@ -392,7 +339,7 @@ Errors Sensor::Load(ElementPtr _sdf)
   else if (type == "imu")
   {
     this->dataPtr->type = SensorType::IMU;
-    this->dataPtr->imu.reset(new Imu());
+    this->dataPtr->imu.emplace();
     Errors err = this->dataPtr->imu->Load(_sdf->GetElement("imu"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
@@ -403,7 +350,7 @@ Errors Sensor::Load(ElementPtr _sdf)
   else if (type == "magnetometer")
   {
     this->dataPtr->type = SensorType::MAGNETOMETER;
-    this->dataPtr->magnetometer.reset(new Magnetometer());
+    this->dataPtr->magnetometer.emplace();
     Errors err = this->dataPtr->magnetometer->Load(
         _sdf->GetElement("magnetometer"));
     errors.insert(errors.end(), err.begin(), err.end());
@@ -415,7 +362,7 @@ Errors Sensor::Load(ElementPtr _sdf)
   else if (type == "ray" || type == "lidar")
   {
     this->dataPtr->type = SensorType::LIDAR;
-    this->dataPtr->lidar.reset(new Lidar());
+    this->dataPtr->lidar.emplace();
     Errors err = this->dataPtr->lidar->Load(
         _sdf->GetElement(_sdf->HasElement("lidar") ? "lidar" : "ray"));
     errors.insert(errors.end(), err.begin(), err.end());
@@ -478,48 +425,24 @@ void Sensor::SetTopic(const std::string &_topic)
   this->dataPtr->topic = _topic;
 }
 
-/////////////////////////////////////////////////
-const ignition::math::Pose3d &Sensor::Pose() const
-{
-  return this->RawPose();
-}
-
 /////////////////////////////////////////////////
 const ignition::math::Pose3d &Sensor::RawPose() const
 {
   return this->dataPtr->pose;
 }
 
-/////////////////////////////////////////////////
-const std::string &Sensor::PoseFrame() const
-{
-  return this->PoseRelativeTo();
-}
-
 /////////////////////////////////////////////////
 const std::string &Sensor::PoseRelativeTo() const
 {
   return this->dataPtr->poseRelativeTo;
 }
 
-/////////////////////////////////////////////////
-void Sensor::SetPose(const ignition::math::Pose3d &_pose)
-{
-  this->SetRawPose(_pose);
-}
-
 /////////////////////////////////////////////////
 void Sensor::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
-/////////////////////////////////////////////////
-void Sensor::SetPoseFrame(const std::string &_frame)
-{
-  this->SetPoseRelativeTo(_frame);
-}
-
 /////////////////////////////////////////////////
 void Sensor::SetPoseRelativeTo(const std::string &_frame)
 {
@@ -534,7 +457,7 @@ void Sensor::SetXmlParentName(const std::string &_xmlParentName)
 
 /////////////////////////////////////////////////
 void Sensor::SetPoseRelativeToGraph(
-    std::weak_ptr<const PoseRelativeToGraph> _graph)
+    sdf::ScopedGraph<PoseRelativeToGraph> _graph)
 {
   this->dataPtr->poseRelativeToGraph = _graph;
 }
@@ -596,49 +519,73 @@ bool Sensor::SetType(const std::string &_typeStr)
 /////////////////////////////////////////////////
 const Magnetometer *Sensor::MagnetometerSensor() const
 {
-  return this->dataPtr->magnetometer.get();
+  return optionalToPointer(this->dataPtr->magnetometer);
+}
+
+/////////////////////////////////////////////////
+Magnetometer *Sensor::MagnetometerSensor()
+{
+  return optionalToPointer(this->dataPtr->magnetometer);
 }
 
 /////////////////////////////////////////////////
 void Sensor::SetMagnetometerSensor(const Magnetometer &_mag)
 {
-  this->dataPtr->magnetometer = std::make_unique<Magnetometer>(_mag);
+  this->dataPtr->magnetometer = _mag;
 }
 
 /////////////////////////////////////////////////
 const Altimeter *Sensor::AltimeterSensor() const
 {
-  return this->dataPtr->altimeter.get();
+  return optionalToPointer(this->dataPtr->altimeter);
+}
+
+/////////////////////////////////////////////////
+Altimeter *Sensor::AltimeterSensor()
+{
+  return optionalToPointer(this->dataPtr->altimeter);
 }
 
 /////////////////////////////////////////////////
 void Sensor::SetAltimeterSensor(const Altimeter &_alt)
 {
-  this->dataPtr->altimeter = std::make_unique<Altimeter>(_alt);
+  this->dataPtr->altimeter = _alt;
 }
 
 /////////////////////////////////////////////////
 const AirPressure *Sensor::AirPressureSensor() const
 {
-  return this->dataPtr->airPressure.get();
+  return optionalToPointer(this->dataPtr->airPressure);
+}
+
+/////////////////////////////////////////////////
+AirPressure *Sensor::AirPressureSensor()
+{
+  return optionalToPointer(this->dataPtr->airPressure);
 }
 
 /////////////////////////////////////////////////
 void Sensor::SetAirPressureSensor(const AirPressure &_air)
 {
-  this->dataPtr->airPressure = std::make_unique<AirPressure>(_air);
+  this->dataPtr->airPressure = _air;
 }
 
 /////////////////////////////////////////////////
 const Lidar *Sensor::LidarSensor() const
 {
-  return this->dataPtr->lidar.get();
+  return optionalToPointer(this->dataPtr->lidar);
+}
+
+/////////////////////////////////////////////////
+Lidar *Sensor::LidarSensor()
+{
+  return optionalToPointer(this->dataPtr->lidar);
 }
 
 /////////////////////////////////////////////////
 void Sensor::SetLidarSensor(const Lidar &_lidar)
 {
-  this->dataPtr->lidar = std::make_unique<Lidar>(_lidar);
+  this->dataPtr->lidar = _lidar;
 }
 
 /////////////////////////////////////////////////
@@ -665,47 +612,148 @@ std::string Sensor::TypeStr() const
 /////////////////////////////////////////////////
 void Sensor::SetCameraSensor(const Camera &_cam)
 {
-  this->dataPtr->camera = std::make_unique<Camera>(_cam);
+  this->dataPtr->camera = _cam;
 }
 
 /////////////////////////////////////////////////
 const Camera *Sensor::CameraSensor() const
 {
-  return this->dataPtr->camera.get();
+  return optionalToPointer(this->dataPtr->camera);
+}
+
+/////////////////////////////////////////////////
+Camera *Sensor::CameraSensor()
+{
+  return optionalToPointer(this->dataPtr->camera);
 }
 
 /////////////////////////////////////////////////
 void Sensor::SetForceTorqueSensor(const ForceTorque &_ft)
 {
-  this->dataPtr->forceTorque = std::make_unique<ForceTorque>(_ft);
+  this->dataPtr->forceTorque = _ft;
 }
 
 /////////////////////////////////////////////////
 const ForceTorque *Sensor::ForceTorqueSensor() const
 {
-  return this->dataPtr->forceTorque.get();
+  return optionalToPointer(this->dataPtr->forceTorque);
+}
+
+/////////////////////////////////////////////////
+ForceTorque *Sensor::ForceTorqueSensor()
+{
+  return optionalToPointer(this->dataPtr->forceTorque);
 }
 
 /////////////////////////////////////////////////
 void Sensor::SetNavSatSensor(const NavSat &_gps)
 {
-  this->dataPtr->navSat = std::make_unique<NavSat>(_gps);
+  this->dataPtr->navSat = _gps;
 }
 
 /////////////////////////////////////////////////
 const NavSat *Sensor::NavSatSensor() const
 {
-  return this->dataPtr->navSat.get();
+  return optionalToPointer(this->dataPtr->navSat);
+}
+
+/////////////////////////////////////////////////
+NavSat *Sensor::NavSatSensor()
+{
+  return optionalToPointer(this->dataPtr->navSat);
 }
 
 /////////////////////////////////////////////////
 void Sensor::SetImuSensor(const Imu &_imu)
 {
-  this->dataPtr->imu = std::make_unique<Imu>(_imu);
+  this->dataPtr->imu = _imu;
 }
 
 /////////////////////////////////////////////////
 const Imu *Sensor::ImuSensor() const
 {
-  return this->dataPtr->imu.get();
+  return optionalToPointer(this->dataPtr->imu);
+}
+
+/////////////////////////////////////////////////
+Imu *Sensor::ImuSensor()
+{
+  return optionalToPointer(this->dataPtr->imu);
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Sensor::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("sensor.sdf", elem);
+
+  elem->GetAttribute("type")->Set<std::string>(this->TypeStr());
+  elem->GetAttribute("name")->Set<std::string>(this->Name());
+  sdf::ElementPtr poseElem = elem->GetElement("pose");
+  if (!this->dataPtr->poseRelativeTo.empty())
+  {
+    poseElem->GetAttribute("relative_to")->Set<std::string>(
+        this->dataPtr->poseRelativeTo);
+  }
+  poseElem->Set<ignition::math::Pose3d>(this->RawPose());
+
+  elem->GetElement("topic")->Set<std::string>(this->Topic());
+  elem->GetElement("update_rate")->Set<double>(this->UpdateRate());
+  elem->GetElement("enable_metrics")->Set<double>(this->EnableMetrics());
+
+  // air pressure
+  if (this->Type() == sdf::SensorType::AIR_PRESSURE &&
+      this->dataPtr->airPressure)
+  {
+    sdf::ElementPtr airPressureElem = elem->GetElement("air_pressure");
+    airPressureElem->Copy(this->dataPtr->airPressure->ToElement());
+  }
+  // altimeter
+  else if (this->Type() == sdf::SensorType::ALTIMETER &&
+      this->dataPtr->altimeter)
+  {
+    sdf::ElementPtr altimeterElem = elem->GetElement("altimeter");
+    altimeterElem->Copy(this->dataPtr->altimeter->ToElement());
+  }
+  // camera, depth, thermal, segmentation
+  else if (this->CameraSensor())
+  {
+    sdf::ElementPtr cameraElem = elem->GetElement("camera");
+    cameraElem->Copy(this->dataPtr->camera->ToElement());
+  }
+  // force torque
+  else if (this->Type() == sdf::SensorType::FORCE_TORQUE  &&
+      this->dataPtr->forceTorque)
+  {
+    sdf::ElementPtr forceTorqueElem = elem->GetElement("force_torque");
+    forceTorqueElem->Copy(this->dataPtr->forceTorque->ToElement());
+  }
+  // imu
+  else if (this->Type() == sdf::SensorType::IMU && this->dataPtr->imu)
+  {
+    sdf::ElementPtr imuElem = elem->GetElement("imu");
+    imuElem->Copy(this->dataPtr->imu->ToElement());
+  }
+  // lidar, gpu_lidar
+  else if ((this->Type() == sdf::SensorType::GPU_LIDAR ||
+            this->Type() == sdf::SensorType::LIDAR) &&
+           this->dataPtr->lidar)
+  {
+    sdf::ElementPtr rayElem = (elem->HasElement("ray")) ?
+        elem->GetElement("ray") : elem->GetElement("lidar");
+    rayElem->Copy(this->dataPtr->lidar->ToElement());
+  }
+  // magnetometer
+  else if (this->Type() == sdf::SensorType::MAGNETOMETER &&
+      this->dataPtr->magnetometer)
+  {
+    sdf::ElementPtr magnetometerElem = elem->GetElement("magnetometer");
+    magnetometerElem->Copy(this->dataPtr->magnetometer->ToElement());
+  }
+  else
+  {
+    std::cout << "Conversion of sensor type: [" << this->TypeStr() << "] from "
+      << "SDF DOM to Element is not supported yet." << std::endl;
+  }
+  return elem;
 }
