@@ -244,6 +244,7 @@ TEST(DOMSensor, Type)
   std::vector<sdf::SensorType> types = {
     sdf::SensorType::NONE,
     sdf::SensorType::ALTIMETER,
+    sdf::SensorType::BOUNDINGBOX_CAMERA,
     sdf::SensorType::CAMERA,
     sdf::SensorType::CONTACT,
     sdf::SensorType::DEPTH_CAMERA,
@@ -257,15 +258,19 @@ TEST(DOMSensor, Type)
     sdf::SensorType::LIDAR,
     sdf::SensorType::RFID,
     sdf::SensorType::RFIDTAG,
+    sdf::SensorType::SEGMENTATION_CAMERA,
     sdf::SensorType::SONAR,
     sdf::SensorType::WIRELESS_RECEIVER,
     sdf::SensorType::WIRELESS_TRANSMITTER,
-    sdf::SensorType::THERMAL_CAMERA
+    sdf::SensorType::THERMAL_CAMERA,
+    sdf::SensorType::CUSTOM,
+    sdf::SensorType::WIDE_ANGLE_CAMERA
   };
   std::vector<std::string> typeStrs =
   {
     "none",
     "altimeter",
+    "boundingbox_camera",
     "camera",
     "contact",
     "depth_camera",
@@ -279,10 +284,13 @@ TEST(DOMSensor, Type)
     "lidar",
     "rfid",
     "rfidtag",
+    "segmentation_camera",
     "sonar",
     "wireless_receiver",
     "wireless_transmitter",
-    "thermal_camera"
+    "thermal_camera",
+    "custom",
+    "wide_angle_camera"
   };
 
   for (size_t i = 0; i < types.size(); ++i)
@@ -330,3 +338,210 @@ TEST(DOMSensor, EnableMetrics)
   sensor.SetEnableMetrics(false);
   EXPECT_EQ(false, sensor.EnableMetrics());
 }
+
+/////////////////////////////////////////////////
+TEST(DOMSensor, MutableSensors)
+{
+  // Altimeter
+  {
+    sdf::Sensor sensor;
+    sensor.SetType(sdf::SensorType::ALTIMETER);
+
+    sdf::Altimeter alt;
+    sensor.SetAltimeterSensor(alt);
+
+    sdf::Altimeter *altMutable = sensor.AltimeterSensor();
+    ASSERT_NE(nullptr, altMutable);
+    EXPECT_DOUBLE_EQ(altMutable->VerticalPositionNoise().Mean(),
+        sensor.AltimeterSensor()->VerticalPositionNoise().Mean());
+
+    sdf::Noise noise;
+    noise.SetMean(2.0);
+    altMutable->SetVerticalPositionNoise(noise);
+    EXPECT_DOUBLE_EQ(altMutable->VerticalPositionNoise().Mean(), 2.0);
+    EXPECT_DOUBLE_EQ(
+        sensor.AltimeterSensor()->VerticalPositionNoise().Mean(), 2.0);
+  }
+
+  // Air pressure
+  {
+    sdf::Sensor sensor;
+    sensor.SetType(sdf::SensorType::AIR_PRESSURE);
+
+    sdf::AirPressure air;
+    sensor.SetAirPressureSensor(air);
+
+    sdf::AirPressure *airMutable = sensor.AirPressureSensor();
+    ASSERT_NE(nullptr, airMutable);
+    EXPECT_DOUBLE_EQ(airMutable->ReferenceAltitude(),
+        sensor.AirPressureSensor()->ReferenceAltitude());
+
+    airMutable->SetReferenceAltitude(2.0);
+    EXPECT_DOUBLE_EQ(airMutable->ReferenceAltitude(), 2.0);
+    EXPECT_DOUBLE_EQ(sensor.AirPressureSensor()->ReferenceAltitude(), 2.0);
+  }
+
+  // Camera
+  {
+    sdf::Sensor sensor;
+    sensor.SetType(sdf::SensorType::CAMERA);
+
+    sdf::Camera cam;
+    sensor.SetCameraSensor(cam);
+
+    sdf::Camera *camMutable = sensor.CameraSensor();
+    ASSERT_NE(nullptr, camMutable);
+    EXPECT_DOUBLE_EQ(camMutable->NearClip(), sensor.CameraSensor()->NearClip());
+
+    camMutable->SetNearClip(2.0);
+    EXPECT_DOUBLE_EQ(camMutable->NearClip(), 2.0);
+    EXPECT_DOUBLE_EQ(sensor.CameraSensor()->NearClip(), 2.0);
+  }
+
+  // Force torque
+  {
+    sdf::Sensor sensor;
+    sensor.SetType(sdf::SensorType::FORCE_TORQUE);
+
+    sdf::ForceTorque ftq;
+    sensor.SetForceTorqueSensor(ftq);
+
+    sdf::ForceTorque *ftqMutable = sensor.ForceTorqueSensor();
+    ASSERT_NE(nullptr, ftqMutable);
+    EXPECT_DOUBLE_EQ(ftqMutable->ForceXNoise().Mean(),
+        sensor.ForceTorqueSensor()->ForceXNoise().Mean());
+
+    sdf::Noise noise;
+    noise.SetMean(2.0);
+    ftqMutable->SetForceXNoise(noise);
+    EXPECT_DOUBLE_EQ(ftqMutable->ForceXNoise().Mean(), 2.0);
+    EXPECT_DOUBLE_EQ(
+        sensor.ForceTorqueSensor()->ForceXNoise().Mean(), 2.0);
+  }
+
+  // IMU
+  {
+    sdf::Sensor sensor;
+    sensor.SetType(sdf::SensorType::FORCE_TORQUE);
+
+    sdf::Imu imu;
+    sensor.SetImuSensor(imu);
+
+    sdf::Imu *imuMutable = sensor.ImuSensor();
+    ASSERT_NE(nullptr, imuMutable);
+    EXPECT_DOUBLE_EQ(imuMutable->LinearAccelerationXNoise().Mean(),
+        sensor.ImuSensor()->LinearAccelerationXNoise().Mean());
+
+    sdf::Noise noise;
+    noise.SetMean(2.0);
+    imuMutable->SetLinearAccelerationXNoise(noise);
+    EXPECT_DOUBLE_EQ(imuMutable->LinearAccelerationXNoise().Mean(), 2.0);
+    EXPECT_DOUBLE_EQ(
+        sensor.ImuSensor()->LinearAccelerationXNoise().Mean(), 2.0);
+  }
+
+  // Lidar
+  {
+    sdf::Sensor sensor;
+    sensor.SetType(sdf::SensorType::LIDAR);
+
+    sdf::Lidar ldr;
+    sensor.SetLidarSensor(ldr);
+
+    sdf::Lidar *ldrMutable = sensor.LidarSensor();
+    ASSERT_NE(nullptr, ldrMutable);
+    EXPECT_DOUBLE_EQ(ldrMutable->LidarNoise().Mean(),
+        sensor.LidarSensor()->LidarNoise().Mean());
+
+    sdf::Noise noise;
+    noise.SetMean(2.0);
+    ldrMutable->SetLidarNoise(noise);
+    EXPECT_DOUBLE_EQ(ldrMutable->LidarNoise().Mean(), 2.0);
+    EXPECT_DOUBLE_EQ(
+        sensor.LidarSensor()->LidarNoise().Mean(), 2.0);
+  }
+
+  // Magnetometer
+  {
+    sdf::Sensor sensor;
+    sensor.SetType(sdf::SensorType::MAGNETOMETER);
+
+    sdf::Magnetometer mag;
+    sensor.SetMagnetometerSensor(mag);
+
+    sdf::Magnetometer *magMutable = sensor.MagnetometerSensor();
+    ASSERT_NE(nullptr, magMutable);
+    EXPECT_DOUBLE_EQ(magMutable->XNoise().Mean(),
+        sensor.MagnetometerSensor()->XNoise().Mean());
+
+    sdf::Noise noise;
+    noise.SetMean(2.0);
+    magMutable->SetXNoise(noise);
+    EXPECT_DOUBLE_EQ(magMutable->XNoise().Mean(), 2.0);
+    EXPECT_DOUBLE_EQ(sensor.MagnetometerSensor()->XNoise().Mean(), 2.0);
+  }
+
+  // NavSat
+  {
+    sdf::Sensor sensor;
+    sensor.SetType(sdf::SensorType::NAVSAT);
+
+    sdf::NavSat nav;
+    sensor.SetNavSatSensor(nav);
+
+    sdf::NavSat *navMutable = sensor.NavSatSensor();
+    ASSERT_NE(nullptr, navMutable);
+    EXPECT_DOUBLE_EQ(navMutable->HorizontalPositionNoise().Mean(),
+        sensor.NavSatSensor()->HorizontalPositionNoise().Mean());
+
+    sdf::Noise noise;
+    noise.SetMean(2.0);
+    navMutable->SetHorizontalPositionNoise(noise);
+    EXPECT_DOUBLE_EQ(navMutable->HorizontalPositionNoise().Mean(), 2.0);
+    EXPECT_DOUBLE_EQ(
+        sensor.NavSatSensor()->HorizontalPositionNoise().Mean(), 2.0);
+  }
+}
+
+/////////////////////////////////////////////////
+TEST(DOMSensor, ToElement)
+{
+  // test calling ToElement on a DOM object constructed without calling Load
+  sdf::Sensor sensor;
+  sensor.SetName("my_sensor");
+  sensor.SetRawPose(ignition::math::Pose3d(1, 2, 3, 0, 0, 0));
+  sensor.SetType(sdf::SensorType::MAGNETOMETER);
+  sensor.SetPoseRelativeTo("a_frame");
+  sensor.SetUpdateRate(0.123);
+
+  sdf::Noise noise;
+  noise.SetMean(0.1);
+  sdf::Magnetometer mag;
+  mag.SetXNoise(noise);
+  sensor.SetMagnetometerSensor(mag);
+
+  sdf::ElementPtr sensorElem = sensor.ToElement();
+  EXPECT_NE(nullptr, sensorElem);
+  EXPECT_EQ(nullptr, sensor.Element());
+
+  // verify values after loading the element back
+  sdf::Sensor sensor2;
+  sensor2.Load(sensorElem);
+
+  EXPECT_EQ("my_sensor", sensor2.Name());
+  EXPECT_EQ(sdf::SensorType::MAGNETOMETER, sensor2.Type());
+  EXPECT_EQ(ignition::math::Pose3d(1, 2, 3, 0, 0, 0), sensor2.RawPose());
+  EXPECT_EQ("a_frame", sensor2.PoseRelativeTo());
+  ASSERT_TRUE(nullptr != sensor2.MagnetometerSensor());
+  EXPECT_DOUBLE_EQ(mag.XNoise().Mean(),
+                   sensor2.MagnetometerSensor()->XNoise().Mean());
+  EXPECT_DOUBLE_EQ(0.123, sensor2.UpdateRate());
+
+  // make changes to DOM and verify ToElement produces updated values
+  sensor2.SetUpdateRate(1.23);
+  sdf::ElementPtr sensor2Elem = sensor2.ToElement();
+  EXPECT_NE(nullptr, sensor2Elem);
+  sdf::Sensor sensor3;
+  sensor3.Load(sensor2Elem);
+  EXPECT_DOUBLE_EQ(1.23, sensor3.UpdateRate());
+}
