@@ -21,22 +21,14 @@
 #include "sdf/Types.hh"
 #include "sdf/Visual.hh"
 #include "sdf/Geometry.hh"
+#include "FrameSemantics.hh"
+#include "ScopedGraph.hh"
 #include "Utils.hh"
 
 using namespace sdf;
 
-class sdf::VisualPrivate
+class sdf::Visual::Implementation
 {
-  /// \brief Default constructor
-  public: VisualPrivate() = default;
-
-  /// \brief Copy constructor
-  /// \param[in] _visualPrivate Joint axis to move.
-  public: explicit VisualPrivate(const VisualPrivate &_visualPrivate);
-
-  // Delete copy assignment so it is not accidentally used
-  public: VisualPrivate &operator=(const VisualPrivate &) = delete;
-
   /// \brief Name of the visual.
   public: std::string name = "";
 
@@ -58,14 +50,14 @@ class sdf::VisualPrivate
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
 
-  /// \brief Pointer to the visual's material properties.
-  public: std::unique_ptr<Material> material;
+  /// \brief The visual's material properties.
+  public: std::optional<sdf::Material> material;
 
   /// \brief Name of xml parent object.
   public: std::string xmlParentName;
 
-  /// \brief Weak pointer to model's Pose Relative-To Graph.
-  public: std::weak_ptr<const sdf::PoseRelativeToGraph> poseRelativeToGraph;
+  /// \brief Scoped Pose Relative-To graph at the parent model scope.
+  public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseRelativeToGraph;
 
   /// \brief Visibility flags of a visual. Defaults to 0xFFFFFFFF
   public: uint32_t visibilityFlags = 4294967295u;
@@ -77,63 +69,12 @@ class sdf::VisualPrivate
   public: double laserRetro = 0;
 };
 
-/////////////////////////////////////////////////
-VisualPrivate::VisualPrivate(const VisualPrivate &_visualPrivate)
-    : name(_visualPrivate.name),
-      castShadows(_visualPrivate.castShadows),
-      transparency(_visualPrivate.transparency),
-      pose(_visualPrivate.pose),
-      poseRelativeTo(_visualPrivate.poseRelativeTo),
-      geom(_visualPrivate.geom),
-      sdf(_visualPrivate.sdf),
-      visibilityFlags(_visualPrivate.visibilityFlags),
-      hasLaserRetro(_visualPrivate.hasLaserRetro),
-      laserRetro(_visualPrivate.laserRetro)
-{
-  if (_visualPrivate.material)
-  {
-    this->material = std::make_unique<Material>(*(_visualPrivate.material));
-  }
-}
-
 /////////////////////////////////////////////////
 Visual::Visual()
-  : dataPtr(new VisualPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Visual::~Visual()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-Visual::Visual(const Visual &_visual)
-  : dataPtr(new VisualPrivate(*_visual.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Visual::Visual(Visual &&_visual) noexcept
-  : dataPtr(std::exchange(_visual.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-Visual &Visual::operator=(const Visual &_visual)
-{
-  return *this = Visual(_visual);
-}
-
-/////////////////////////////////////////////////
-Visual &Visual::operator=(Visual &&_visual)
-{
-  std::swap(this->dataPtr, _visual.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Visual::Load(ElementPtr _sdf)
 {
@@ -181,7 +122,7 @@ Errors Visual::Load(ElementPtr _sdf)
 
   if (_sdf->HasElement("material"))
   {
-    this->dataPtr->material.reset(new sdf::Material());
+    this->dataPtr->material.emplace();
     Errors err = this->dataPtr->material->Load(_sdf->GetElement("material"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
@@ -217,7 +158,7 @@ std::string Visual::Name() const
 }
 
 /////////////////////////////////////////////////
-void Visual::SetName(const std::string &_name) const
+void Visual::SetName(const std::string &_name)
 {
   this->dataPtr->name = _name;
 }
@@ -246,48 +187,24 @@ void Visual::SetTransparency(float _transparency)
   this->dataPtr->transparency = _transparency;
 }
 
-/////////////////////////////////////////////////
-const ignition::math::Pose3d &Visual::Pose() const
-{
-  return this->RawPose();
-}
-
 /////////////////////////////////////////////////
 const ignition::math::Pose3d &Visual::RawPose() const
 {
   return this->dataPtr->pose;
 }
 
-/////////////////////////////////////////////////
-const std::string &Visual::PoseFrame() const
-{
-  return this->PoseRelativeTo();
-}
-
 /////////////////////////////////////////////////
 const std::string &Visual::PoseRelativeTo() const
 {
   return this->dataPtr->poseRelativeTo;
 }
 
-/////////////////////////////////////////////////
-void Visual::SetPose(const ignition::math::Pose3d &_pose)
-{
-  this->SetRawPose(_pose);
-}
-
 /////////////////////////////////////////////////
 void Visual::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
 
-/////////////////////////////////////////////////
-void Visual::SetPoseFrame(const std::string &_frame)
-{
-  this->SetPoseRelativeTo(_frame);
-}
-
 /////////////////////////////////////////////////
 void Visual::SetPoseRelativeTo(const std::string &_frame)
 {
@@ -323,15 +240,15 @@ sdf::ElementPtr Visual::Element() const
 }
 
 /////////////////////////////////////////////////
-sdf::Material *Visual::Material() const
+const sdf::Material *Visual::Material() const
 {
-  return this->dataPtr->material.get();
+  return optionalToPointer(this->dataPtr->material);
 }
 
 /////////////////////////////////////////////////
 void Visual::SetMaterial(const sdf::Material &_material)
 {
-  this->dataPtr->material.reset(new sdf::Material(_material));
+  this->dataPtr->material = _material;
 }
 
 /////////////////////////////////////////////////
@@ -379,7 +296,7 @@ void Visual::SetXmlParentName(const std::string &_xmlParentName)
 
 /////////////////////////////////////////////////
 void Visual::SetPoseRelativeToGraph(
-    std::weak_ptr<const PoseRelativeToGraph> _graph)
+    sdf::ScopedGraph<PoseRelativeToGraph> _graph)
 {
   this->dataPtr->poseRelativeToGraph = _graph;
 }
