@@ -17,35 +17,36 @@
 #include <string>
 #include <unordered_set>
 #include <vector>
+#include <optional>
 #include <ignition/math/Vector3.hh>
 
 #include "sdf/Actor.hh"
 #include "sdf/Frame.hh"
+#include "sdf/InterfaceElements.hh"
+#include "sdf/InterfaceModel.hh"
+#include "sdf/InterfaceModelPoseGraph.hh"
 #include "sdf/Light.hh"
 #include "sdf/Model.hh"
+#include "sdf/ParserConfig.hh"
 #include "sdf/Physics.hh"
 #include "sdf/Types.hh"
 #include "sdf/World.hh"
 #include "FrameSemantics.hh"
+#include "ScopedGraph.hh"
 #include "Utils.hh"
 #include "sdf/parser.hh"
 
 using namespace sdf;
 
-class sdf::WorldPrivate
+class sdf::World::Implementation
 {
-  /// \brief Default constructor
-  public: WorldPrivate() = default;
+  /// \brief Populate sphericalCoordinates
+  /// \param[in] _elem `<spherical_coordinates>` element
+  /// \return Errors, if any.
+  public: Errors LoadSphericalCoordinates(sdf::ElementPtr _elem);
 
-  /// \brief Copy constructor
-  /// \param[in] _worldPrivate Joint axis to move.
-  public: explicit WorldPrivate(const WorldPrivate &_worldPrivate);
-
-  // Delete copy assignment so it is not accidentally used
-  public: WorldPrivate &operator=(const WorldPrivate &) = delete;
-
-  /// \brief Pointer to an atmosphere model.
-  public: std::unique_ptr<Atmosphere> atmosphere;
+  /// \brief Optional atmosphere model.
+  public: std::optional<sdf::Atmosphere> atmosphere;
 
   /// \brief Audio device name
   public: std::string audioDevice = "default";
@@ -54,11 +55,11 @@ class sdf::WorldPrivate
   public: ignition::math::Vector3d gravity =
            ignition::math::Vector3d(0, 0, -9.80665);
 
-  /// \brief Pointer to Gui parameters.
-  public: std::unique_ptr<Gui> gui;
+  /// \brief Optional Gui parameters.
+  public: std::optional<sdf::Gui> gui;
 
-  /// \brief Pointer to Sene parameters.
-  public: std::unique_ptr<Scene> scene;
+  /// \brief Optional Scene parameters.
+  public: std::optional<sdf::Scene> scene;
 
   /// \brief The frames specified in this world.
   public: std::vector<Frame> frames;
@@ -73,9 +74,17 @@ class sdf::WorldPrivate
   public: ignition::math::Vector3d magneticField =
            ignition::math::Vector3d(5.5645e-6, 22.8758e-6, -42.3884e-6);
 
+  /// \brief Spherical coordinates
+  public: std::optional<ignition::math::SphericalCoordinates>
+      sphericalCoordinates;
+
   /// \brief The models specified in this world.
   public: std::vector<Model> models;
 
+  /// \brief The interface models specified in this world.
+  public: std::vector<std::pair<sdf::NestedInclude, sdf::InterfaceModelPtr>>
+      interfaceModels;
+
   /// \brief Name of the world.
   public: std::string name = "";
 
@@ -89,107 +98,31 @@ class sdf::WorldPrivate
   public: ignition::math::Vector3d windLinearVelocity =
            ignition::math::Vector3d::Zero;
 
-  /// \brief Frame Attached-To Graph constructed during Load.
-  public: std::shared_ptr<sdf::FrameAttachedToGraph> frameAttachedToGraph;
+  /// \brief Scoped Frame Attached-To graph that points to a graph owned
+  /// by this world.
+  public: sdf::ScopedGraph<sdf::FrameAttachedToGraph> frameAttachedToGraph;
 
-  /// \brief Pose Relative-To Graph constructed during Load.
-  public: std::shared_ptr<sdf::PoseRelativeToGraph> poseRelativeToGraph;
+  /// \brief Scoped Pose Relative-To graph that points to a graph owned by this
+  /// world.
+  public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseRelativeToGraph;
 };
 
-/////////////////////////////////////////////////
-WorldPrivate::WorldPrivate(const WorldPrivate &_worldPrivate)
-    : audioDevice(_worldPrivate.audioDevice),
-      gravity(_worldPrivate.gravity),
-      frames(_worldPrivate.frames),
-      lights(_worldPrivate.lights),
-      actors(_worldPrivate.actors),
-      magneticField(_worldPrivate.magneticField),
-      models(_worldPrivate.models),
-      name(_worldPrivate.name),
-      physics(_worldPrivate.physics),
-      sdf(_worldPrivate.sdf),
-      windLinearVelocity(_worldPrivate.windLinearVelocity)
-{
-  if (_worldPrivate.atmosphere)
-  {
-    this->atmosphere =
-        std::make_unique<Atmosphere>(*(_worldPrivate.atmosphere));
-  }
-  if (_worldPrivate.gui)
-  {
-    this->gui = std::make_unique<Gui>(*(_worldPrivate.gui));
-  }
-  if (_worldPrivate.frameAttachedToGraph)
-  {
-    this->frameAttachedToGraph = std::make_shared<sdf::FrameAttachedToGraph>(
-        *(_worldPrivate.frameAttachedToGraph));
-  }
-  if (_worldPrivate.poseRelativeToGraph)
-  {
-    this->poseRelativeToGraph = std::make_shared<sdf::PoseRelativeToGraph>(
-        *(_worldPrivate.poseRelativeToGraph));
-  }
-  if (_worldPrivate.scene)
-  {
-    this->scene = std::make_unique<Scene>(*(_worldPrivate.scene));
-  }
-}
 
 /////////////////////////////////////////////////
 World::World()
-  : dataPtr(new WorldPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
   this->dataPtr->physics.emplace_back(Physics());
 }
 
-/////////////////////////////////////////////////
-World::~World()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-World::World(const World &_world)
-  : dataPtr(new WorldPrivate(*_world.dataPtr))
-{
-  for (auto &frame : this->dataPtr->frames)
-  {
-    frame.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
-    frame.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
-  }
-  for (auto &model : this->dataPtr->models)
-  {
-    model.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
-  }
-  for (auto &light : this->dataPtr->lights)
-  {
-    light.SetXmlParentName("world");
-    light.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
-  }
-}
-
-/////////////////////////////////////////////////
-World::World(World &&_world) noexcept
-  : dataPtr(std::exchange(_world.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-World &World::operator=(const World &_world)
-{
-  return *this = World(_world);
-}
-
-/////////////////////////////////////////////////
-World &World::operator=(World &&_world)
-{
-  std::swap(this->dataPtr, _world.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors World::Load(sdf::ElementPtr _sdf)
+{
+  return this->Load(_sdf, ParserConfig::GlobalConfig());
+}
+
+/////////////////////////////////////////////////
+Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
 {
   Errors errors;
 
@@ -240,7 +173,7 @@ Errors World::Load(sdf::ElementPtr _sdf)
   // Read the atmosphere element
   if (_sdf->HasElement("atmosphere"))
   {
-    this->dataPtr->atmosphere.reset(new sdf::Atmosphere());
+    this->dataPtr->atmosphere.emplace();
     Errors atmosphereLoadErrors =
       this->dataPtr->atmosphere->Load(_sdf->GetElement("atmosphere"));
     errors.insert(errors.end(), atmosphereLoadErrors.begin(),
@@ -256,6 +189,26 @@ Errors World::Load(sdf::ElementPtr _sdf)
     _sdf->Get<ignition::math::Vector3d>("magnetic_field",
         this->dataPtr->magneticField).first;
 
+  // Read the spherical coordinates
+  if (_sdf->HasElement("spherical_coordinates"))
+  {
+    Errors sphericalCoordsErrors = this->dataPtr->LoadSphericalCoordinates(
+        _sdf->GetElement("spherical_coordinates"));
+    errors.insert(errors.end(), sphericalCoordsErrors.begin(),
+        sphericalCoordsErrors.end());
+  }
+
+  for (const auto &[name, size] :
+       _sdf->CountNamedElements("", Element::NameUniquenessExceptions()))
+  {
+    if (size > 1)
+    {
+      sdfwarn << "Non-unique name[" << name << "] detected " << size
+              << " times in XML children of world with name[" << this->Name()
+              << "].\n";
+    }
+  }
+
   for (const auto &[name, size] :
        _sdf->CountNamedElements("", Element::NameUniquenessExceptions()))
   {
@@ -272,8 +225,8 @@ Errors World::Load(sdf::ElementPtr _sdf)
   std::unordered_set<std::string> frameNames;
 
   // Load all the models.
-  Errors modelLoadErrors = loadUniqueRepeated<Model>(_sdf, "model",
-      this->dataPtr->models);
+  Errors modelLoadErrors =
+      loadUniqueRepeated<Model>(_sdf, "model", this->dataPtr->models, _config);
   errors.insert(errors.end(), modelLoadErrors.begin(), modelLoadErrors.end());
 
   // Models are loaded first, and loadUniqueRepeated ensures there are no
@@ -284,6 +237,17 @@ Errors World::Load(sdf::ElementPtr _sdf)
     frameNames.insert(model.Name());
   }
 
+  // Load included models via the interface API
+  Errors interfaceModelLoadErrors = loadIncludedInterfaceModels(
+      _sdf, _config, this->dataPtr->interfaceModels);
+  errors.insert(errors.end(), interfaceModelLoadErrors.begin(),
+      interfaceModelLoadErrors.end());
+
+  for (const auto &ifaceModelPair : this->dataPtr->interfaceModels)
+  {
+    frameNames.insert(ifaceModelPair.second->Name()).second;
+  }
+
   // Load all the physics.
   if (_sdf->HasElement("physics"))
   {
@@ -333,7 +297,7 @@ Errors World::Load(sdf::ElementPtr _sdf)
   // Load the Gui
   if (_sdf->HasElement("gui"))
   {
-    this->dataPtr->gui.reset(new sdf::Gui());
+    this->dataPtr->gui.emplace();
     Errors guiLoadErrors = this->dataPtr->gui->Load(_sdf->GetElement("gui"));
     errors.insert(errors.end(), guiLoadErrors.begin(), guiLoadErrors.end());
   }
@@ -341,83 +305,23 @@ Errors World::Load(sdf::ElementPtr _sdf)
   // Load the Scene
   if (_sdf->HasElement("scene"))
   {
-    this->dataPtr->scene.reset(new sdf::Scene());
+    this->dataPtr->scene.emplace();
     Errors sceneLoadErrors =
         this->dataPtr->scene->Load(_sdf->GetElement("scene"));
     errors.insert(errors.end(), sceneLoadErrors.begin(), sceneLoadErrors.end());
   }
 
-  // Build the graphs.
-  this->dataPtr->frameAttachedToGraph =
-      std::make_shared<FrameAttachedToGraph>();
-  Errors frameAttachedToGraphErrors =
-  buildFrameAttachedToGraph(*this->dataPtr->frameAttachedToGraph, this);
-  errors.insert(errors.end(), frameAttachedToGraphErrors.begin(),
-                              frameAttachedToGraphErrors.end());
-  Errors validateFrameAttachedGraphErrors =
-    validateFrameAttachedToGraph(*this->dataPtr->frameAttachedToGraph);
-  errors.insert(errors.end(), validateFrameAttachedGraphErrors.begin(),
-                              validateFrameAttachedGraphErrors.end());
-  for (auto &frame : this->dataPtr->frames)
-  {
-    frame.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
-  }
-
-  this->dataPtr->poseRelativeToGraph = std::make_shared<PoseRelativeToGraph>();
-  Errors poseRelativeToGraphErrors =
-  buildPoseRelativeToGraph(*this->dataPtr->poseRelativeToGraph, this);
-  errors.insert(errors.end(), poseRelativeToGraphErrors.begin(),
-                              poseRelativeToGraphErrors.end());
-  Errors validatePoseGraphErrors =
-    validatePoseRelativeToGraph(*this->dataPtr->poseRelativeToGraph);
-  errors.insert(errors.end(), validatePoseGraphErrors.begin(),
-                              validatePoseGraphErrors.end());
-  for (auto &frame : this->dataPtr->frames)
-  {
-    frame.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
-  }
-  for (auto &model : this->dataPtr->models)
-  {
-    Errors setPoseRelativeToGraphErrors =
-      model.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
-    errors.insert(errors.end(), setPoseRelativeToGraphErrors.begin(),
-                                setPoseRelativeToGraphErrors.end());
-  }
-  for (auto &light : this->dataPtr->lights)
-  {
-    light.SetXmlParentName("world");
-    light.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
-  }
-
   return errors;
 }
 
 /////////////////////////////////////////////////
 Errors World::ValidateGraphs() const
 {
-  Errors errors;
-  if (!this->dataPtr->frameAttachedToGraph)
-  {
-    errors.push_back({ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR,
-        "FrameAttachedToGraph pointer is null."});
-  }
-  else
-  {
-    errors = validateFrameAttachedToGraph(*this->dataPtr->frameAttachedToGraph);
-  }
-
-  if (!this->dataPtr->poseRelativeToGraph)
-  {
-    errors.push_back({ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR,
-        "PoseRelativeToGraph pointer is null."});
-  }
-  else
-  {
-    Errors poseErrors =
-        validatePoseRelativeToGraph(*this->dataPtr->poseRelativeToGraph);
-    errors.insert(errors.end(), poseErrors.begin(), poseErrors.end());
-  }
-
+  Errors errors =
+      validateFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
+  Errors poseErrors =
+      validatePoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
+  errors.insert(errors.end(), poseErrors.begin(), poseErrors.end());
   return errors;
 }
 
@@ -428,7 +332,7 @@ std::string World::Name() const
 }
 
 /////////////////////////////////////////////////
-void World::SetName(const std::string &_name) const
+void World::SetName(const std::string &_name)
 {
   this->dataPtr->name = _name;
 }
@@ -498,63 +402,80 @@ const Model *World::ModelByIndex(const uint64_t _index) const
 /////////////////////////////////////////////////
 bool World::ModelNameExists(const std::string &_name) const
 {
-  for (auto const &m : this->dataPtr->models)
-  {
-    if (m.Name() == _name)
-    {
-      return true;
-    }
-  }
-  return false;
+  return nullptr != this->ModelByName(_name);
 }
 
 /////////////////////////////////////////////////
 const Model *World::ModelByName(const std::string &_name) const
 {
+  auto index = _name.find("::");
+  const std::string nextModelName = _name.substr(0, index);
+  const Model *nextModel = nullptr;
+
   for (auto const &m : this->dataPtr->models)
   {
-    if (m.Name() == _name)
+    if (m.Name() == nextModelName)
     {
-      return &m;
+      nextModel = &m;
+      break;
     }
   }
-  return nullptr;
+
+  if (nullptr != nextModel && index != std::string::npos)
+  {
+    return nextModel->ModelByName(_name.substr(index + 2));
+  }
+  return nextModel;
 }
 
 /////////////////////////////////////////////////
 const sdf::Atmosphere *World::Atmosphere() const
 {
-  return this->dataPtr->atmosphere.get();
+  return optionalToPointer(this->dataPtr->atmosphere);
 }
 
 /////////////////////////////////////////////////
-void World::SetAtmosphere(const sdf::Atmosphere &_atmosphere) const
+void World::SetAtmosphere(const sdf::Atmosphere &_atmosphere)
 {
-  this->dataPtr->atmosphere.reset(new sdf::Atmosphere(_atmosphere));
+  this->dataPtr->atmosphere = _atmosphere;
 }
 
 /////////////////////////////////////////////////
-sdf::Gui *World::Gui() const
+const ignition::math::SphericalCoordinates *
+    World::SphericalCoordinates() const
 {
-  return this->dataPtr->gui.get();
+  return optionalToPointer(this->dataPtr->sphericalCoordinates);
+}
+
+/////////////////////////////////////////////////
+void World::SetSphericalCoordinates(const ignition::math::SphericalCoordinates
+    &_sphericalCoordinates)
+{
+  this->dataPtr->sphericalCoordinates = _sphericalCoordinates;
+}
+
+/////////////////////////////////////////////////
+const sdf::Gui *World::Gui() const
+{
+  return optionalToPointer(this->dataPtr->gui);
 }
 
 /////////////////////////////////////////////////
 void World::SetGui(const sdf::Gui &_gui)
 {
-  return this->dataPtr->gui.reset(new sdf::Gui(_gui));
+  this->dataPtr->gui = _gui;
 }
 
 /////////////////////////////////////////////////
 const sdf::Scene *World::Scene() const
 {
-  return this->dataPtr->scene.get();
+  return optionalToPointer(this->dataPtr->scene);
 }
 
 /////////////////////////////////////////////////
 void World::SetScene(const sdf::Scene &_scene)
 {
-  return this->dataPtr->scene.reset(new sdf::Scene(_scene));
+  this->dataPtr->scene = _scene;
 }
 
 /////////////////////////////////////////////////
@@ -580,19 +501,28 @@ const Frame *World::FrameByIndex(const uint64_t _index) const
 /////////////////////////////////////////////////
 bool World::FrameNameExists(const std::string &_name) const
 {
-  for (auto const &f : this->dataPtr->frames)
-  {
-    if (f.Name() == _name)
-    {
-      return true;
-    }
-  }
-  return false;
+  return nullptr != this->FrameByName(_name);
 }
 
 /////////////////////////////////////////////////
 const Frame *World::FrameByName(const std::string &_name) const
 {
+  auto index = _name.rfind("::");
+  if (index != std::string::npos)
+  {
+    const Model *model = this->ModelByName(_name.substr(0, index));
+    if (nullptr != model)
+    {
+      return model->FrameByName(_name.substr(index + 2));
+    }
+
+    // The nested model name preceding the last "::" could not be found.
+    // For now, try to find a link that matches _name exactly.
+    // When "::" are reserved and not allowed in names, then uncomment
+    // the following line to return a nullptr.
+    // return nullptr;
+  }
+
   for (auto const &f : this->dataPtr->frames)
   {
     if (f.Name() == _name)
@@ -701,3 +631,216 @@ bool World::PhysicsNameExists(const std::string &_name) const
 
   return false;
 }
+
+/////////////////////////////////////////////////
+uint64_t World::InterfaceModelCount() const
+{
+  return this->dataPtr->interfaceModels.size();
+}
+
+/////////////////////////////////////////////////
+InterfaceModelConstPtr World::InterfaceModelByIndex(
+    const uint64_t _index) const
+{
+  if (_index < this->dataPtr->interfaceModels.size())
+    return this->dataPtr->interfaceModels[_index].second;
+  return nullptr;
+}
+
+/////////////////////////////////////////////////
+const NestedInclude *World::InterfaceModelNestedIncludeByIndex(
+    const uint64_t _index) const
+{
+  if (_index < this->dataPtr->interfaceModels.size())
+    return &this->dataPtr->interfaceModels[_index].first;
+  return nullptr;
+}
+
+/////////////////////////////////////////////////
+void World::SetPoseRelativeToGraph(sdf::ScopedGraph<PoseRelativeToGraph> _graph)
+{
+  this->dataPtr->poseRelativeToGraph = _graph;
+
+  for (auto &model : this->dataPtr->models)
+  {
+    model.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
+  }
+  for (auto &ifaceModelPair : this->dataPtr->interfaceModels)
+  {
+    ifaceModelPair.second->InvokeRespostureFunction(
+        this->dataPtr->poseRelativeToGraph);
+  }
+  for (auto &frame : this->dataPtr->frames)
+  {
+    frame.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
+  }
+  for (auto &light : this->dataPtr->lights)
+  {
+    light.SetXmlParentName("world");
+    light.SetPoseRelativeToGraph(this->dataPtr->poseRelativeToGraph);
+  }
+}
+
+/////////////////////////////////////////////////
+void World::SetFrameAttachedToGraph(
+    sdf::ScopedGraph<FrameAttachedToGraph> _graph)
+{
+  this->dataPtr->frameAttachedToGraph = _graph;
+
+  for (auto &frame : this->dataPtr->frames)
+  {
+    frame.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
+  }
+  for (auto &model : this->dataPtr->models)
+  {
+    model.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
+  }
+}
+
+/////////////////////////////////////////////////
+Errors World::Implementation::LoadSphericalCoordinates(
+    sdf::ElementPtr _elem)
+{
+  Errors errors;
+
+  if (_elem->GetName() != "spherical_coordinates")
+  {
+    errors.push_back({ErrorCode::ELEMENT_INCORRECT_TYPE,
+        "Attempting to load <spherical_coordinates>, but the provided SDF "
+        "element is a <" + _elem->GetName() + ">."});
+    return errors;
+  }
+
+  // Get elements
+  auto surfaceModel =
+      ignition::math::SphericalCoordinates::SurfaceType::EARTH_WGS84;
+  if (!_elem->HasElement("surface_model"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <surface_model>"});
+  }
+  else
+  {
+    auto surfaceModelStr = _elem->Get<std::string>("surface_model");
+    if (surfaceModelStr != "EARTH_WGS84")
+    {
+      errors.push_back({ErrorCode::ELEMENT_INVALID,
+          "The supplied <surface_model> [" + surfaceModelStr +
+          "] is not supported."});
+    }
+    surfaceModel = ignition::math::SphericalCoordinates::Convert(
+        surfaceModelStr);
+  }
+
+  std::string worldFrameOrientation{"ENU"};
+  if (_elem->HasElement("world_frame_orientation"))
+  {
+    worldFrameOrientation = _elem->Get<std::string>(
+        "world_frame_orientation");
+    if (worldFrameOrientation != "ENU")
+    {
+      errors.push_back({ErrorCode::ELEMENT_INVALID,
+          "The supplied <world_frame_orientation> [" + worldFrameOrientation +
+          "] is not supported."});
+    }
+  }
+
+  ignition::math::Angle latitude{0.0};
+  if (!_elem->HasElement("latitude_deg"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <latitude_deg>"});
+  }
+  else
+  {
+    latitude.SetDegree(_elem->Get<double>("latitude_deg"));
+  }
+
+  ignition::math::Angle longitude{0.0};
+  if (!_elem->HasElement("longitude_deg"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <longitude_deg>"});
+  }
+  else
+  {
+    longitude.SetDegree(_elem->Get<double>("longitude_deg"));
+  }
+
+  double elevation{0.0};
+  if (!_elem->HasElement("elevation"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <elevation>"});
+  }
+  else
+  {
+    elevation = _elem->Get<double>("elevation");
+  }
+
+  ignition::math::Angle heading{0.0};
+  if (!_elem->HasElement("heading_deg"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <heading_deg>"});
+  }
+  else
+  {
+    heading.SetDegree(_elem->Get<double>("heading_deg"));
+  }
+
+  // Create coordinates
+  this->sphericalCoordinates.emplace();
+  this->sphericalCoordinates =
+      ignition::math::SphericalCoordinates(surfaceModel, latitude, longitude,
+      elevation, heading);
+
+  return errors;
+}
+
+/////////////////////////////////////////////////
+void World::ClearModels()
+{
+  this->dataPtr->models.clear();
+}
+
+/////////////////////////////////////////////////
+void World::ClearActors()
+{
+  this->dataPtr->actors.clear();
+}
+
+/////////////////////////////////////////////////
+void World::ClearLights()
+{
+  this->dataPtr->lights.clear();
+}
+
+/////////////////////////////////////////////////
+bool World::AddModel(const Model &_model)
+{
+  if (this->ModelNameExists(_model.Name()))
+    return false;
+  this->dataPtr->models.push_back(_model);
+  return true;
+}
+
+/////////////////////////////////////////////////
+bool World::AddActor(const Actor &_actor)
+{
+  if (this->ActorNameExists(_actor.Name()))
+    return false;
+  this->dataPtr->actors.push_back(_actor);
+
+  return true;
+}
+
+/////////////////////////////////////////////////
+bool World::AddLight(const Light &_light)
+{
+  if (this->LightNameExists(_light.Name()))
+    return false;
+  this->dataPtr->lights.push_back(_light);
+
+  return true;
+}
