@@ -18,6 +18,9 @@
 #include <gtest/gtest.h>
 #include <ignition/math/Color.hh>
 #include <ignition/math/Vector3.hh>
+#include "sdf/Light.hh"
+#include "sdf/Actor.hh"
+#include "sdf/Model.hh"
 #include "sdf/World.hh"
 
 /////////////////////////////////////////////////
@@ -37,8 +40,14 @@ TEST(DOMWorld, Construction)
   EXPECT_EQ(nullptr, world.ModelByIndex(1));
   EXPECT_FALSE(world.ModelNameExists(""));
   EXPECT_FALSE(world.ModelNameExists("default"));
+  EXPECT_FALSE(world.ModelNameExists("a::b"));
+  EXPECT_FALSE(world.ModelNameExists("a::b::c"));
+  EXPECT_FALSE(world.ModelNameExists("::::"));
   EXPECT_EQ(nullptr, world.ModelByName(""));
   EXPECT_EQ(nullptr, world.ModelByName("default"));
+  EXPECT_EQ(nullptr, world.ModelByName("a::b"));
+  EXPECT_EQ(nullptr, world.ModelByName("a::b::c"));
+  EXPECT_EQ(nullptr, world.ModelByName("::::"));
 
   EXPECT_EQ(0u, world.FrameCount());
   EXPECT_EQ(nullptr, world.FrameByIndex(0));
@@ -55,15 +64,15 @@ TEST(DOMWorld, Construction)
   EXPECT_EQ(1u, world.PhysicsCount());
 
   auto errors = world.ValidateGraphs();
-  EXPECT_EQ(2u, errors.size());
+  EXPECT_EQ(2u, errors.size()) << errors;
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
-      "FrameAttachedToGraph pointer is null"));
+      "FrameAttachedToGraph error: scope does not point to a valid graph"));
   EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[1].Message().find(
-      "PoseRelativeToGraph pointer is null"));
+      "PoseRelativeToGraph error: scope does not point to a valid graph"));
 }
 
 /////////////////////////////////////////////////
@@ -75,6 +84,7 @@ TEST(DOMWorld, CopyConstructor)
   world.SetAtmosphere(atmosphere);
   world.SetAudioDevice("test_audio_device");
   world.SetGravity({1, 0, 0});
+  world.SetSphericalCoordinates(ignition::math::SphericalCoordinates());
 
   sdf::Gui gui;
   gui.SetFullscreen(true);
@@ -93,6 +103,9 @@ TEST(DOMWorld, CopyConstructor)
 
   ASSERT_TRUE(nullptr != world.Atmosphere());
   EXPECT_DOUBLE_EQ(0.1, world.Atmosphere()->Pressure());
+  ASSERT_TRUE(nullptr != world.SphericalCoordinates());
+  EXPECT_EQ(ignition::math::SphericalCoordinates::EARTH_WGS84,
+      world.SphericalCoordinates()->Surface());
   EXPECT_EQ("test_audio_device", world.AudioDevice());
   EXPECT_EQ(ignition::math::Vector3d::UnitX, world.Gravity());
 
@@ -338,3 +351,72 @@ TEST(DOMWorld, SetScene)
   EXPECT_TRUE(world.Scene()->Shadows());
   EXPECT_TRUE(world.Scene()->OriginVisual());
 }
+
+/////////////////////////////////////////////////
+TEST(DOMWorld, AddModel)
+{
+  sdf::World world;
+  EXPECT_EQ(0u, world.ModelCount());
+
+  sdf::Model model;
+  model.SetName("model1");
+  EXPECT_TRUE(world.AddModel(model));
+  EXPECT_EQ(1u, world.ModelCount());
+  EXPECT_FALSE(world.AddModel(model));
+  EXPECT_EQ(1u, world.ModelCount());
+
+  world.ClearModels();
+  EXPECT_EQ(0u, world.ModelCount());
+
+  EXPECT_TRUE(world.AddModel(model));
+  EXPECT_EQ(1u, world.ModelCount());
+  const sdf::Model *modelFromWorld = world.ModelByIndex(0);
+  ASSERT_NE(nullptr, modelFromWorld);
+  EXPECT_EQ(modelFromWorld->Name(), model.Name());
+}
+
+/////////////////////////////////////////////////
+TEST(DOMWorld, AddActor)
+{
+  sdf::World world;
+  EXPECT_EQ(0u, world.ActorCount());
+
+  sdf::Actor actor;
+  actor.SetName("actor1");
+  EXPECT_TRUE(world.AddActor(actor));
+  EXPECT_EQ(1u, world.ActorCount());
+  EXPECT_FALSE(world.AddActor(actor));
+  EXPECT_EQ(1u, world.ActorCount());
+
+  world.ClearActors();
+  EXPECT_EQ(0u, world.ActorCount());
+
+  EXPECT_TRUE(world.AddActor(actor));
+  EXPECT_EQ(1u, world.ActorCount());
+  const sdf::Actor *actorFromWorld = world.ActorByIndex(0);
+  ASSERT_NE(nullptr, actorFromWorld);
+  EXPECT_EQ(actorFromWorld->Name(), actor.Name());
+}
+
+/////////////////////////////////////////////////
+TEST(DOMWorld, AddLight)
+{
+  sdf::World world;
+  EXPECT_EQ(0u, world.LightCount());
+
+  sdf::Light light;
+  light.SetName("light1");
+  EXPECT_TRUE(world.AddLight(light));
+  EXPECT_EQ(1u, world.LightCount());
+  EXPECT_FALSE(world.AddLight(light));
+  EXPECT_EQ(1u, world.LightCount());
+
+  world.ClearLights();
+  EXPECT_EQ(0u, world.LightCount());
+
+  EXPECT_TRUE(world.AddLight(light));
+  EXPECT_EQ(1u, world.LightCount());
+  const sdf::Light *lightFromWorld = world.LightByIndex(0);
+  ASSERT_NE(nullptr, lightFromWorld);
+  EXPECT_EQ(lightFromWorld->Name(), light.Name());
+}
