@@ -1,13 +1,13 @@
 /*********************************************************************
 * Software License Agreement (BSD License)
-* 
+*
 *  Copyright (c) 2008, Willow Garage, Inc.
 *  All rights reserved.
-* 
+*
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
-* 
+*
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
@@ -17,7 +17,7 @@
 *   * Neither the name of the Willow Garage nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
-* 
+*
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
@@ -42,14 +42,14 @@
 #include <stdexcept>
 #include <string>
 #include <algorithm>
-#include <tinyxml.h>
+#include <tinyxml2.h>
 // #include <console_bridge/console.h>
 
 namespace urdf{
 
-bool parsePose(Pose &pose, TiXmlElement* xml);
+bool parsePose(Pose &pose, tinyxml2::XMLElement* xml);
 
-bool parseMaterial(Material &material, TiXmlElement *config, bool only_name_is_ok)
+bool parseMaterial(Material &material, tinyxml2::XMLElement *config, bool only_name_is_ok)
 {
   bool has_rgb = false;
   bool has_filename = false;
@@ -60,11 +60,11 @@ bool parseMaterial(Material &material, TiXmlElement *config, bool only_name_is_o
   {
     return false;
   }
-  
+
   material.name = config->Attribute("name");
 
   // texture
-  TiXmlElement *t = config->FirstChildElement("texture");
+  tinyxml2::XMLElement *t = config->FirstChildElement("texture");
   if (t)
   {
     if (t->Attribute("filename"))
@@ -75,7 +75,7 @@ bool parseMaterial(Material &material, TiXmlElement *config, bool only_name_is_o
   }
 
   // color
-  TiXmlElement *c = config->FirstChildElement("color");
+  tinyxml2::XMLElement *c = config->FirstChildElement("color");
   if (c)
   {
     if (c->Attribute("rgba")) {
@@ -84,7 +84,7 @@ bool parseMaterial(Material &material, TiXmlElement *config, bool only_name_is_o
         material.color.init(c->Attribute("rgba"));
         has_rgb = true;
       }
-      catch (ParseError &e) {  
+      catch (ParseError &e) {
         material.color.clear();
       }
     }
@@ -100,7 +100,7 @@ bool parseMaterial(Material &material, TiXmlElement *config, bool only_name_is_o
 }
 
 
-bool parseSphere(Sphere &s, TiXmlElement *c)
+bool parseSphere(Sphere &s, tinyxml2::XMLElement *c)
 {
   s.clear();
 
@@ -126,14 +126,14 @@ bool parseSphere(Sphere &s, TiXmlElement *c)
     stm << "radius [" << c->Attribute("radius") << "] is out of range: " << e.what();
     return false;
   }
-  
+
   return true;
 }
 
-bool parseBox(Box &b, TiXmlElement *c)
+bool parseBox(Box &b, tinyxml2::XMLElement *c)
 {
   b.clear();
-  
+
   b.type = Geometry::BOX;
   if (!c->Attribute("size"))
   {
@@ -151,7 +151,7 @@ bool parseBox(Box &b, TiXmlElement *c)
   return true;
 }
 
-bool parseCylinder(Cylinder &y, TiXmlElement *c)
+bool parseCylinder(Cylinder &y, tinyxml2::XMLElement *c)
 {
   y.clear();
 
@@ -199,7 +199,7 @@ bool parseCylinder(Cylinder &y, TiXmlElement *c)
 }
 
 
-bool parseMesh(Mesh &m, TiXmlElement *c)
+bool parseMesh(Mesh &m, tinyxml2::XMLElement *c)
 {
   m.clear();
 
@@ -226,18 +226,18 @@ bool parseMesh(Mesh &m, TiXmlElement *c)
   return true;
 }
 
-GeometrySharedPtr parseGeometry(TiXmlElement *g)
+GeometrySharedPtr parseGeometry(tinyxml2::XMLElement *g)
 {
   GeometrySharedPtr geom;
   if (!g) return geom;
 
-  TiXmlElement *shape = g->FirstChildElement();
+  tinyxml2::XMLElement *shape = g->FirstChildElement();
   if (!shape)
   {
     return geom;
   }
 
-  std::string type_name = shape->ValueStr();
+  std::string type_name = shape->Name();
   if (type_name == "sphere")
   {
     Sphere *s = new Sphere();
@@ -264,29 +264,29 @@ GeometrySharedPtr parseGeometry(TiXmlElement *g)
     Mesh *m = new Mesh();
     geom.reset(m);
     if (parseMesh(*m, shape))
-      return geom;    
+      return geom;
   }
   else
   {
     return geom;
   }
-  
+
   return GeometrySharedPtr();
 }
 
-bool parseInertial(Inertial &i, TiXmlElement *config)
+bool parseInertial(Inertial &i, tinyxml2::XMLElement *config)
 {
   i.clear();
 
   // Origin
-  TiXmlElement *o = config->FirstChildElement("origin");
+  tinyxml2::XMLElement *o = config->FirstChildElement("origin");
   if (o)
   {
     if (!parsePose(i.origin, o))
       return false;
   }
 
-  TiXmlElement *mass_xml = config->FirstChildElement("mass");
+  tinyxml2::XMLElement *mass_xml = config->FirstChildElement("mass");
   if (!mass_xml)
   {
     return false;
@@ -315,7 +315,7 @@ bool parseInertial(Inertial &i, TiXmlElement *config)
     return false;
   }
 
-  TiXmlElement *inertia_xml = config->FirstChildElement("inertia");
+  tinyxml2::XMLElement *inertia_xml = config->FirstChildElement("inertia");
   if (!inertia_xml)
   {
     return false;
@@ -362,19 +362,19 @@ bool parseInertial(Inertial &i, TiXmlElement *config)
   return true;
 }
 
-bool parseVisual(Visual &vis, TiXmlElement *config)
+bool parseVisual(Visual &vis, tinyxml2::XMLElement *config)
 {
   vis.clear();
 
   // Origin
-  TiXmlElement *o = config->FirstChildElement("origin");
+  tinyxml2::XMLElement *o = config->FirstChildElement("origin");
   if (o) {
     if (!parsePose(vis.origin, o))
       return false;
   }
 
   // Geometry
-  TiXmlElement *geom = config->FirstChildElement("geometry");
+  tinyxml2::XMLElement *geom = config->FirstChildElement("geometry");
   vis.geometry = parseGeometry(geom);
   if (!vis.geometry)
     return false;
@@ -384,37 +384,37 @@ bool parseVisual(Visual &vis, TiXmlElement *config)
     vis.name = name_char;
 
   // Material
-  TiXmlElement *mat = config->FirstChildElement("material");
+  tinyxml2::XMLElement *mat = config->FirstChildElement("material");
   if (mat) {
     // get material name
     if (!mat->Attribute("name")) {
       return false;
     }
     vis.material_name = mat->Attribute("name");
-    
+
     // try to parse material element in place
     vis.material.reset(new Material());
     if (!parseMaterial(*vis.material, mat, true))
     {
     }
   }
-  
+
   return true;
 }
 
-bool parseCollision(Collision &col, TiXmlElement* config)
-{  
+bool parseCollision(Collision &col, tinyxml2::XMLElement* config)
+{
   col.clear();
 
   // Origin
-  TiXmlElement *o = config->FirstChildElement("origin");
+  tinyxml2::XMLElement *o = config->FirstChildElement("origin");
   if (o) {
     if (!parsePose(col.origin, o))
       return false;
   }
-  
+
   // Geometry
-  TiXmlElement *geom = config->FirstChildElement("geometry");
+  tinyxml2::XMLElement *geom = config->FirstChildElement("geometry");
   col.geometry = parseGeometry(geom);
   if (!col.geometry)
     return false;
@@ -426,9 +426,9 @@ bool parseCollision(Collision &col, TiXmlElement* config)
   return true;
 }
 
-bool parseLink(Link &link, TiXmlElement* config)
+bool parseLink(Link &link, tinyxml2::XMLElement* config)
 {
-  
+
   link.clear();
 
   const char *name_char = config->Attribute("name");
@@ -439,7 +439,7 @@ bool parseLink(Link &link, TiXmlElement* config)
   link.name = std::string(name_char);
 
   // Inertial (optional)
-  TiXmlElement *i = config->FirstChildElement("inertial");
+  tinyxml2::XMLElement *i = config->FirstChildElement("inertial");
   if (i)
   {
     link.inertial.reset(new Inertial());
@@ -450,7 +450,7 @@ bool parseLink(Link &link, TiXmlElement* config)
   }
 
   // Multiple Visuals (optional)
-  for (TiXmlElement* vis_xml = config->FirstChildElement("visual"); vis_xml; vis_xml = vis_xml->NextSiblingElement("visual"))
+  for (tinyxml2::XMLElement* vis_xml = config->FirstChildElement("visual"); vis_xml; vis_xml = vis_xml->NextSiblingElement("visual"))
   {
 
     VisualSharedPtr vis;
@@ -470,14 +470,14 @@ bool parseLink(Link &link, TiXmlElement* config)
   // Assign the first visual to the .visual ptr, if it exists
   if (!link.visual_array.empty())
     link.visual = link.visual_array[0];
-  
+
   // Multiple Collisions (optional)
-  for (TiXmlElement* col_xml = config->FirstChildElement("collision"); col_xml; col_xml = col_xml->NextSiblingElement("collision"))
+  for (tinyxml2::XMLElement* col_xml = config->FirstChildElement("collision"); col_xml; col_xml = col_xml->NextSiblingElement("collision"))
   {
     CollisionSharedPtr col;
     col.reset(new Collision());
     if (parseCollision(*col, col_xml))
-    {      
+    {
       link.collision_array.push_back(col);
     }
     else
@@ -486,8 +486,8 @@ bool parseLink(Link &link, TiXmlElement* config)
       return false;
     }
   }
-  
-  // Collision (optional)  
+
+  // Collision (optional)
   // Assign the first collision to the .collision ptr, if it exists
   if (!link.collision_array.empty())
     link.collision = link.collision_array[0];
@@ -496,67 +496,67 @@ bool parseLink(Link &link, TiXmlElement* config)
 }
 
 /* exports */
-bool exportPose(Pose &pose, TiXmlElement* xml);
+bool exportPose(Pose &pose, tinyxml2::XMLElement* xml);
 
-bool exportMaterial(Material &material, TiXmlElement *xml)
+bool exportMaterial(Material &material, tinyxml2::XMLElement *xml)
 {
-  TiXmlElement *material_xml = new TiXmlElement("material");
-  material_xml->SetAttribute("name", material.name);
+  auto *material_xml = xml->GetDocument()->NewElement("material");
+  material_xml->SetAttribute("name", material.name.c_str());
 
-  TiXmlElement* texture = new TiXmlElement("texture");
+  auto *texture= xml->GetDocument()->NewElement("texture");
   if (!material.texture_filename.empty())
-    texture->SetAttribute("filename", material.texture_filename);
+    texture->SetAttribute("filename", material.texture_filename.c_str());
   material_xml->LinkEndChild(texture);
 
-  TiXmlElement* color = new TiXmlElement("color");
-  color->SetAttribute("rgba", urdf_export_helpers::values2str(material.color));
+  auto *color = xml->GetDocument()->NewElement("color");
+  color->SetAttribute("rgba", urdf_export_helpers::values2str(material.color).c_str());
   material_xml->LinkEndChild(color);
   xml->LinkEndChild(material_xml);
   return true;
 }
 
-bool exportSphere(Sphere &s, TiXmlElement *xml)
+bool exportSphere(Sphere &s, tinyxml2::XMLElement *xml)
 {
   // e.g. add <sphere radius="1"/>
-  TiXmlElement *sphere_xml = new TiXmlElement("sphere");
-  sphere_xml->SetAttribute("radius", urdf_export_helpers::values2str(s.radius));
+  auto *sphere_xml = xml->GetDocument()->NewElement("sphere");
+  sphere_xml->SetAttribute("radius", urdf_export_helpers::values2str(s.radius).c_str());
   xml->LinkEndChild(sphere_xml);
   return true;
 }
 
-bool exportBox(Box &b, TiXmlElement *xml)
+bool exportBox(Box &b, tinyxml2::XMLElement *xml)
 {
   // e.g. add <box size="1 1 1"/>
-  TiXmlElement *box_xml = new TiXmlElement("box");
-  box_xml->SetAttribute("size", urdf_export_helpers::values2str(b.dim));
+  auto *box_xml = xml->GetDocument()->NewElement("box");
+  box_xml->SetAttribute("size", urdf_export_helpers::values2str(b.dim).c_str());
   xml->LinkEndChild(box_xml);
   return true;
 }
 
-bool exportCylinder(Cylinder &y, TiXmlElement *xml)
+bool exportCylinder(Cylinder &y, tinyxml2::XMLElement *xml)
 {
   // e.g. add <cylinder radius="1"/>
-  TiXmlElement *cylinder_xml = new TiXmlElement("cylinder");
-  cylinder_xml->SetAttribute("radius", urdf_export_helpers::values2str(y.radius));
-  cylinder_xml->SetAttribute("length", urdf_export_helpers::values2str(y.length));
+  auto *cylinder_xml = xml->GetDocument()->NewElement("cylinder");
+  cylinder_xml->SetAttribute("radius", urdf_export_helpers::values2str(y.radius).c_str());
+  cylinder_xml->SetAttribute("length", urdf_export_helpers::values2str(y.length).c_str());
   xml->LinkEndChild(cylinder_xml);
   return true;
 }
 
-bool exportMesh(Mesh &m, TiXmlElement *xml)
+bool exportMesh(Mesh &m, tinyxml2::XMLElement *xml)
 {
   // e.g. add <mesh filename="my_file" scale="1 1 1"/>
-  TiXmlElement *mesh_xml = new TiXmlElement("mesh");
+  auto *mesh_xml = xml->GetDocument()->NewElement("mesh");
   if (!m.filename.empty())
-    mesh_xml->SetAttribute("filename", m.filename);
-  mesh_xml->SetAttribute("scale", urdf_export_helpers::values2str(m.scale));
+    mesh_xml->SetAttribute("filename", m.filename.c_str());
+  mesh_xml->SetAttribute("scale", urdf_export_helpers::values2str(m.scale).c_str());
   xml->LinkEndChild(mesh_xml);
   return true;
 }
 
-bool exportGeometry(GeometrySharedPtr &geom, TiXmlElement *xml)
+bool exportGeometry(GeometrySharedPtr &geom, tinyxml2::XMLElement *xml)
 {
-  TiXmlElement *geometry_xml = new TiXmlElement("geometry");
+  auto *geometry_xml= xml->GetDocument()->NewElement("geometry");
   if (urdf::dynamic_pointer_cast<Sphere>(geom))
   {
     exportSphere((*(urdf::dynamic_pointer_cast<Sphere>(geom).get())), geometry_xml);
@@ -585,36 +585,36 @@ bool exportGeometry(GeometrySharedPtr &geom, TiXmlElement *xml)
   return true;
 }
 
-bool exportInertial(Inertial &i, TiXmlElement *xml)
+bool exportInertial(Inertial &i, tinyxml2::XMLElement *xml)
 {
   // adds <inertial>
   //        <mass value="1"/>
   //        <pose xyz="0 0 0" rpy="0 0 0"/>
   //        <inertia ixx="1" ixy="0" />
   //      </inertial>
-  TiXmlElement *inertial_xml = new TiXmlElement("inertial");
+  auto *inertial_xml = xml->GetDocument()->NewElement("inertial");
+  auto *mass_xml = xml->GetDocument()->NewElement("mass");
 
-  TiXmlElement *mass_xml = new TiXmlElement("mass");
-  mass_xml->SetAttribute("value", urdf_export_helpers::values2str(i.mass));
+  mass_xml->SetAttribute("value", urdf_export_helpers::values2str(i.mass).c_str());
   inertial_xml->LinkEndChild(mass_xml);
 
   exportPose(i.origin, inertial_xml);
 
-  TiXmlElement *inertia_xml = new TiXmlElement("inertia");
-  inertia_xml->SetAttribute("ixx", urdf_export_helpers::values2str(i.ixx));
-  inertia_xml->SetAttribute("ixy", urdf_export_helpers::values2str(i.ixy));
-  inertia_xml->SetAttribute("ixz", urdf_export_helpers::values2str(i.ixz));
-  inertia_xml->SetAttribute("iyy", urdf_export_helpers::values2str(i.iyy));
-  inertia_xml->SetAttribute("iyz", urdf_export_helpers::values2str(i.iyz));
-  inertia_xml->SetAttribute("izz", urdf_export_helpers::values2str(i.izz));
+  auto *inertia_xml = xml->GetDocument()->NewElement("inertia");
+  inertia_xml->SetAttribute("ixx", urdf_export_helpers::values2str(i.ixx).c_str());
+  inertia_xml->SetAttribute("ixy", urdf_export_helpers::values2str(i.ixy).c_str());
+  inertia_xml->SetAttribute("ixz", urdf_export_helpers::values2str(i.ixz).c_str());
+  inertia_xml->SetAttribute("iyy", urdf_export_helpers::values2str(i.iyy).c_str());
+  inertia_xml->SetAttribute("iyz", urdf_export_helpers::values2str(i.iyz).c_str());
+  inertia_xml->SetAttribute("izz", urdf_export_helpers::values2str(i.izz).c_str());
   inertial_xml->LinkEndChild(inertia_xml);
 
   xml->LinkEndChild(inertial_xml);
-  
+
   return true;
 }
 
-bool exportVisual(Visual &vis, TiXmlElement *xml)
+bool exportVisual(Visual &vis, tinyxml2::XMLElement *xml)
 {
   // <visual group="default">
   //   <origin rpy="0 0 0" xyz="0 0 0"/>
@@ -623,7 +623,7 @@ bool exportVisual(Visual &vis, TiXmlElement *xml)
   //   </geometry>
   //   <material name="Grey"/>
   // </visual>
-  TiXmlElement * visual_xml = new TiXmlElement("visual");
+  auto *visual_xml = xml->GetDocument()->NewElement("visual");
 
   exportPose(vis.origin, visual_xml);
 
@@ -637,8 +637,8 @@ bool exportVisual(Visual &vis, TiXmlElement *xml)
   return true;
 }
 
-bool exportCollision(Collision &col, TiXmlElement* xml)
-{  
+bool exportCollision(Collision &col, tinyxml2::XMLElement* xml)
+{
   // <collision group="default">
   //   <origin rpy="0 0 0" xyz="0 0 0"/>
   //   <geometry>
@@ -646,7 +646,7 @@ bool exportCollision(Collision &col, TiXmlElement* xml)
   //   </geometry>
   //   <material name="Grey"/>
   // </collision>
-  TiXmlElement * collision_xml = new TiXmlElement("collision");
+  auto *collision_xml = xml->GetDocument()->NewElement("collision");
 
   exportPose(col.origin, collision_xml);
 
@@ -657,10 +657,10 @@ bool exportCollision(Collision &col, TiXmlElement* xml)
   return true;
 }
 
-bool exportLink(Link &link, TiXmlElement* xml)
+bool exportLink(Link &link, tinyxml2::XMLElement* xml)
 {
-  TiXmlElement * link_xml = new TiXmlElement("link");
-  link_xml->SetAttribute("name", link.name);
+  auto *link_xml = xml->GetDocument()->NewElement("link");
+  link_xml->SetAttribute("name", link.name.c_str());
 
   if (link.inertial)
     exportInertial(*link.inertial, link_xml);
