@@ -42,13 +42,15 @@ TEST(Frame, ModelFrame)
   std::string version = SDF_VERSION;
   stream
     << "<sdf version='" << version << "'>"
-    << "<model name='my_model'>"
-    << "  <frame name='mframe'>"
-    << "    <pose relative_to='/world'>1 1 0 0 0 0</pose>"
-    << "  </frame>"
-    << "  <pose relative_to='mframe'>1 0 0 0 0 0</pose>"
-    << "  <link name='link'/>"
-    << "</model>"
+    << "<world name='default'>"
+    << "  <model name='my_model'>"
+    << "    <frame name='mframe'>"
+    << "      <pose relative_to='/world'>1 1 0 0 0 0</pose>"
+    << "    </frame>"
+    << "    <pose relative_to='mframe'>1 0 0 0 0 0</pose>"
+    << "    <link name='link'/>"
+    << "  </model>"
+    << "</world>"
     << "</sdf>";
 
   sdf::SDFPtr sdfParsed(new sdf::SDF());
@@ -57,9 +59,15 @@ TEST(Frame, ModelFrame)
 
   // Verify correct parsing
 
+  // world
+  ASSERT_TRUE(sdfParsed->Root()->HasElement("world"));
+  sdf::ElementPtr worldElem = sdfParsed->Root()->GetElement("world");
+  ASSERT_NE(worldElem, nullptr);
+
   // model
-  EXPECT_TRUE(sdfParsed->Root()->HasElement("model"));
-  sdf::ElementPtr modelElem = sdfParsed->Root()->GetElement("model");
+  ASSERT_TRUE(worldElem->HasElement("model"));
+  sdf::ElementPtr modelElem = worldElem->GetElement("model");
+  ASSERT_TRUE(modelElem);
   EXPECT_TRUE(modelElem->HasAttribute("name"));
   EXPECT_EQ(modelElem->Get<std::string>("name"), "my_model");
 
@@ -420,7 +428,7 @@ TEST(DOMFrame, LoadModelFramesAttachedTo)
   EXPECT_TRUE(root.Load(testFile).empty());
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_attached_to", model->Name());
   EXPECT_EQ(1u, model->LinkCount());
@@ -511,7 +519,7 @@ TEST(DOMFrame, LoadModelFramesInvalidAttachedTo)
   EXPECT_NE(std::string::npos,
     errors[9].Message().find(
       "PoseRelativeToGraph cycle detected, "
-      "already visited vertex [F4]"));
+      "already visited vertex [model_frame_invalid_attached_to::F4]"));
 
   // Try loading sdf::Model object
   sdf::SDFPtr sdfParsed(new sdf::SDF());
@@ -524,50 +532,22 @@ TEST(DOMFrame, LoadModelFramesInvalidAttachedTo)
   auto modelElem = rootElem->GetElement("model");
   ASSERT_NE(nullptr, modelElem);
 
-  // the first 10 errors from loading an sdf::Model match the first 10 errors
-  // from loading an sdf::Root
+  // there are no errors when loading an sdf::Model directly since it doesn't
+  // build graphs
   sdf::Model model;
   auto modelLoadErrors = model.Load(modelElem);
-  ASSERT_EQ(10u, modelLoadErrors.size());
-  for (int i = 0; i < 10; ++i)
-  {
-    EXPECT_EQ(errors[i].Code(), modelLoadErrors[i].Code());
-    EXPECT_EQ(errors[i].Message(), modelLoadErrors[i].Message());
-  }
+  EXPECT_EQ(0u, modelLoadErrors.size()) << modelLoadErrors;
 
   errors = model.ValidateGraphs();
-  EXPECT_EQ(6u, errors.size());
+  ASSERT_EQ(2u, errors.size()) << errors;
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
-      "FrameAttachedToGraph error, Non-LINK vertex with name [F3] is "
-      "disconnected; it should have 1 outgoing edge in MODEL attached_to "
-      "graph.")) << errors[0];
-  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
+      "FrameAttachedToGraph error: scope does not point to a valid graph"));
+  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[1].Message().find(
-      "Graph has __model__ scope but sink vertex named [F3] does not have "
-      "FrameType LINK or MODEL when starting from vertex with name [F3]"))
-      << errors[1];
-  EXPECT_EQ(errors[2].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_CYCLE);
-  EXPECT_NE(std::string::npos,
-    errors[2].Message().find(
-      "cycle detected, already visited vertex [F4]")) << errors[2];
-  EXPECT_EQ(errors[3].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
-  EXPECT_NE(std::string::npos,
-    errors[3].Message().find(
-      "Vertex with name [F3] is disconnected; it should have "
-      "1 incoming edge in MODEL relative_to graph")) << errors[3];
-  EXPECT_EQ(errors[4].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
-  EXPECT_NE(std::string::npos,
-    errors[4].Message().find(
-      "frame with name [F3] is disconnected; its source vertex has name [F3], "
-      "but its source name should be __model__")) << errors[4];
-  EXPECT_EQ(errors[5].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_CYCLE);
-  EXPECT_NE(std::string::npos,
-    errors[5].Message().find(
-      "PoseRelativeToGraph cycle detected, "
-      "already visited vertex [F4]")) << errors[5];
+      "PoseRelativeToGraph error: scope does not point to a valid graph"));
 }
 
 /////////////////////////////////////////////////
@@ -582,7 +562,7 @@ TEST(DOMFrame, LoadModelFramesAttachedToJoint)
   EXPECT_TRUE(root.Load(testFile).empty());
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_attached_to_joint", model->Name());
   EXPECT_EQ(2u, model->LinkCount());
@@ -644,10 +624,11 @@ TEST(DOMFrame, LoadModelFramesAttachedToNestedModel)
 
   // Load the SDF file
   sdf::Root root;
-  EXPECT_TRUE(root.Load(testFile).empty());
+  sdf::Errors errors = root.Load(testFile);
+  EXPECT_TRUE(errors.empty()) << errors;
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_attached_to_nested_model", model->Name());
   EXPECT_EQ(1u, model->LinkCount());
@@ -679,9 +660,9 @@ TEST(DOMFrame, LoadModelFramesAttachedToNestedModel)
 
   std::string body;
   EXPECT_TRUE(model->FrameByName("F1")->ResolveAttachedToBody(body).empty());
-  EXPECT_EQ("nested_model", body);
+  EXPECT_EQ("nested_model::nested_link", body);
   EXPECT_TRUE(model->FrameByName("F2")->ResolveAttachedToBody(body).empty());
-  EXPECT_EQ("nested_model", body);
+  EXPECT_EQ("nested_model::nested_link", body);
 }
 
 /////////////////////////////////////////////////
@@ -748,7 +729,7 @@ TEST(DOMFrame, LoadWorldFramesAttachedTo)
   EXPECT_TRUE(world->FrameByName("F1")->ResolveAttachedToBody(body).empty());
   EXPECT_EQ("world", body);
   EXPECT_TRUE(world->FrameByName("F2")->ResolveAttachedToBody(body).empty());
-  EXPECT_EQ("M1", body);
+  EXPECT_EQ("M1::L", body);
 }
 
 /////////////////////////////////////////////////
@@ -811,48 +792,22 @@ TEST(DOMFrame, LoadWorldFramesInvalidAttachedTo)
   auto worldElem = rootElem->GetElement("world");
   ASSERT_NE(nullptr, worldElem);
 
-  // the first 10 errors from loading an sdf::World match the first 10 errors
-  // from loading an sdf::Root
+  // there are no errors when loading an sdf::World directly since it doesn't
+  // build graphs
   sdf::World world;
   auto worldLoadErrors = world.Load(worldElem);
-  ASSERT_EQ(10u, worldLoadErrors.size());
-  for (int i = 0; i < 10; ++i)
-  {
-    EXPECT_EQ(errors[i].Code(), worldLoadErrors[i].Code());
-    EXPECT_EQ(errors[i].Message(), worldLoadErrors[i].Message());
-  }
+  EXPECT_EQ(0u, worldLoadErrors.size()) << worldLoadErrors;
 
   errors = world.ValidateGraphs();
-  EXPECT_EQ(6u, errors.size());
+  ASSERT_EQ(2u, errors.size()) << errors;
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
-      "FrameAttachedToGraph error, FRAME vertices in WORLD attached_to graph "
-      "should have 1 outgoing edge"));
-  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_GRAPH_ERROR);
+      "FrameAttachedToGraph error: scope does not point to a valid graph"));
+  EXPECT_EQ(errors[1].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
   EXPECT_NE(std::string::npos,
     errors[1].Message().find(
-      "Graph has world scope but sink vertex named [F] does not have "
-      "FrameType WORLD or MODEL when starting from vertex with name [F]"));
-  EXPECT_EQ(errors[2].Code(), sdf::ErrorCode::FRAME_ATTACHED_TO_CYCLE);
-  EXPECT_NE(std::string::npos,
-    errors[2].Message().find(
-      "cycle detected, already visited vertex [self_cycle]"));
-  EXPECT_EQ(errors[3].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
-  EXPECT_NE(std::string::npos,
-    errors[3].Message().find(
-      "MODEL / FRAME vertex with name [F] is disconnected; it should have "
-      "1 incoming edge in WORLD relative_to graph"));
-  EXPECT_EQ(errors[4].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_GRAPH_ERROR);
-  EXPECT_NE(std::string::npos,
-    errors[4].Message().find(
-      "frame with name [F] is disconnected; its source vertex has name [F], "
-      "but its source name should be world"));
-  EXPECT_EQ(errors[5].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_CYCLE);
-  EXPECT_NE(std::string::npos,
-    errors[5].Message().find(
-      "PoseRelativeToGraph cycle detected, "
-      "already visited vertex [self_cycle]"));
+      "PoseRelativeToGraph error: scope does not point to a valid graph"));
 }
 
 /////////////////////////////////////////////////
@@ -869,7 +824,7 @@ TEST(DOMFrame, LoadModelFramesRelativeTo)
   using Pose = ignition::math::Pose3d;
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_relative_to", model->Name());
   EXPECT_EQ(1u, model->LinkCount());
@@ -1030,7 +985,7 @@ TEST(DOMFrame, LoadModelFramesRelativeToJoint)
   using Pose = ignition::math::Pose3d;
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_frame_relative_to_joint", model->Name());
   EXPECT_EQ(2u, model->LinkCount());
@@ -1229,7 +1184,7 @@ TEST(DOMFrame, LoadWorldFramesInvalidRelativeTo)
   for (auto e : errors)
     std::cout << e << std::endl;
   EXPECT_FALSE(errors.empty());
-  EXPECT_EQ(11u, errors.size());
+  EXPECT_EQ(15u, errors.size());
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::POSE_RELATIVE_TO_INVALID);
   EXPECT_NE(std::string::npos,
     errors[0].Message().find(
@@ -1294,7 +1249,86 @@ TEST(DOMFrame, WorldIncludeModel)
     ASSERT_NE(nullptr, model);
     ignition::math::Pose3d modelPose;
     sdf::Errors resolveErrors = model->SemanticPose().Resolve(modelPose);
-    EXPECT_TRUE(resolveErrors.empty());
+    EXPECT_TRUE(resolveErrors.empty()) << resolveErrors;
     EXPECT_EQ(expectedPoses[i], modelPose);
   }
 }
+
+////////////////////////////////////////
+// Test parsing a include element that has a pose element and includes a
+// submodel
+TEST(Frame, IncludeFrameWithSubmodel)
+{
+  const std::string MODEL_PATH =
+    sdf::filesystem::append(PROJECT_SOURCE_PATH, "test", "integration",
+                            "model", "box_with_submodel");
+
+  std::ostringstream stream;
+  std::string version = SDF_VERSION;
+  stream
+    << "<sdf version='" << version << "'>"
+    << "<world name='default'>"
+    << "<model name='top_level_model'>"
+    << "  <include>"
+    << "    <static>false</static>"
+    << "    <pose>5 5 0 0 0 0</pose>"
+    << "    <uri>" + MODEL_PATH +"</uri>"
+    << "  </include>"
+    << "</model>"
+    << "</world>"
+    << "</sdf>";
+
+  sdf::Root root;
+  sdf::Errors errors = root.LoadSdfString(stream.str());
+  EXPECT_TRUE(errors.empty()) << errors[0].Message();
+
+  const sdf::World *world = root.WorldByIndex(0);
+  ASSERT_NE(nullptr, world);
+  ASSERT_EQ(1u, world->ModelCount());
+
+  /* top level model: using include will merge top_level_model and box_with_submodel into:
+   *  <model name='top_level_model'>
+   *   <link name='box_with_submodel::link'>
+   *     <pose frame=''>5 5 0 0 -0 0</pose>  <<-- pose has been translated
+   *   </link>
+   *   <model name='box_with_submodel::submodel_box_with_submodel'>
+   *   ...
+   *   </model>
+   * </model>
+  */
+  const sdf::Model *model = world->ModelByIndex(0);
+  ASSERT_NE(nullptr, model);
+  EXPECT_EQ(model->Name(), "top_level_model");
+
+  const sdf::Link *link0 = model->LinkByName("box_with_submodel::link");
+  ASSERT_NE(nullptr, link0);
+  EXPECT_EQ(link0->Name(), "link");
+
+  ignition::math::Pose3d linkPose;
+  sdf::Errors resolveErrors = model->SemanticPose().Resolve(linkPose,
+      "top_level_model::box_with_submodel::link");
+  EXPECT_TRUE(resolveErrors.empty()) << resolveErrors[0].Message();
+  EXPECT_EQ(linkPose.Inverse(),
+            ignition::math::Pose3d(5, 5, 0, 0, 0, 0));
+  /* submodel: pose from parent is translated to model. links are the same
+   * ...
+   *   <model name='box_with_submodel::submodel_box_with_submodel'>
+   *     <link name='submodel_link'>
+   *       <pose frame=''>0 0 0 0 -0 0</pose>
+   *     </link>
+   *     <pose frame=''>5 5 0 0 -0 0</pose>
+   *   </model>
+   */
+  const sdf::Model *submodel = model->ModelByName(
+      "box_with_submodel::submodel_of_box_with_submodel");
+  ASSERT_NE(nullptr, submodel);
+  EXPECT_EQ(submodel->Name(),
+            "submodel_of_box_with_submodel");
+
+  ignition::math::Pose3d submodelPose;
+  resolveErrors = model->SemanticPose().Resolve(submodelPose,
+      "top_level_model::box_with_submodel");
+  EXPECT_TRUE(resolveErrors.empty()) << resolveErrors[0].Message();
+  EXPECT_EQ(submodelPose.Inverse(),
+            ignition::math::Pose3d(5, 5, 0, 0, 0, 0));
+}
