@@ -15,6 +15,7 @@
  *
  */
 
+#include <limits>
 #include <string>
 #include <vector>
 #include <gtest/gtest.h>
@@ -41,7 +42,7 @@ TEST(DOMJointAxis, Complete)
   EXPECT_TRUE(errors.empty());
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
 
   // The model should have nine joints.
@@ -125,7 +126,7 @@ TEST(DOMJointAxis, XyzExpressedIn)
   using Vector3 = ignition::math::Vector3d;
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_joint_axis_expressed_in", model->Name());
   EXPECT_EQ(4u, model->LinkCount());
@@ -217,3 +218,120 @@ TEST(DOMJointAxis, XyzExpressedIn)
   EXPECT_EQ(0u, model->FrameCount());
   EXPECT_EQ(nullptr, model->FrameByIndex(0));
 }
+
+//////////////////////////////////////////////////
+TEST(DOMJointAxis, InfiniteLimits)
+{
+  const std::string testFile =
+    sdf::testing::TestFile("sdf", "joint_axis_infinite_limits.sdf");
+
+  // Load the SDF file
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile);
+
+  EXPECT_TRUE(errors.empty());
+  for (auto e : errors)
+    std::cout << e << std::endl;
+
+  // Get the first model
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(nullptr, model);
+  EXPECT_EQ("joint_axis_infinite_limits", model->Name());
+
+  const double kInf = std::numeric_limits<double>::infinity();
+  {
+    auto joint = model->JointByName("default_joint_limits");
+    ASSERT_NE(nullptr, joint);
+    auto axis = joint->Axis(0);
+    ASSERT_NE(nullptr, axis);
+    EXPECT_DOUBLE_EQ(-1e16, axis->Lower());
+    EXPECT_DOUBLE_EQ(1e16, axis->Upper());
+    EXPECT_DOUBLE_EQ(kInf, axis->Effort());
+    EXPECT_DOUBLE_EQ(kInf, axis->MaxVelocity());
+  }
+
+  {
+    auto joint = model->JointByName("finite_joint_limits");
+    ASSERT_NE(nullptr, joint);
+    auto axis = joint->Axis(0);
+    ASSERT_NE(nullptr, axis);
+    EXPECT_DOUBLE_EQ(-1.5, axis->Lower());
+    EXPECT_DOUBLE_EQ(1.5, axis->Upper());
+    EXPECT_DOUBLE_EQ(2.5, axis->MaxVelocity());
+    EXPECT_DOUBLE_EQ(5.5, axis->Effort());
+  }
+
+  {
+    auto joint = model->JointByName("infinite_joint_limits_inf");
+    ASSERT_NE(nullptr, joint);
+    auto axis = joint->Axis(0);
+    ASSERT_NE(nullptr, axis);
+    EXPECT_DOUBLE_EQ(-kInf, axis->Lower());
+    EXPECT_DOUBLE_EQ(kInf, axis->Upper());
+    EXPECT_DOUBLE_EQ(kInf, axis->Effort());
+    EXPECT_DOUBLE_EQ(kInf, axis->MaxVelocity());
+  }
+
+  {
+    auto joint = model->JointByName("infinite_joint_limits_neg");
+    ASSERT_NE(nullptr, joint);
+    auto axis = joint->Axis(0);
+    ASSERT_NE(nullptr, axis);
+    EXPECT_DOUBLE_EQ(-kInf, axis->Lower());
+    EXPECT_DOUBLE_EQ(kInf, axis->Upper());
+    EXPECT_DOUBLE_EQ(kInf, axis->Effort());
+    EXPECT_DOUBLE_EQ(kInf, axis->MaxVelocity());
+  }
+}
+
+//////////////////////////////////////////////////
+TEST(DOMJointAxis, XyzNormalization)
+{
+  const std::string testFile =
+    sdf::testing::TestFile("sdf", "joint_axis_xyz_normalization.sdf");
+
+  // Load the SDF file
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile);
+
+  ASSERT_EQ(1u, errors.size());
+  EXPECT_TRUE(
+      errors[0].Message().find("The norm of the xyz vector cannot be zero") !=
+      std::string::npos);
+
+  using ignition::math::Vector3d;
+
+  // Get the first model
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(nullptr, model);
+
+  {
+    auto joint1 = model->JointByName("joint1");
+    ASSERT_FALSE(nullptr == joint1);
+    ASSERT_FALSE(nullptr == joint1->Axis(0));
+    EXPECT_EQ(Vector3d::UnitZ, joint1->Axis(0)->Xyz());
+  }
+
+  {
+    auto joint2 = model->JointByName("joint2");
+    ASSERT_FALSE(nullptr == joint2);
+    ASSERT_FALSE(nullptr == joint2->Axis(0));
+    EXPECT_EQ(Vector3d::UnitX, joint2->Axis(0)->Xyz());
+  }
+
+  {
+    auto joint3 = model->JointByName("joint3");
+    ASSERT_FALSE(nullptr == joint3);
+    ASSERT_FALSE(nullptr == joint3->Axis(0));
+    EXPECT_EQ(-Vector3d::UnitX, joint3->Axis(0)->Xyz());
+    ASSERT_FALSE(nullptr == joint3->Axis(1));
+    EXPECT_EQ(Vector3d::UnitY, joint3->Axis(1)->Xyz());
+  }
+
+  {
+    auto joint4 = model->JointByName("joint4");
+    ASSERT_FALSE(nullptr == joint4);
+    ASSERT_FALSE(nullptr == joint4->Axis(0));
+    EXPECT_EQ(Vector3d::UnitZ, joint4->Axis(0)->Xyz());
+  }
+}
