@@ -15,6 +15,7 @@
  *
  */
 
+#include <sstream>
 #include <string>
 #include <gtest/gtest.h>
 
@@ -121,7 +122,7 @@ TEST(DOMLink, InertialDoublePendulum)
   sdf::Root root;
   EXPECT_TRUE(root.Load(testFile).empty());
 
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
 
   const sdf::Link *baseLink = model->LinkByIndex(0);
@@ -175,7 +176,7 @@ TEST(DOMLink, InertialComplete)
   sdf::Root root;
   EXPECT_TRUE(root.Load(testFile).empty());
 
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
 
   const sdf::Link *link = model->LinkByIndex(0);
@@ -212,7 +213,7 @@ TEST(DOMLink, InertialInvalid)
   EXPECT_EQ(errors[0].Code(), sdf::ErrorCode::LINK_INERTIA_INVALID);
   EXPECT_EQ(errors[0].Message(), "A link named link has invalid inertia.");
 
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
 
   ASSERT_EQ(1u, model->LinkCount());
@@ -234,7 +235,7 @@ TEST(DOMLink, Sensors)
   EXPECT_TRUE(errors.empty());
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model", model->Name());
 
@@ -242,7 +243,7 @@ TEST(DOMLink, Sensors)
   const sdf::Link *link = model->LinkByIndex(0);
   ASSERT_NE(nullptr, link);
   EXPECT_EQ("link", link->Name());
-  EXPECT_EQ(22u, link->SensorCount());
+  EXPECT_EQ(26u, link->SensorCount());
 
   // Get the altimeter sensor
   const sdf::Sensor *altimeterSensor = link->SensorByIndex(0);
@@ -317,6 +318,19 @@ TEST(DOMLink, Sensors)
   EXPECT_EQ(ignition::math::Pose3d(4, 5, 3, 0, 0, 0), pose);
   EXPECT_TRUE(contactSensor->EnableMetrics());
 
+  // Get the custom sensor
+  const sdf::Sensor *customSensor = link->SensorByName("custom_sensor");
+  ASSERT_NE(nullptr, customSensor);
+  EXPECT_EQ("custom_sensor", customSensor->Name());
+  EXPECT_EQ(sdf::SensorType::CUSTOM, customSensor->Type());
+  ASSERT_NE(nullptr, customSensor->Element());
+  auto myNamespaceElem = customSensor->Element()->GetElement(
+      "my_namespace:custom_sensor");
+  ASSERT_NE(nullptr, myNamespaceElem);
+  auto someParamElem = myNamespaceElem->GetElement("some_param");
+  ASSERT_NE(nullptr, someParamElem);
+  EXPECT_EQ(123, someParamElem->Get<int>());
+
   // Get the depth sensor
   const sdf::Sensor *depthSensor = link->SensorByName("depth_sensor");
   ASSERT_NE(nullptr, depthSensor);
@@ -351,6 +365,35 @@ TEST(DOMLink, Sensors)
   ASSERT_NE(nullptr, thermalCamSensor);
   EXPECT_EQ("my_thermal_camera", thermalCamSensor->Name());
 
+  // Get the segmentation sensor
+  const sdf::Sensor *segmentationSensor =
+    link->SensorByName("segmentation_sensor");
+  ASSERT_NE(nullptr, segmentationSensor);
+  EXPECT_EQ("segmentation_sensor", segmentationSensor->Name());
+  EXPECT_EQ(sdf::SensorType::SEGMENTATION_CAMERA, segmentationSensor->Type());
+  EXPECT_EQ(ignition::math::Pose3d(37, 38, 39, 0, 0, 0),
+            segmentationSensor->RawPose());
+  const sdf::Camera *segmentationCameraSensor =
+    segmentationSensor->CameraSensor();
+  ASSERT_NE(nullptr, segmentationCameraSensor);
+  EXPECT_EQ("my_segmentation_camera", segmentationCameraSensor->Name());
+  EXPECT_TRUE(segmentationCameraSensor->HasSegmentationType());
+  EXPECT_EQ("semantic", segmentationCameraSensor->SegmentationType());
+
+  // Get the boundingbox sensor
+  const sdf::Sensor *boundingboxSensor =
+    link->SensorByName("boundingbox_sensor");
+  ASSERT_NE(nullptr, boundingboxSensor);
+  EXPECT_EQ("boundingbox_sensor", boundingboxSensor->Name());
+  EXPECT_EQ(sdf::SensorType::BOUNDINGBOX_CAMERA, boundingboxSensor->Type());
+  EXPECT_EQ(ignition::math::Pose3d(37, 38, 39, 0, 0, 0),
+            boundingboxSensor->RawPose());
+  const sdf::Camera *boundingboxCamSensor = boundingboxSensor->CameraSensor();
+  ASSERT_NE(nullptr, boundingboxCamSensor);
+  EXPECT_EQ("my_boundingbox_camera", boundingboxCamSensor->Name());
+  EXPECT_TRUE(boundingboxCamSensor->HasBoundingBoxType());
+  EXPECT_EQ("2d", boundingboxCamSensor->BoundingBoxType());
+
   // Get the force_torque sensor
   const sdf::Sensor *forceTorqueSensor =
     link->SensorByName("force_torque_sensor");
@@ -447,7 +490,6 @@ TEST(DOMLink, Sensors)
   EXPECT_DOUBLE_EQ(5.0,
       imuSensorObj->AngularVelocityYNoise().DynamicBiasCorrelationTime());
 
-
   EXPECT_DOUBLE_EQ(5.0, imuSensorObj->AngularVelocityZNoise().Mean());
   EXPECT_DOUBLE_EQ(5.1, imuSensorObj->AngularVelocityZNoise().StdDev());
   EXPECT_DOUBLE_EQ(6.2,
@@ -455,13 +497,14 @@ TEST(DOMLink, Sensors)
   EXPECT_DOUBLE_EQ(6.0,
       imuSensorObj->AngularVelocityZNoise().DynamicBiasCorrelationTime());
 
-
   EXPECT_EQ("ENU", imuSensorObj->Localization());
   EXPECT_EQ("linka", imuSensorObj->CustomRpyParentFrame());
   EXPECT_EQ(ignition::math::Vector3d::UnitY, imuSensorObj->CustomRpy());
   EXPECT_EQ("linkb", imuSensorObj->GravityDirXParentFrame());
   EXPECT_EQ(ignition::math::Vector3d::UnitZ, imuSensorObj->GravityDirX());
 
+  EXPECT_FALSE(imuSensorObj->OrientationEnabled());
+
   // Get the logical camera sensor
   const sdf::Sensor *logicalCameraSensor =
     link->SensorByName("logical_camera_sensor");
@@ -604,6 +647,37 @@ TEST(DOMLink, Sensors)
   EXPECT_DOUBLE_EQ(3.4, airSensor->PressureNoise().Mean());
   EXPECT_DOUBLE_EQ(5.6, airSensor->PressureNoise().StdDev());
   EXPECT_DOUBLE_EQ(123.4, airSensor->ReferenceAltitude());
+
+  // Get the wide angle camera sensor
+  EXPECT_TRUE(link->SensorNameExists("wide_angle_camera_sensor"));
+  const sdf::Sensor *wideAngleCameraSensor =
+      link->SensorByName("wide_angle_camera_sensor");
+  ASSERT_NE(nullptr, wideAngleCameraSensor);
+  EXPECT_EQ("wide_angle_camera_sensor", wideAngleCameraSensor->Name());
+  EXPECT_EQ(sdf::SensorType::WIDE_ANGLE_CAMERA, wideAngleCameraSensor->Type());
+  EXPECT_EQ(ignition::math::Pose3d(20, 30, 40, 0, 0, 0),
+      wideAngleCameraSensor->RawPose());
+  EXPECT_FALSE(wideAngleCameraSensor->EnableMetrics());
+  const sdf::Camera *wideAngleCam = wideAngleCameraSensor->CameraSensor();
+  ASSERT_NE(nullptr, wideAngleCam);
+  EXPECT_EQ("wide_angle_cam", wideAngleCam->Name());
+  EXPECT_EQ(ignition::math::Pose3d(0.2, 0.3, 0.4, 0, 0, 0),
+            wideAngleCam->RawPose());
+  EXPECT_DOUBLE_EQ(3.14, wideAngleCam->HorizontalFov().Radian());
+  EXPECT_EQ(320u, wideAngleCam->ImageWidth());
+  EXPECT_EQ(240u, wideAngleCam->ImageHeight());
+  EXPECT_EQ(sdf::PixelFormatType::RGB_INT8, wideAngleCam->PixelFormat());
+  EXPECT_DOUBLE_EQ(0.1, wideAngleCam->NearClip());
+  EXPECT_DOUBLE_EQ(100, wideAngleCam->FarClip());
+  EXPECT_EQ("custom", wideAngleCam->LensType());
+  EXPECT_TRUE(wideAngleCam->LensScaleToHfov());
+  EXPECT_DOUBLE_EQ(1.05, wideAngleCam->LensC1());
+  EXPECT_DOUBLE_EQ(4.0, wideAngleCam->LensC2());
+  EXPECT_DOUBLE_EQ(0.0, wideAngleCam->LensC3());
+  EXPECT_DOUBLE_EQ(1.0, wideAngleCam->LensFocalLength());
+  EXPECT_EQ("tan", wideAngleCam->LensFunction());
+  EXPECT_DOUBLE_EQ(3.1415, wideAngleCam->LensCutoffAngle().Radian());
+  EXPECT_EQ(512, wideAngleCam->LensEnvironmentTextureSize());
 }
 
 /////////////////////////////////////////////////
@@ -619,7 +693,7 @@ TEST(DOMLink, LoadLinkPoseRelativeTo)
   using Pose = ignition::math::Pose3d;
 
   // Get the first model
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   ASSERT_NE(nullptr, model);
   EXPECT_EQ("model_link_relative_to", model->Name());
   EXPECT_EQ(3u, model->LinkCount());
@@ -709,3 +783,65 @@ TEST(DOMLink, LoadInvalidLinkPoseRelativeTo)
   // errors[3]
   // errors[4]
 }
+
+/////////////////////////////////////////////////
+TEST(DOMLink, ValidInertialPoseRelTo)
+{
+  std::ostringstream stream;
+  stream << "<?xml version=\"1.0\"?>"
+         << "<sdf version='1.8'>"
+         << "  <model name='A'>"
+         << "    <link name='B'>"
+         << "      <inertial>"
+         << "        <pose relative_to=''>0.1 1 0.2 0 0 -0.52</pose>"
+         << "      </inertial>"
+         << "    </link>"
+         << "  </model>"
+         << "</sdf>";
+
+  sdf::Root root;
+  sdf::Errors errors = root.LoadSdfString(stream.str());
+  EXPECT_TRUE(errors.empty());
+
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(model, nullptr);
+
+  const sdf::Link *link = model->LinkByName("B");
+  ASSERT_NE(link, nullptr);
+
+  EXPECT_EQ(link->Inertial().Pose(),
+      ignition::math::Pose3d(0.1, 1, 0.2, 0, 0, -0.52));
+}
+
+/////////////////////////////////////////////////
+TEST(DOMLink, InvalidInertialPoseRelTo)
+{
+  std::ostringstream stream;
+  stream << "<?xml version=\"1.0\"?>"
+         << "<sdf version='1.8'>"
+         << "  <model name='A'>"
+         << "    <frame name='C'>"
+         << "      <pose>0 0 1 0 0 0</pose>"
+         << "    </frame>"
+         << "    <link name='B'>"
+         << "      <inertial>"
+         << "        <pose relative_to='C'>0.1 1 0.2 0 0 -0.52</pose>"
+         << "      </inertial>"
+         << "    </link>"
+         << "  </model>"
+         << "</sdf>";
+
+  sdf::Root root;
+  sdf::Errors errors = root.LoadSdfString(stream.str());
+
+  // TODO(anyone) add test for warnings once it's implemented
+
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(model, nullptr);
+
+  const sdf::Link *link = model->LinkByName("B");
+  ASSERT_NE(link, nullptr);
+
+  EXPECT_EQ(link->Inertial().Pose(),
+      ignition::math::Pose3d(0.1, 1, 0.2, 0, 0, -0.52));
+}
