@@ -21,7 +21,6 @@
 #include <gtest/gtest.h>
 
 #include "sdf/sdf.hh"
-#include "sdf/parser_urdf.hh"
 
 #include "test_config.h"
 
@@ -30,8 +29,8 @@
 #ifndef _MSC_VER
 TEST(CheckFixForLocal, MakeTestToFail)
 {
-  const std::string sdfTestFile = sdf::filesystem::append(
-      PROJECT_SOURCE_PATH, "test", "integration", "numeric.sdf");
+  const std::string sdfTestFile =
+      sdf::testing::TestFile("integration", "numeric.sdf");
 
   // Check if any of the latin locales is avilable
   FILE *fp = popen("locale -a | grep '^es\\|^pt_\\|^it_' | head -n 1", "r");
