@@ -58,8 +58,9 @@ TEST(DOMRoot, Load)
 
   sdf::Root root;
   EXPECT_EQ(0u, root.WorldCount());
-  EXPECT_TRUE(root.Load(testFile).empty());
-  EXPECT_EQ("1.7", root.Version());
+  sdf::Errors errors = root.Load(testFile);
+  EXPECT_TRUE(errors.empty()) << errors;
+  EXPECT_EQ(SDF_PROTOCOL_VERSION, root.Version());
   EXPECT_EQ(1u, root.WorldCount());
   EXPECT_TRUE(root.WorldByIndex(0) != nullptr);
   EXPECT_TRUE(root.WorldByIndex(1) == nullptr);
@@ -79,17 +80,17 @@ TEST(DOMRoot, LoadMultipleModels)
     sdf::testing::TestFile("sdf", "root_multiple_models.sdf");
 
   sdf::Root root;
-  EXPECT_TRUE(root.Load(testFile).empty());
-  EXPECT_EQ(3u, root.ModelCount());
+  sdf::Errors errors = root.Load(testFile);
 
-  EXPECT_EQ("robot1", root.ModelByIndex(0)->Name());
-  EXPECT_EQ("robot2", root.ModelByIndex(1)->Name());
-  EXPECT_EQ("last_robot", root.ModelByIndex(2)->Name());
+  // An error should be emitted since there are multiple root models.
+  // errors. For now, only the first model is loaded.
+  ASSERT_FALSE(errors.empty());
+  EXPECT_EQ(sdf::ErrorCode::ELEMENT_INCORRECT_TYPE, errors[0].Code());
+  EXPECT_EQ("Root object can only contain one model. Using the first one found",
+            errors[0].Message());
 
-  EXPECT_FALSE(root.ModelNameExists("robot"));
-  EXPECT_TRUE(root.ModelNameExists("robot1"));
-  EXPECT_TRUE(root.ModelNameExists("robot2"));
-  EXPECT_TRUE(root.ModelNameExists("last_robot"));
+  ASSERT_NE(nullptr, root.Model());
+  EXPECT_EQ("robot1", root.Model()->Name());
 }
 
 /////////////////////////////////////////////////
@@ -100,7 +101,10 @@ TEST(DOMRoot, LoadDuplicateModels)
 
   sdf::Root root;
   sdf::Errors errors = root.Load(testFile);
-  EXPECT_FALSE(errors.empty());
-  EXPECT_EQ(1u, root.ModelCount());
-  EXPECT_EQ("robot1", root.ModelByIndex(0)->Name());
+  ASSERT_FALSE(errors.empty());
+  EXPECT_EQ(sdf::ErrorCode::DUPLICATE_NAME, errors[0].Code());
+  EXPECT_EQ("model with name[robot1] already exists.", errors[0].Message());
+
+  EXPECT_NE(nullptr, root.Model());
+  EXPECT_EQ("robot1", root.Model()->Name());
 }
